@@ -1,7 +1,8 @@
 <?php
 /**
  * Mahara: Electronic portfolio, weblog, resume builder and social networking
- * Copyright (C) 2006-2008 Catalyst IT Ltd (http://www.catalyst.net.nz)
+ * Copyright (C) 2006-2009 Catalyst IT Ltd and others; see:
+ *                         http://wiki.mahara.org/Contributors
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +21,7 @@
  * @subpackage core
  * @author     Catalyst IT Ltd
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL
- * @copyright  (C) 2006-2008 Catalyst IT Ltd http://catalyst.net.nz
+ * @copyright  (C) 2006-2009 Catalyst IT Ltd http://catalyst.net.nz
  *
  */
 
@@ -34,30 +35,35 @@ require_once('pieforms/pieform.php');
 $institutions = get_records_assoc('institution', '', '', '', 'name,displayname,registerallowed');
 
 // For all institutions the user is already a member of, create a
-// button to leave the institution
+// button to leave the institution, unless the institution does not
+// allow registration.
 $member = $USER->get('institutions');
 if (!empty($member)) {
     $elements = array();
     foreach ($member as $i) {
-        $elements[] = array(
-            'type' => 'submit',
-            'name' => '_leave_' . $i->institution,
-            'confirm' => get_string('reallyleaveinstitution'),
-            'title' => get_string('youareamemberof', 'mahara', 
-                                  $institutions[$i->institution]->displayname),
-            'value' => get_string('leaveinstitution')
-        );
-        $elements[] = array(
-            'type' => 'html',
-            'name' => '_leavehtml_' . $i->institution,
-            'value' => '<br>',
-        );
+        if ($institutions[$i->institution]->registerallowed) {
+            $elements[] = array(
+                'type' => 'submit',
+                'class' => 'btn-del',
+                'name' => '_leave_' . $i->institution,
+                'confirm' => get_string('reallyleaveinstitution'),
+                'title' => get_string('youareamemberof', 'mahara', $institutions[$i->institution]->displayname),
+                'value' => get_string('leaveinstitution')
+            );
+        }
+        else {
+            $elements[] = array(
+                'type' => 'html',
+                'name' => '_noleave_' . $i->institution,
+                'title' => get_string('youareamemberof', 'mahara', $institutions[$i->institution]->displayname),
+                'value' => '',
+            );
+        }
         unset($institutions[$i->institution]);
     }
     $memberform = pieform(array(
         'name'        => 'leaveinstitution',
         'method'      => 'post',
-        'renderer'    => 'oneline',
         'plugintype'  => 'core',
         'pluginname'  => 'account',
         'elements'    => $elements
@@ -101,7 +107,6 @@ if (!empty($requested)) {
     $requestedform = pieform(array(
         'name'        => 'cancelrequest',
         'method'      => 'post',
-        'renderer'    => 'oneline',
         'plugintype'  => 'core',
         'pluginname'  => 'account',
         'elements'    => $elements
@@ -150,7 +155,6 @@ if (!empty($invited)) {
     $invitedform = pieform(array(
         'name'        => 'confirminvite',
         'method'      => 'post',
-        'renderer'    => 'oneline',
         'plugintype'  => 'core',
         'pluginname'  => 'account',
         'elements'    => $elements
@@ -196,15 +200,16 @@ if (!empty($institutions) &&
             $options[$i->name] = $i->displayname;
         }
     }
+    natcasesort($options);
     $joinform = pieform(array(
         'name'        => 'requestmembership',
         'method'      => 'post',
-        'renderer'    => 'oneline',
         'plugintype'  => 'core',
         'pluginname'  => 'account',
         'elements'    => array(
             'institution' => array(
                 'type' => 'select',
+                'title' => get_string('institution'),
                 'collapseifoneoption' => false,
                 'options' => $options,
                 'defaultvalue' => key($options),
@@ -242,6 +247,7 @@ $smarty->assign('memberform', $memberform);
 $smarty->assign('requestedform', $requestedform);
 $smarty->assign('invitedform', $invitedform);
 $smarty->assign('joinform', $joinform);
+$smarty->assign('PAGEHEADING', hsc(TITLE));
 $smarty->display('account/institutions.tpl');
 
 ?>
