@@ -1,7 +1,8 @@
 <?php
 /**
  * Mahara: Electronic portfolio, weblog, resume builder and social networking
- * Copyright (C) 2006-2008 Catalyst IT Ltd (http://www.catalyst.net.nz)
+ * Copyright (C) 2006-2009 Catalyst IT Ltd and others; see:
+ *                         http://wiki.mahara.org/Contributors
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,13 +21,16 @@
  * @subpackage admin
  * @author     Catalyst IT Ltd
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL
- * @copyright  (C) 2006-2008 Catalyst IT Ltd http://catalyst.net.nz
+ * @copyright  (C) 2006-2009 Catalyst IT Ltd http://catalyst.net.nz
  *
  */
 
 define('INTERNAL', 1);
 define('ADMIN', 1);
 define('MENUITEM', 'configsite/networking');
+define('SECTION_PLUGINTYPE', 'core');
+define('SECTION_PLUGINNAME', 'admin');
+define('SECTION_PAGE', 'networking');
 
 require(dirname(dirname(dirname(__FILE__))) . '/init.php');
 require_once(get_config('docroot') . 'api/xmlrpc/lib.php');
@@ -69,7 +73,17 @@ $networkingform = pieform(
                 'type'         => 'html',
                 'title'        => get_string('publickey','admin'),
                 'description'  => get_string('publickeydescription2', 'admin', 365),
-                'value'        => '<pre style="font-size: 0.7em">'.$openssl->certificate.'</pre>'
+                'value'        => '<pre style="font-size: 0.7em; white-space: pre;">'.$openssl->certificate.'</pre>'
+            ),
+            'sha1fingerprint' => array(
+                'type'         => 'html',
+                'title'        => 'SHA1 Fingerprint',
+                'value'        => $openssl->sha1_fingerprint
+            ),
+            'md5fingerprint' => array(
+                'type'         => 'html',
+                'title'        => 'MD5 Fingerprint',
+                'value'        => $openssl->md5_fingerprint
             ),
             'expires' => array(
                 'type'         => 'html',
@@ -90,6 +104,34 @@ $networkingform = pieform(
                 'defaultvalue' => get_config('promiscuousmode'),
                 'options'      => $yesno,
             ),
+            'proxyfieldset'    => array(
+                'type'         => 'fieldset',
+                'legend'       => get_string('proxysettings', 'admin'),
+                'elements'     => array(
+                    'proxyaddress' => array(
+                        'type'          => 'text',
+                        'title'         => get_string('proxyaddress', 'admin'),
+                        'description'   => get_string('proxyaddressdescription', 'admin'),
+                        'defaultvalue'  => get_config('proxyaddress'),
+                    ),
+                    'proxyauthmodel' => array(
+                        'type'          => 'select',
+                        'title'         => get_string('proxyauthmodel', 'admin'),
+                        'description'   => get_string('proxyauthmodeldescription', 'admin'),
+                        'defaultvalue'  => get_config('proxyauthmodel'),
+                        'options'       => array(
+                                            '' => 'None',
+                                            'basic' => 'Basic (NCSA)',
+                                        ),
+                    ),
+                    'proxyauthcredentials' => array(
+                        'type'          => 'text',
+                        'title'         => get_string('proxyauthcredentials', 'admin'),
+                        'description'   => get_string('proxyauthcredentialsdescription', 'admin'),
+                        'defaultvalue'  => get_config('proxyauthcredentials'),
+                    ),
+                ),
+            ),
             'submit' => array(
                 'type'  => 'submit',
                 'value' => get_string('savechanges','admin')
@@ -136,6 +178,33 @@ function networkingform_submit(Pieform $form, $values) {
         }
     }
 
+    if(get_config('proxyaddress') != $values['proxyaddress']) {
+        if(!set_config('proxyaddress', $values['proxyaddress'])) {
+            networkingform_fail($form);
+        }
+        else {
+            $reply .= get_string('proxyaddressset', 'admin');
+        }
+    }
+
+    if(get_config('proxyauthmodel') != $values['proxyauthmodel']) {
+        if(!set_config('proxyauthmodel', $values['proxyauthmodel'])) {
+            networkingform_fail($form);
+        }
+        else {
+            $reply .= get_string('proxyauthmodelset', 'admin');
+        }
+    }
+
+    if(get_config('proxyauthcredentials') != $values['proxyauthcredentials']) {
+        if(!set_config('proxyauthcredentials', $values['proxyauthcredentials'])) {
+            networkingform_fail($form);
+        }
+        else {
+            $reply .= get_string('proxyauthcredntialsset', 'admin');
+        }
+    }
+
     $form->reply(PIEFORM_OK, array(
         'message' => ($reply == '') ? get_string('networkingunchanged','admin') : $reply,
         'goto'    => '/admin/site/networking.php',
@@ -144,6 +213,7 @@ function networkingform_submit(Pieform $form, $values) {
 
 $smarty = smarty();
 $smarty->assign('networkingform', $networkingform);
+$smarty->assign('PAGEHEADING', hsc(TITLE));
 $smarty->display('admin/site/networking.tpl');
 
 ?>
