@@ -1,7 +1,8 @@
 <?php
 /**
  * Mahara: Electronic portfolio, weblog, resume builder and social networking
- * Copyright (C) 2006-2008 Catalyst IT Ltd (http://www.catalyst.net.nz)
+ * Copyright (C) 2006-2009 Catalyst IT Ltd and others; see:
+ *                         http://wiki.mahara.org/Contributors
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +21,7 @@
  * @subpackage admin
  * @author     Catalyst IT Ltd
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL
- * @copyright  (C) 2006-2008 Catalyst IT Ltd http://catalyst.net.nz
+ * @copyright  (C) 2006-2009 Catalyst IT Ltd http://catalyst.net.nz
  *
  */
 define('INTERNAL', 1);
@@ -88,13 +89,14 @@ if ($institution && $plugin) {
         'goto'  => 'addauthority.php?c=1'
     );
 
+    $form = pieform($form);
     $smarty = smarty();
     if ($add) {
         $smarty->assign('PAGETITLE', get_string('addauthority', 'auth'));
     } else {
         $smarty->assign('PAGETITLE', get_string('editauthority', 'auth'));
     }
-    $smarty->assign('auth_imap_form', pieform($form));
+    $smarty->assign('auth_imap_form', $form);
 }
 
 function auth_config_validate(Pieform $form, $values) {
@@ -154,5 +156,53 @@ function execute_javascript_and_close($js='') {
     exit;
 }
 
+$js = <<<EOF
+function authloginmsgVisibility() {
+    // If Parent authority is 'None'
+    if ($('auth_config_parent').value != 0) {
+      addElementClass('auth_config_authloginmsg_container', 'hidden');
+      addElementClass(nextSiblingTR($('auth_config_authloginmsg_container')), 'hidden');
+    }
+    else {
+      removeElementClass('auth_config_authloginmsg_container', 'hidden');
+      removeElementClass(nextSiblingTR($('auth_config_authloginmsg_container')), 'hidden');
+    }
+}
+function nextSiblingTR(node) {
+    while (node.nextSibling.tagName != 'TR') {
+        node = node.nextSibling;
+    }
+    return node.nextSibling;
+}
+var ssoAllOptions = {
+    'updateuserinfoonlogin': 'theyssoin',
+    'weautocreateusers': 'theyssoin',
+    'theyautocreateusers': 'wessoout',
+    'weimportcontent': 'theyssoin'
+};
+function updateSsoOptions() {
+    var current = $('auth_config_ssodirection').value;
+    for (var opt in ssoAllOptions) {
+        if (ssoAllOptions[opt] == current) {
+            removeElementClass('auth_config_' + opt + '_container', 'hidden');
+        }
+        else {
+            addElementClass('auth_config_' + opt + '_container', 'hidden');
+        }
+    }
+}
+addLoadEvent(
+    function() {
+        connect('auth_config_parent', 'onchange', authloginmsgVisibility);
+        authloginmsgVisibility();
+        if ($('auth_config_ssodirection')) {
+            connect('auth_config_ssodirection', 'onchange', updateSsoOptions);
+        }
+        updateSsoOptions();
+    }
+);
+EOF;
+
+$smarty->assign('INLINEJAVASCRIPT', $js);
 $smarty->display('admin/users/addauthority.tpl');
 ?>
