@@ -1,7 +1,8 @@
 <?php
 /**
  * Mahara: Electronic portfolio, weblog, resume builder and social networking
- * Copyright (C) 2006-2008 Catalyst IT Ltd (http://www.catalyst.net.nz)
+ * Copyright (C) 2006-2009 Catalyst IT Ltd and others; see:
+ *                         http://wiki.mahara.org/Contributors
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +21,7 @@
  * @subpackage admin
  * @author     Catalyst IT Ltd
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL
- * @copyright  (C) 2006-2008 Catalyst IT Ltd http://catalyst.net.nz
+ * @copyright  (C) 2006-2009 Catalyst IT Ltd http://catalyst.net.nz
  *
  */
 
@@ -32,70 +33,15 @@ define('TITLE', get_string('accountsettings', 'admin'));
 define('SECTION_PLUGINTYPE', 'core');
 define('SECTION_PLUGINNAME', 'admin');
 require_once('pieforms/pieform.php');
+require_once('activity.php');
 
 $id = param_integer('id');
 $user = new User;
 $user->find_by_id($id);
 
 if (!$USER->is_admin_for_user($user)) {
-    redirect(get_config('wwwroot').'user/view.php?id='.$id);
-}
-
-$suspended = $user->get('suspendedcusr');
-if (empty($suspended)) {
-    $suspendform = pieform(array(
-        'name'       => 'edituser_suspend',
-        'plugintype' => 'core',
-        'pluginname' => 'admin',
-        'elements'   => array(
-            'id' => array(
-                 'type'    => 'hidden',
-                 'value'   => $id,
-            ),
-            'reason' => array(
-                'type'        => 'textarea',
-                'rows'        => 5,
-                'cols'        => 60,
-                'title'       => get_string('reason'),
-                'description' => get_string('suspendedreasondescription', 'admin'),
-            ),
-            'submit' => array(
-                'type'  => 'submit',
-                'value' => get_string('suspenduser','admin'),
-            ),
-        )
-    ));
-} else {
-    $suspendform = pieform(array(
-        'name'       => 'edituser_unsuspend',
-        'plugintype' => 'core',
-        'pluginname' => 'admin',
-        'elements'   => array(
-            'id' => array(
-                 'type'    => 'hidden',
-                 'value'   => $id,
-            ),
-            'submit' => array(
-                'type'  => 'submit',
-                'value' => get_string('unsuspenduser','admin'),
-            ),
-        )
-    ));
-    $suspender = display_name(get_record('usr', 'id', $suspended));
-}
-
-function edituser_suspend_submit(Pieform $form, $values) {
-    global $SESSION;
-    suspend_user($values['id'], $values['reason']);
-    $SESSION->add_ok_msg(get_string('usersuspended', 'admin'));
-    redirect('/admin/users/edit.php?id=' . $values['id']);
-}
-
-function edituser_unsuspend_submit(Pieform $form, $values) {
-    global $SESSION;
-    unsuspend_user($values['id']);
-    $SESSION->add_ok_msg(get_string('userunsuspended', 'admin'));
-    redirect('/admin/users/edit.php?id=' . $values['id']);
+    $SESSION->add_error_msg(get_string('youcannotadministerthisuser', 'admin'));
+    redirect('/user/view.php?id=' . $id);
 }
 
 
@@ -122,14 +68,14 @@ if ($USER->get('admin')) {
     $elements['staff'] = array(
         'type'         => 'checkbox',
         'title'        => get_string('sitestaff','admin'),
-        //'description'  => get_string('sitestaffdescription','admin'),
         'defaultvalue' => $user->staff,
+        'help'         => true,
     );
     $elements['admin'] = array(
         'type'         => 'checkbox',
         'title'        => get_string('siteadmin','admin'),
-        //'description'  => get_string('siteadmindescription','admin'),
         'defaultvalue' => $user->admin,
+        'help'         => true,
     );
 }
 $elements['expiry'] = array(
@@ -152,10 +98,18 @@ $authinstances = auth_get_auth_instances();
 if (count($authinstances) > 1) {
     $options = array();
 
+    // NOTE: This is a little broken at the moment. The "username in the remote 
+    // system" setting is only actively used by the XMLRPC authentication 
+    // plugin, and thus only makes sense when the user is authenticating in 
+    // this manner.
+    //
+    // We hope to one day make it possible for users to get into accounts via 
+    // multiple methods, at which time we can tie the username-in-remote-system 
+    // setting to the XMLRPC plugin only, making the UI a bit more consistent
     $external = false;
     foreach ($authinstances as $authinstance) {
         if ($USER->can_edit_institution($authinstance->name)) {
-            $options[$authinstance->id] = $authinstance->displayname. ': '.$authinstance->instancename;
+            $options[$authinstance->id] = $authinstance->instancename . ' (' . $authinstance->displayname . ')';
             if ($authinstance->authname != 'internal') {
                 $external = true;
             }
@@ -166,16 +120,17 @@ if (count($authinstances) > 1) {
         $elements['authinstance'] = array(
             'type'         => 'select',
             'title'        => get_string('authenticatedby', 'admin'),
-            //'description'  => get_string('authenticatedbydescription', 'admin'),
+            'description'  => get_string('authenticatedbydescription', 'admin'),
             'options'      => $options,
             'defaultvalue' => $user->authinstance,
+            'help'         => true,
         );
         if ($external) {
             $un = get_field('auth_remote_user', 'remoteusername', 'authinstance', $user->authinstance, 'localusr', $user->id);
             $elements['remoteusername'] = array(
                 'type'         => 'text',
                 'title'        => get_string('remoteusername', 'admin'),
-                'description'  => get_string('remoteusernamedescription', 'admin'),
+                'description'  => get_string('remoteusernamedescription', 'admin', hsc(get_config('sitename'))),
                 'defaultvalue' => $un ? $un : $user->username,
             );
         }
@@ -257,8 +212,117 @@ function edituser_site_submit(Pieform $form, $values) {
 }
 
 
-// Institution settings form
+// Suspension/deletion controls
+$suspended = $user->get('suspendedcusr');
+if (empty($suspended)) {
+    $suspendform = pieform(array(
+        'name'       => 'edituser_suspend',
+        'plugintype' => 'core',
+        'pluginname' => 'admin',
+        'elements'   => array(
+            'id' => array(
+                 'type'    => 'hidden',
+                 'value'   => $id,
+            ),
+            'reason' => array(
+                'type'        => 'textarea',
+                'rows'        => 5,
+                'cols'        => 60,
+                'title'       => get_string('reason'),
+                'description' => get_string('suspendedreasondescription', 'admin'),
+            ),
+            'submit' => array(
+                'type'  => 'submit',
+                'value' => get_string('suspenduser','admin'),
+            ),
+        )
+    ));
+}
+else {
+    $suspendformdef = array(
+        'name'       => 'edituser_unsuspend',
+        'plugintype' => 'core',
+        'pluginname' => 'admin',
+        'renderer'   => 'oneline',
+        'elements'   => array(
+            'id' => array(
+                 'type'    => 'hidden',
+                 'value'   => $id,
+            ),
+            'submit' => array(
+                'type'  => 'submit',
+                'value' => get_string('unsuspenduser','admin'),
+            ),
+        )
+    );
 
+    // Create two forms for unsuspension - one in the suspend message and the 
+    // other where the 'suspend' button normally goes. This keeps the HTML IDs 
+    // unique
+    $suspendform  = pieform($suspendformdef);
+    $suspendformdef['name'] = 'edituser_suspend2';
+    $suspendformdef['successcallback'] = 'edituser_unsuspend_submit';
+    $suspendform2 = pieform($suspendformdef);
+
+    $suspender = display_name(get_record('usr', 'id', $suspended));
+}
+
+function edituser_suspend_submit(Pieform $form, $values) {
+    global $SESSION, $USER, $user;
+    if (!$USER->get('admin') && ($user->get('admin') || $user->get('staff'))) {
+        $SESSION->add_error_msg(get_string('errorwhilesuspending', 'admin'));
+    }
+    else {
+        suspend_user($user->get('id'), $values['reason']);
+        $SESSION->add_ok_msg(get_string('usersuspended', 'admin'));
+    }
+    redirect('/admin/users/edit.php?id=' . $user->get('id'));
+}
+
+function edituser_unsuspend_submit(Pieform $form, $values) {
+    global $SESSION;
+    unsuspend_user($values['id']);
+    $SESSION->add_ok_msg(get_string('userunsuspended', 'admin'));
+    redirect('/admin/users/edit.php?id=' . $values['id']);
+}
+
+$deleteform = pieform(array(
+    'name' => 'edituser_delete',
+    'plugintype' => 'core',
+    'pluginname' => 'admin',
+    'renderer' => 'oneline',
+    'elements'   => array(
+        'id' => array(
+            'type' => 'hidden',
+            'value' => $id,
+        ),
+        'submit' => array(
+            'type' => 'submit',
+            'value' => get_string('deleteuser', 'admin'),
+            'confirm' => get_string('confirmdeleteuser', 'admin'),
+        ),
+    ),
+));
+
+function edituser_delete_validate(Pieform $form, $values) {
+    global $USER, $SESSION;
+    if (!$USER->get('admin')) {
+        $form->set_error('submit', get_string('deletefailed', 'admin'));
+        $SESSION->add_error_msg(get_string('deletefailed', 'admin'));
+    }
+}
+
+function edituser_delete_submit(Pieform $form, $values) {
+    global $SESSION, $USER;
+    if ($USER->get('admin')) {
+        delete_user($values['id']);
+        $SESSION->add_ok_msg(get_string('userdeletedsuccessfully', 'admin'));
+    }
+    redirect('/admin/users/search.php');
+}
+
+
+// Institution settings form
 $elements = array(
     'id' => array(
          'type'    => 'hidden',
@@ -278,7 +342,7 @@ foreach ($user->get('institutions') as $i) {
                 'description'  => get_string('membershipexpirydescription', 'admin'),
                 'minyear'      => $currentdate['year'],
                 'maxyear'      => $currentdate['year'] + 20,
-                'defaultvalue' => $i->expiry
+                'defaultvalue' => $i->membership_expiry
             ),
             $i->institution.'_studentid' => array(
                 'type'         => 'text',
@@ -301,13 +365,13 @@ foreach ($user->get('institutions') as $i) {
                 'type'  => 'submit',
                 'value' => get_string('update'),
             ),
+            $i->institution.'_remove' => array(
+                'type'  => 'submit',
+                'value' => get_string('removeuserfrominstitution', 'admin'),
+                'confirm' => get_string('confirmremoveuserfrominstitution', 'admin'),
+            ),
         ),
     );
-    $elements[$i->institution.'_remove'] = array(
-        'type'  => 'submit',
-        'value' => get_string('remove'),
-        'confirm' => get_string('confirmremoveuserfrominstitution', 'admin'),
-    );
 }
 
 // Only site admins can add institutions; institutional admins must invite
@@ -320,14 +384,18 @@ if ($USER->get('admin')
         }
     }
     if (!empty($options)) {
+        $elements['addinstitutionheader'] = array(
+            'type'  => 'markup',
+            'value' => '<tr><td colspan="2"><h4>' . get_string('addusertoinstitution', 'admin') . '</h4></td></tr>',
+        );
         $elements['addinstitution'] = array(
             'type'         => 'select',
-            'title'        => get_string('addinstitution', 'admin'),
+            'title'        => get_string('institution'),
             'options'      => $options,
         );
         $elements['add'] = array(
             'type'  => 'submit',
-            'value' => get_string('addinstitution','admin'),
+            'value' => get_string('addusertoinstitution', 'admin'),
         );
     }
 }
@@ -393,27 +461,7 @@ function edituser_institution_submit(Pieform $form, $values) {
 
     if (isset($values['add']) && $USER->get('admin')
         && (empty($userinstitutions) || get_config('usersallowedmultipleinstitutions'))) {
-        // Do nothing if the user is already in the institution
-        $addinstitution = get_record('institution', 'name', $values['addinstitution']);
-        if (!$addinstitution || $addinstitution->name == 'mahara'
-            || $user->in_institution($addinstitution->name)) {
-            redirect('/admin/users/edit.php?id='.$user->id);
-        }
-        $now = time();
-        if (!empty($addinstitution->defaultmembershipperiod)) {
-            $expiry = db_format_timestamp($now + $addinstitution->defaultmembershipperiod);
-        } else {
-            $expiry = null;
-        }
-        db_begin();
-        insert_record('usr_institution', (object) array(
-            'usr' => $user->id,
-            'institution' => $addinstitution->name,
-            'ctime' => db_format_timestamp($now),
-            'expiry' => $expiry,
-        ));
-        handle_event('updateuser', $user->id);
-        db_commit();
+        $user->join_institution($values['addinstitution']);
     }
 
     redirect('/admin/users/edit.php?id='.$user->id);
@@ -426,16 +474,21 @@ if ($suspended) {
     $smarty->assign('suspendedby', get_string('suspendedby', 'admin', $suspender));
 }
 $smarty->assign('suspendform', $suspendform);
+if (isset($suspendform2)) {
+    $smarty->assign('suspendform2', $suspendform2);
+}
+$smarty->assign('deleteform', $deleteform);
 $smarty->assign('siteform', $siteform);
 $smarty->assign('institutions', count($allinstitutions) > 1);
 $smarty->assign('institutionform', $institutionform);
 
 if ($id != $USER->get('id') && is_null($USER->get('parentuser'))) {
-    $loginas = get_string('loginasuser', 'admin', $user->username);
+    $loginas = get_string('loginasuser', 'admin', hsc($user->username));
 } else {
     $loginas = null;
 }
 $smarty->assign('loginas', $loginas);
+$smarty->assign('PAGEHEADING', hsc(TITLE . ': ' . display_name($user)));
 $smarty->display('admin/users/edit.tpl');
 
 ?>
