@@ -1,7 +1,8 @@
 <?php
 /**
  * Mahara: Electronic portfolio, weblog, resume builder and social networking
- * Copyright (C) 2006-2008 Catalyst IT Ltd (http://www.catalyst.net.nz)
+ * Copyright (C) 2006-2009 Catalyst IT Ltd and others; see:
+ *                         http://wiki.mahara.org/Contributors
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +21,7 @@
  * @subpackage admin
  * @author     Catalyst IT Ltd
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL
- * @copyright  (C) 2006-2008 Catalyst IT Ltd http://catalyst.net.nz
+ * @copyright  (C) 2006-2009 Catalyst IT Ltd http://catalyst.net.nz
  *
  */
 
@@ -29,15 +30,16 @@
 define('INTERNAL', 1);
 define('INSTITUTIONALADMIN', 1);
 require(dirname(dirname(dirname(__FILE__))) . '/init.php');
-define('TITLE', get_string('adminusers', 'admin'));
+define('TITLE', get_string('institutionadmins', 'admin'));
 require_once('pieforms/pieform.php');
 define('MENUITEM', 'manageinstitutions/institutionadmins');
-$smarty = smarty();
 
 require_once('institution.php');
-$institution = add_institution_selector_to_page($smarty, param_alphanum('institution', false),
-                                                get_config('wwwroot') . 'admin/users/institutionadmins.php');
+$s = institution_selector_for_page(param_alphanum('institution', false),
+                                   get_config('wwwroot') . 'admin/users/institutionadmins.php');
+$institution = $s['institution'];
 
+$smarty = smarty();
 if ($institution === false) {
     $smarty->display('admin/users/noinstitutions.tpl');
     exit;
@@ -94,13 +96,17 @@ function adminusers_submit(Pieform $form, $values) {
             SET admin = 1
             WHERE usr IN (' . join(',', $values['users']) . ') AND institution = ' . db_quote($inst));
     }
+    require_once('activity.php');
     activity_add_admin_defaults($values['users']);
     db_commit();
     $SESSION->add_ok_msg(get_string('adminusersupdated', 'admin'));
     redirect('/admin/users/institutionadmins.php?institution=' . $inst);
 }
 
+$smarty->assign('institutionselector', $s['institutionselector']);
+$smarty->assign('INLINEJAVASCRIPT', $s['institutionselectorjs']);
 $smarty->assign('adminusersform', pieform($form));
+$smarty->assign('PAGEHEADING', hsc(TITLE));
 $smarty->display('admin/users/institutionadmins.tpl');
 
 ?>
