@@ -1,7 +1,8 @@
 <?php
 /**
  * Mahara: Electronic portfolio, weblog, resume builder and social networking
- * Copyright (C) 2006-2008 Catalyst IT Ltd (http://www.catalyst.net.nz)
+ * Copyright (C) 2006-2009 Catalyst IT Ltd and others; see:
+ *                         http://wiki.mahara.org/Contributors
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +21,7 @@
  * @subpackage admin
  * @author     Catalyst IT Ltd
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL
- * @copyright  (C) 2006-2008 Catalyst IT Ltd http://catalyst.net.nz
+ * @copyright  (C) 2006-2009 Catalyst IT Ltd http://catalyst.net.nz
  *
  */
 define('INTERNAL', 1);
@@ -32,7 +33,7 @@ define('SECTION_PLUGINNAME', 'admin');
 define('SECTION_PAGE', 'institutions');
 require_once('pieforms/pieform.php');
 define('MENUITEM', 'manageinstitutions/institutions');
-$smarty = smarty();
+$smarty = smarty(array('lib/pieforms/static/core/pieforms.js'));
 
 $institution = param_variable('i', '');
 $add         = param_boolean('add');
@@ -77,8 +78,28 @@ if ($institution || $add) {
             global $SESSION;
 
             $authinstanceids = get_column('auth_instance', 'id', 'institution', $values['i']);
+            $viewids = get_column('view', 'id', 'institution', $values['i']);
+            $artefactids = get_column('artefact', 'id', 'institution', $values['i']);
 
             db_begin();
+            if ($viewids) {
+                require_once(get_config('libroot') . 'view.php');
+                foreach ($viewids as $viewid) {
+                    $view = new View($viewid);
+                    $view->delete();
+                }
+            }
+            if ($artefactids) {
+                foreach ($artefactids as $artefactid) {
+                    try {
+                        $a = artefact_instance_from_id($artefactid);
+                        $a->delete();
+                    }
+                    catch (ArtefactNotFoundException $e) {
+                        // Awesome, it's already gone.
+                    }
+                }
+            }
             foreach ($authinstanceids as $id) {
                 delete_records('auth_instance_config', 'instance', $id);
             }
@@ -122,7 +143,7 @@ if ($institution || $add) {
     if (!$add) {
         $data = get_record('institution', 'name', $institution);
         $lockedprofilefields = (array) get_column('institution_locked_profile_field', 'profilefield', 'name', $institution);
-        
+
         // TODO: Find a better way to work around Smarty's minimal looping logic
         if (!empty($authinstances)) {
             foreach($authinstances as $key => $val) {
@@ -145,6 +166,7 @@ if ($institution || $add) {
     else {
         $data = new StdClass;
         $data->displayname = '';
+        $data->expiry = null;
         if (!get_config('usersuniquebyusername')) {
             $data->registerallowed = 1;
         }
@@ -158,7 +180,9 @@ if ($institution || $add) {
     $themeoptions = get_themes();
     $themeoptions['sitedefault'] = '- ' . get_string('sitedefault', 'admin') . ' (' . $themeoptions[get_config('theme')] . ') -';
     uksort($themeoptions, 'theme_sort');
-    
+
+    $sitename = get_config('sitename');
+
     safe_require('artefact', 'internal');
     $elements = array(
         'name' => array(
@@ -199,7 +223,17 @@ if ($institution || $add) {
             'help'   => true,
         ),
     );
-
+    if ($institution != 'mahara') {
+       $elements['expiry'] = array(
+            'type'         => 'date',
+            'title'        => get_string('institutionexpiry', 'admin'),
+            'description'  => get_string('institutionexpirydescription', 'admin', hsc($sitename)),
+            'defaultvalue' => is_null($data->expiry) ? null : strtotime($data->expiry),
+            'help'         => true,
+            'minyear'      => date('Y') - 2,
+            'maxyear'      => date('Y') + 10,
+        );
+    }
     if ($USER->get('admin')) {
         $elements['authplugin'] = array(
             'type'    => 'authlist',
@@ -267,7 +301,7 @@ if ($institution || $add) {
 
     $elements['lockedfields'] = array(
         'type' => 'fieldset',
-        'legend' => 'Locked fields ',
+        'legend' => get_string('Lockedfields', 'admin'),
         'collapsible' => true,
         'collapsed' => true,
         'elements' => array(),
@@ -280,7 +314,7 @@ if ($institution || $add) {
         );
     }
     $elements['lockedfieldshelp'] = array(
-        'value' => '<tr><th colspan="2">'
+        'value' => '<tr id="lockedfieldshelp"><th colspan="2">'
         . get_help_icon('core', 'admin', 'institution', 'lockedfields') 
         . '</th></tr>'
     );
@@ -320,6 +354,7 @@ else {
             ii.name,
             ii.displayname,
             ii.maxuseraccounts,
+            ii.suspended,
             COALESCE(a.members, 0) AS members,
             COALESCE(a.staff, 0) AS staff,
             COALESCE(a.admins, 0) AS admins
@@ -355,7 +390,7 @@ function institution_validate(Pieform $form, $values) {
 }
 
 function institution_submit(Pieform $form, $values) {
-    global $SESSION, $institution, $add, $instancearray, $USER;
+    global $SESSION, $institution, $add, $instancearray, $USER, $authinstances;
 
     db_begin();
     // Update the basic institution record...
@@ -377,6 +412,7 @@ function institution_submit(Pieform $form, $values) {
     }
     $newinstitution->theme                        = (empty($values['theme']) || $values['theme'] == 'sitedefault') ? null : $values['theme'];
     if ($institution != 'mahara') {
+        $newinstitution->expiry                   = db_format_timestamp($values['expiry']);
         $newinstitution->defaultmembershipperiod  = ($values['defaultmembershipperiod']) ? intval($values['defaultmembershipperiod']) : null;
         if ($USER->get('admin')) {
             $newinstitution->maxuseraccounts      = ($values['maxuseraccounts']) ? intval($values['maxuseraccounts']) : null;
@@ -409,7 +445,21 @@ function institution_submit(Pieform $form, $values) {
         }
 
         foreach($values['authplugin']['deletearray'] as $instanceid) {
-            execute_sql('UPDATE {usr} SET lastauthinstance = NULL WHERE lastauthinstance = ?', array($instanceid));
+            // If this authinstance is the only xmlrpc authinstance that references a host, delete the host record.
+            $hostwwwroot = null;
+            foreach ($authinstances as $ai) {
+                if ($ai->id == $instanceid && $ai->authname == 'xmlrpc') {
+                    $hostwwwroot = get_field_sql("SELECT value FROM {auth_instance_config} WHERE instance = ? AND field = 'wwwroot'", array($instanceid));
+                    if ($hostwwwroot && count_records_select('auth_instance_config', "field = 'wwwroot' AND value = ?", array($hostwwwroot)) == 1) {
+                        // Unfortunately, it's possible that this host record could belong to a different institution,
+                        // so specify the institution here.
+                        delete_records('host', 'wwwroot', $hostwwwroot, 'institution', $institution);
+                        // We really need to fix this, either by removing the institution from the host table, or refusing to allow the
+                        // institution to be changed in the host record when another institution's authinstance is still pointing at it.
+                    }
+                    break;
+                }
+            }
             delete_records('auth_remote_user', 'authinstance', $instanceid);
             delete_records('auth_instance_config', 'instance', $instanceid);
             delete_records('auth_instance', 'id', $instanceid);
@@ -461,6 +511,7 @@ function institution_submit(Pieform $form, $values) {
         $message = get_string('institutionupdatedsuccessfully', 'admin');
         if (isset($values['theme']) && $oldtheme != $values['theme']
             && (!empty($oldtheme) || $values['theme'] != 'sitedefault')) {
+            $USER->update_theme();
             $message .= '  ' . get_string('usersseenewthemeonlogin', 'admin');
         }
         $SESSION->add_ok_msg($message);
@@ -474,7 +525,90 @@ function institution_cancel_submit() {
     redirect('/admin/users/institutions.php');
 }
 
-$smarty->assign('heading', get_string('admininstitutions', 'admin'));			
+if ($institution && $institution != 'mahara') {
+    function institution_suspend_submit(Pieform $form, $values) {
+        global $SESSION, $USER;
+        if (!$USER->get('admin')) {
+            $SESSION->add_error_msg(get_string('errorwhilesuspending', 'admin'));
+        }
+        else {
+            set_field('institution', 'suspended', 1, 'name', $values['i']);
+            $SESSION->add_ok_msg(get_string('institutionsuspended', 'admin'));
+        }
+        redirect('/admin/users/institutions.php?i=' . $values['i']);
+    }
+
+    function institution_unsuspend_submit(Pieform $form, $values) {
+        global $SESSION, $USER;
+        if (!$USER->get('admin')) {
+            $SESSION->add_error_msg(get_string('errorwhileunsuspending', 'admin'));
+        }
+        else {
+            set_field('institution', 'suspended', 0, 'name', $values['i']);
+            $SESSION->add_ok_msg(get_string('institutionunsuspended', 'admin'));
+        }
+        redirect('/admin/users/institutions.php?i=' . $values['i']);
+    }
+
+    $_institution = get_record('institution', 'name', $institution);
+    // Suspension controls
+    $suspended = $_institution->suspended;
+    if (empty($suspended)) {
+        $suspendformdef = array(
+            'name'       => 'institution_suspend',
+            'plugintype' => 'core',
+            'pluginname' => 'admin',
+            'elements'   => array(
+                'i' => array(
+                     'type'    => 'hidden',
+                     'value'   => $institution,
+                ),
+                'submit' => array(
+                    'type'        => 'submit',
+                    'value'       => get_string('suspendinstitution','admin'),
+                    'description' => get_string('suspendinstitutiondescription','admin'),
+                ),
+            )
+        );
+
+        $suspendform  = pieform($suspendformdef);
+    }
+    else {
+        $suspendformdef = array(
+            'name'       => 'institution_unsuspend',
+            'plugintype' => 'core',
+            'pluginname' => 'admin',
+            'elements'   => array(
+                'i' => array(
+                     'type'    => 'hidden',
+                     'value'   => $institution,
+                ),
+                'submit' => array(
+                    'type'        => 'submit',
+                    'value'       => get_string('unsuspendinstitution','admin'),
+                    'description' => get_string('unsuspendinstitutiondescription','admin'),
+                ),
+            )
+        );
+        $suspendform  = pieform($suspendformdef);
+
+        // Create a second forms for unsuspension to go in the suspend message.
+        // This keeps the HTML IDs unique
+        $suspendformdef['name'] = 'institution_unsuspend_top';
+        $suspendformdef['renderer'] = 'oneline';
+        $suspendformdef['successcallback'] = 'institution_unsuspend_submit';
+        $suspendform_top = pieform($suspendformdef);
+    }
+    $smarty->assign('suspendform', $suspendform);
+    if (isset($suspendform_top)) {
+        $smarty->assign('suspendform_top', $suspendform_top);
+    }
+    if ($suspended) {
+        $smarty->assign('suspended', get_string('suspendedinstitutionmessage', 'admin'));
+    }
+}
+
+$smarty->assign('PAGEHEADING', hsc(get_string('admininstitutions', 'admin')));
 $smarty->display('admin/users/institutions.tpl');
 
 function theme_sort($a, $b) {
