@@ -1,7 +1,8 @@
 <?php
 /**
  * Mahara: Electronic portfolio, weblog, resume builder and social networking
- * Copyright (C) 2006-2008 Catalyst IT Ltd (http://www.catalyst.net.nz)
+ * Copyright (C) 2006-2009 Catalyst IT Ltd and others; see:
+ *                         http://wiki.mahara.org/Contributors
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +21,7 @@
  * @subpackage admin
  * @author     Catalyst IT Ltd
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL
- * @copyright  (C) 2006-2008 Catalyst IT Ltd http://catalyst.net.nz
+ * @copyright  (C) 2006-2009 Catalyst IT Ltd http://catalyst.net.nz
  *
  */
 
@@ -29,15 +30,16 @@
 define('INTERNAL', 1);
 define('INSTITUTIONALADMIN', 1);
 require(dirname(dirname(dirname(__FILE__))) . '/init.php');
-define('TITLE', get_string('staffusers', 'admin'));
+define('TITLE', get_string('institutionstaff', 'admin'));
 require_once('pieforms/pieform.php');
 define('MENUITEM', 'manageinstitutions/institutionstaff');
-$smarty = smarty();
 
 require_once('institution.php');
-$institution = add_institution_selector_to_page($smarty, param_alphanum('institution', false),
-                                                get_config('wwwroot') . 'admin/users/institutionstaff.php');
+$s = institution_selector_for_page(param_alphanum('institution', false),
+                                   get_config('wwwroot') . 'admin/users/institutionstaff.php');
+$institution = $s['institution'];
 
+$smarty = smarty();
 if ($institution === false) {
     $smarty->display('admin/users/noinstitutions.tpl');
     exit;
@@ -99,7 +101,10 @@ function staffusers_submit(Pieform $form, $values) {
     redirect('/admin/users/institutionstaff.php?institution=' . $inst);
 }
 
+$smarty->assign('institutionselector', $s['institutionselector']);
+$smarty->assign('INLINEJAVASCRIPT', $s['institutionselectorjs']);
 $smarty->assign('staffusersform', pieform($form));
+$smarty->assign('PAGEHEADING', hsc(TITLE));
 $smarty->display('admin/users/institutionstaff.tpl');
 
 ?>
