@@ -1,7 +1,8 @@
 <?php
 /**
  * Mahara: Electronic portfolio, weblog, resume builder and social networking
- * Copyright (C) 2006-2008 Catalyst IT Ltd (http://www.catalyst.net.nz)
+ * Copyright (C) 2006-2009 Catalyst IT Ltd and others; see:
+ *                         http://wiki.mahara.org/Contributors
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +21,7 @@
  * @subpackage blocktype-internalmedia
  * @author     Catalyst IT Ltd
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL
- * @copyright  (C) 2006-2008 Catalyst IT Ltd http://catalyst.net.nz
+ * @copyright  (C) 2006-2009 Catalyst IT Ltd http://catalyst.net.nz
  *
  */
 
@@ -46,16 +47,11 @@ class PluginBlocktypeInternalmedia extends PluginBlocktype {
 
     public static function postinst($oldversion) {
         if ($oldversion == 0) {
-            $enabledtypes = array_map(
-                create_function('$a', 'return 1;'),
-                self::get_all_extensions());
-            unset($enabledtypes['swf']); // disable by default
-            $enabledtypes = array_keys($enabledtypes);
-            set_config_plugin('blocktype', 'internalmedia', 'enabledtypes', serialize($enabledtypes));
+            set_config_plugin('blocktype', 'internalmedia', 'enabledtypes', serialize(array('flv')));
         }
     }
 
-    public static function render_instance(BlockInstance $instance) {
+    public static function render_instance(BlockInstance $instance, $editing=false) {
         $configdata = $instance->get('configdata');
 
         if (empty($configdata['artefactid'])) {
@@ -63,15 +59,16 @@ class PluginBlocktypeInternalmedia extends PluginBlocktype {
         }
         $result = self::get_js_source();
         require_once(get_config('docroot') . 'artefact/lib.php');
-        $artefact = artefact_instance_from_id($configdata['artefactid']);
+        $artefact = $instance->get_artefact_instance($configdata['artefactid']);
         $width  = (!empty($configdata['width'])) ? hsc($configdata['width']) : '300';
         $height = (!empty($configdata['height'])) ? hsc($configdata['height']) : '300';
-        $extn = strtolower($artefact->get('oldextension'));
-        if (!in_array($extn, self::get_allowed_extensions())) {
+        $mimetype = $artefact->get('filetype');
+        $mimetypefiletypes = self::get_allowed_mimetype_filetypes();
+        if (!isset($mimetypefiletypes[$mimetype])) {
             return get_string('typeremoved', 'blocktype.file/internalmedia');
         }
-        $callbacks = self::get_all_extensions();
-        $result .= call_static_method('PluginBlocktypeInternalmedia', $callbacks[$extn], $artefact, $instance, $width, $height);
+        $callbacks = self::get_all_filetype_players();
+        $result .= '<div class="mediaplayer-container center"><div class="mediaplayer">' . call_static_method('PluginBlocktypeInternalmedia', $callbacks[$mimetypefiletypes[$mimetype]], $artefact, $instance, $width, $height) . '</div></div>';
         return $result;
     }
 
@@ -81,51 +78,52 @@ class PluginBlocktypeInternalmedia extends PluginBlocktype {
 
     public static function instance_config_form($instance) {
         $configdata = $instance->get('configdata');
-        $form = array(
-            self::artefactchooser_element((isset($configdata['artefactid'])) ? $configdata['artefactid'] : null),
-        );
-        $form['width'] = array(
+        safe_require('artefact', 'file');
+        $instance->set('artefactplugin', 'file');
+        return array(
+            'artefactid' => self::filebrowser_element($instance, (isset($configdata['artefactid'])) ? array($configdata['artefactid']) : null),
+            'width' => array(
                 'type' => 'text',
                 'title' => get_string('width'),
                 'size' => 3,
                 'defaultvalue' => (isset($configdata['width'])) ? $configdata['width'] : '',
-        );
-        $form['height'] = array(
+            ),
+            'height' => array(
                 'type' => 'text',
                 'title' => get_string('height'),
                 'size' => 3,
                 'defaultvalue' => (isset($configdata['height'])) ? $configdata['height'] : '',
+            ),
         );
-        return $form;
     }
 
-    public static function get_artefacts(BlockInstance $instance) {
-        $configdata = $instance->get('configdata');
-        if (isset($configdata['artefactid'])) {
-            return array($configdata['artefactid']);
-        }
-        return false;
+    public static function filebrowser_element(&$instance, $default=array()) {
+        $element = ArtefactTypeFileBase::blockconfig_filebrowser_element($instance, $default);
+        $element['title'] = get_string('media', 'blocktype.file/internalmedia');
+        $element['name'] = 'artefactid';
+        $element['config']['selectone'] = true;
+        $element['filters'] = array(
+            'artefacttype'    => array('file'),
+            'filetype'        => self::get_allowed_mimetypes(),
+        );
+        return $element;
     }
 
     public static function artefactchooser_element($default=null) {
-        $extraselect = '(' . implode(' OR ', array_map(
-            create_function('$a', 'return "title LIKE \'%.$a\'";'),
-            self::get_allowed_extensions())
-        ) . ')';
+        $extraselect = 'filetype IN (' . join(',', array_map('db_quote', self::get_allowed_mimetypes())) . ')';
+        $extrajoin   = ' JOIN {artefact_file_files} ON {artefact_file_files}.artefact = a.id ';
 
         return array(
             'name' => 'artefactid',
             'type'  => 'artefactchooser',
             'title' => get_string('media', 'blocktype.file/internalmedia'),
             'defaultvalue' => $default,
-            'rules' => array(
-                'required' => true,
-            ),
             'blocktype' => 'internalmedia',
             'limit' => 5,
             'selectone' => true,
             'artefacttypes' => array('file'),
             'extraselect' => $extraselect,
+            'extrajoin' => $extrajoin,
             'template' => 'artefact:file:artefactchooser-element.tpl',
         );
     }
@@ -138,7 +136,7 @@ class PluginBlocktypeInternalmedia extends PluginBlocktype {
     public static function save_config_options($values) {
         $enabledtypes = array();
         foreach ($values as $type => $enabled) {
-            if (!in_array($type, array_keys(self::get_all_extensions()))) {
+            if (!in_array($type, self::get_all_filetypes())) {
                 continue;
             }
             if (!empty($enabled)) {
@@ -152,29 +150,14 @@ class PluginBlocktypeInternalmedia extends PluginBlocktype {
         $elements = array();
         // Allowed file types
         $filetypes = array();
-        $currenttypes = self::get_allowed_extensions();
+        $currenttypes = self::get_allowed_filetypes();
 
-        if (!$plugindisabled = get_column_sql('SELECT description
-            FROM {artefact_file_file_types} 
-            WHERE enabled = 0')) {
-            $plugindisabled = array();
-        }
-        foreach (array_keys(self::get_all_extensions()) as $filetype) {
-            if (in_array($filetype, $plugindisabled)) {
-                $filetypes[$filetype] = array(
-                    'type'  => 'checkbox',
-                    'title' => get_string($filetype, 'artefact.file'),
-                    'value' => false,
-                    'disabled' => true,
-                );
-            }
-            else {
-                $filetypes[$filetype] = array(
-                    'type'  => 'checkbox',
-                    'title' => get_string($filetype, 'artefact.file'),
-                    'defaultvalue' => in_array($filetype, $currenttypes),
-                );
-            }
+        foreach (self::get_all_filetypes() as $filetype) {
+            $filetypes[$filetype] = array(
+                'type'  => 'checkbox',
+                'title' => get_string($filetype, 'artefact.file'),
+                'defaultvalue' => in_array($filetype, $currenttypes),
+            );
         }
         uasort($filetypes, create_function('$a, $b', 'return $a["title"] > $b["title"];'));
         $filetypes = array_merge(
@@ -192,23 +175,52 @@ class PluginBlocktypeInternalmedia extends PluginBlocktype {
         );
     }
 
-    private static function get_allowed_extensions() {
+
+    private static function get_allowed_filetypes() {
         if ($data = get_config_plugin('blocktype', 'internalmedia', 'enabledtypes')) {
             return unserialize($data);
         }
         return array();
     }
 
-    private static function get_all_extensions() {
+
+    private static function get_allowed_mimetypes() {
+        return array_keys(self::get_allowed_mimetype_filetypes());
+    }
+
+
+    private static function get_allowed_mimetype_filetypes() {
+        if ($data = self::get_allowed_filetypes()) {
+            if ($mimetypes = get_records_sql_assoc('
+                SELECT mimetype, description
+                FROM {artefact_file_mime_types}
+                WHERE description IN (' . join(',', array_map('db_quote', $data)) . ')', array())) {
+                foreach ($mimetypes as &$m) {
+                    $m = $m->description;
+                }
+                return $mimetypes;
+            }
+        }
+        return array();
+    }
+
+
+    private static function get_all_filetypes() {
+        return array_keys(self::get_all_filetype_players());
+    }
+
+
+    private static function get_all_filetype_players() {
+        /* Keyed by the file type descriptions from the artefact_file_mime_types table */
         return array(
-            'mp3'   => 'flash_player', // tested
-            'swf'   => 'flash_player', // tested
-            'flv'   => 'flash_player', // tested
-            'mov'   => 'qt_player',  // tested
-            'wmv'   => 'wmp_player', // tested
-            'mpeg'  => 'qt_player',  // tested
-            'mpg'   => 'qt_player',  // tested
-            'avi'   => 'wmp_player', // tested
+            'mp3'       => 'flash_player', // tested
+            'swf'       => 'flash_player', // tested
+            'flv'       => 'flash_player', // tested
+            'quicktime' => 'qt_player',  // tested
+            'wmv'       => 'wmp_player', // tested
+            'mpeg'      => 'qt_player',  // tested
+            'avi'       => 'wmp_player', // tested
+            'mp4_video' => 'qt_player', // tested
             /* commenting out for now
             'ram'   => 'real_player',
             'rm'    => 'real_player',
@@ -235,13 +247,13 @@ class PluginBlocktypeInternalmedia extends PluginBlocktype {
         }
         $html =  '<a href="' . $url . '">' . hsc($artefact->get('title')) . '</a><br>
                <span class="blocktype_internalmedia_mp3" id="' . $id . '">(' 
-               . get_string('flash', 'blocktype.file/internalmedia') . ')</span>
+               . get_string('flashanimation', 'blocktype.file/internalmedia') . ')</span>
                 <script type="text/javascript">
                     var so = new SWFObject(" ' . $playerurl . '","player","' . $width . '","' . ($height + 20). '","7");
                     so.addParam("allowfullscreen","false");
                     so.addVariable("file","' . urlencode($url) . '");
                     so.addVariable("displayheight"," ' . $height . '");
-                    so.addVariable("type", "' . $artefact->get('oldextension') . '");
+                    so.addVariable("type", "' . $extn . '");
                     so.addVariable("height", "' . $height . '");
                     so.addVariable("width", "' . $width . '");
                 ';
@@ -262,7 +274,7 @@ class PluginBlocktypeInternalmedia extends PluginBlocktype {
         $url = self::get_download_link($artefact, $block);
 
         require_once('file.php');
-        $mimetype = get_mime_type($artefact->get_path());
+        $mimetype = $artefact->get('filetype');
         $autostart = 'false';
 
         return '<a href="' . $url . '">' . hsc($artefact->get('title')) . '</a><br>'
@@ -335,7 +347,7 @@ class PluginBlocktypeInternalmedia extends PluginBlocktype {
         $size = 'width="' . $width . '" height="' . $height . '"';
 
         require_once('file.php');
-        $mimetype = get_mime_type($artefact->get_path());
+        $mimetype = $artefact->get('filetype');
         $autostart = 'false';
 
         return '<a href="' . $url . '">' . hsc($artefact->get('title')) . '</a><br>'
@@ -378,6 +390,10 @@ class PluginBlocktypeInternalmedia extends PluginBlocktype {
              <script defer="defer" src="' . get_config('wwwroot') . 'artefact/file/blocktype/internalmedia/eolas_fix.js" type="text/javascript"></script>';
     }
 
+    public static function default_copy_type() {
+        return 'full';
+    }
+
 }
 
 ?>
