@@ -1,7 +1,8 @@
 <?php
 /**
  * Mahara: Electronic portfolio, weblog, resume builder and social networking
- * Copyright (C) 2006-2008 Catalyst IT Ltd (http://www.catalyst.net.nz)
+ * Copyright (C) 2006-2009 Catalyst IT Ltd and others; see:
+ *                         http://wiki.mahara.org/Contributors
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +21,7 @@
  * @subpackage artefact-internal
  * @author     Catalyst IT Ltd
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL
- * @copyright  (C) 2006-2008 Catalyst IT Ltd http://catalyst.net.nz
+ * @copyright  (C) 2006-2009 Catalyst IT Ltd http://catalyst.net.nz
  *
  */
 
@@ -55,7 +56,6 @@ class PluginArtefactInternal extends PluginArtefact {
             'jabberusername',
             'occupation',
             'industry',
-            'profileicon'
         );
     }
 
@@ -104,12 +104,6 @@ class PluginArtefactInternal extends PluginArtefact {
                 'title' => get_string('editprofile', 'artefact.internal'),
                 'weight' => 10,
             ),
-            array(
-                'path' => 'profile/icons',
-                'url' => 'artefact/internal/profileicons.php',
-                'title' => get_string('profileicons', 'artefact.internal'),
-                'weight' => 11,
-            ),
         );
     }
 
@@ -130,6 +124,143 @@ class PluginArtefactInternal extends PluginArtefact {
     public static function sort_child_data($a, $b) {
         return strnatcasecmp($a->text, $b->text);
     }
+
+    public static function can_be_disabled() {
+        return false;
+    }
+
+    public static function get_artefact_type_content_types() {
+        return array(
+            'introduction' => array('text'),
+        );
+    }
+
+    /**
+     * This method is provided by the plugin class so it can be used by the
+     * profileinfo and contactinfo blocktypes. See the blocktypes'
+     * export_blockinstance_config_leap method for more information.
+     *
+     * LEAP2A export doesn't export profile related artefacts as entries, so we
+     * need to take that into account when exporting config for it.
+     *
+     * @param BlockInstance $bi The blockinstance to export the config for.
+     * @return array The config for the blockinstance
+     */
+    public static function export_blockinstance_config_leap(BlockInstance $bi) {
+        static $cache = array();
+        $owner = $bi->get_view()->get('owner');
+
+        // This blocktype is only allowed in personal views
+        if (!$owner) {
+            return array();
+        }
+
+        if (!isset($cache[$owner])) {
+            $cache[$owner] = get_records_sql_assoc("SELECT id, artefacttype, title
+                FROM {artefact}
+                WHERE owner = ?
+                AND artefacttype IN (
+                    SELECT name
+                    FROM {artefact_installed_type}
+                    WHERE plugin = 'internal'
+            )", array($owner));
+        }
+
+        $configdata = $bi->get('configdata');
+
+        $result = array();
+        if (is_array($configdata)) {
+            // Convert the actual profile artefact IDs to their field names
+            if (isset($configdata['artefactids']) && is_array($configdata['artefactids'])) {
+                $result['fields'] = array();
+                foreach ($configdata['artefactids'] as $id) {
+                    $result['fields'][] = $cache[$owner][$id]->artefacttype;
+                }
+                $result['fields'] = json_encode(array($result['fields']));
+            }
+
+            // Email addresses are not entries in leap2a (they're elements on
+            // the persondata element), so we export the actual address here
+            // instead of an artefact ID.
+            if (!empty($configdata['email']) && isset($cache[$owner][$configdata['email']])) {
+                $result['email'] = json_encode(array($cache[$owner][$configdata['email']]->title));
+            }
+
+            if (!empty($configdata['profileicon'])) {
+                $result['artefactid'] = json_encode(array(intval($configdata['profileicon'])));
+            }
+
+            if (isset($configdata['introtext'])) {
+                $result['introtext'] = json_encode(array($configdata['introtext']));
+            }
+        }
+
+        return $result;
+    }
+
+    /**
+     * This method is provided by the plugin class so it can be used by the
+     * profileinfo and contactinfo blocktypes. See the blocktypes'
+     * import_create_blockinstance_leap method for more information.
+     *
+     * @param array $biconfig   The block instance config
+     * @param array $viewconfig The view config
+     * @return BlockInstance The newly made block instance
+     */
+    public static function import_create_blockinstance_leap(array $biconfig, array $viewconfig) {
+        static $cache = array();
+        $configdata = array();
+
+        // This blocktype is only allowed in personal views
+        if (empty($viewconfig['owner'])) {
+            return;
+        }
+        $owner = $viewconfig['owner'];
+
+        if (isset($biconfig['config']) && is_array($biconfig['config'])) {
+            $impcfg = $biconfig['config'];
+            if (isset($impcfg['fields']) && is_array($impcfg['fields'])) {
+                // Convert the fields to their artefact ids
+                $configdata['artefactids'] = array();
+                foreach ($impcfg['fields'] as $field) {
+                    if (!isset($cache[$owner])) {
+                        $cache[$owner] = get_records_sql_assoc("SELECT artefacttype, id
+                            FROM {artefact}
+                            WHERE owner = ?
+                            AND artefacttype IN (
+                                SELECT name
+                                FROM {artefact_installed_type}
+                                WHERE plugin = 'internal'
+                        )", array($owner));
+                    }
+
+                    $configdata['artefactids'][] = $cache[$owner][$field]->id;
+                }
+            }
+
+            if (!empty($impcfg['email'])) {
+                if ($artefactid = get_field('artefact_internal_profile_email', 'artefact', 'owner', $owner, 'email', $impcfg['email'])) {
+                    $configdata['email'] = $artefactid;
+                }
+            }
+
+            if (!empty($impcfg['artefactid'])) {
+                $configdata['profileicon'] = intval($impcfg['artefactid']);
+            }
+
+            if (isset($impcfg['introtext'])) {
+                $configdata['introtext'] = $impcfg['introtext'];
+            }
+        }
+        $bi = new BlockInstance(0,
+            array(
+                'blocktype'  => $biconfig['type'],
+                'configdata' => $configdata,
+            )
+        );
+
+        return $bi;
+    }
 }
 
 class ArtefactTypeProfile extends ArtefactType {
@@ -309,37 +440,6 @@ class ArtefactTypeProfile extends ArtefactType {
             }
         }
 
-        $form['elements']['emptyrow'] = array(
-            'title' => '    ',
-            'type' => 'html',
-            'value' => '&nbsp;',
-        );
-        $currentwidth = get_config_plugin('artefact', 'internal', 'profileiconwidth');
-        $form['elements']['profileiconwidth'] = array(
-            'type' => 'text',
-            'size' => 4,
-            'suffix' => get_string('widthshort'),
-            'title' => get_string('profileiconsize', 'artefact.internal'),
-            'defaultvalue' => ((!empty($currentwidth)) ? $currentwidth : 100),
-            'rules' => array(
-                'required' => true,
-                'integer'  => true,
-            )
-        );
-        $currentheight = get_config_plugin('artefact', 'internal', 'profileiconheight');
-        $form['elements']['profileiconheight'] = array(
-            'type' => 'text',
-            'suffix' => get_string('heightshort'),
-            'size' => 4,
-            'title' => get_string('profileiconsize', 'artefact.internal'),
-            'defaultvalue' => ((!empty($currentheight)) ? $currentheight : 100),
-            'rules' => array(
-                'required' => true,
-                'integer'  => true,
-            ),
-            'help' => true,
-        );
-
         return $form;
     }
 
@@ -359,8 +459,6 @@ class ArtefactTypeProfile extends ArtefactType {
         }
         set_config_plugin('artefact', 'internal', 'profilepublic', $public);
         set_config_plugin('artefact', 'internal', 'profilemandatory', $mandatory);
-        set_config_plugin('artefact', 'internal', 'profileiconwidth', $values['profileiconwidth']);
-        set_config_plugin('artefact', 'internal', 'profileiconheight', $values['profileiconheight']);
     }
 
     public static function get_links($id) {
@@ -386,7 +484,7 @@ class ArtefactTypeProfileField extends ArtefactTypeProfile {
     }
 
     public function render_self($options) {
-        return array('html' => $this->title, 'javascript' => null);
+        return array('html' => hsc($this->title), 'javascript' => null);
     }
 }
 
@@ -457,15 +555,42 @@ class ArtefactTypeEmail extends ArtefactTypeProfileField {
             }
         }
     }
+
+    public function delete() {
+        delete_records('artefact_internal_profile_email', 'artefact', $this->id);
+        parent::delete();
+    }
+
+    public function render_self($options) {
+        if (array_key_exists('link', $options) && $options['link'] == true) {
+            $html = '<a href="mailto:' . hsc($this->title) . '">' . hsc($this->title) . '</a>';
+        }
+        else {
+            $html = $this->title;
+        }
+        return array('html' => $html, 'javascript' => null);
+    }
 }
 
 class ArtefactTypeStudentid extends ArtefactTypeProfileField {}
-class ArtefactTypeIntroduction extends ArtefactTypeProfileField {}
+class ArtefactTypeIntroduction extends ArtefactTypeProfileField {
+    public function render_self($options) {
+        return array('html' => clean_html($this->title), 'javascript' => null);
+    }
+}
 class ArtefactTypeWebAddress extends ArtefactTypeProfileField {
 
+    public function commit() {
+        $url = $this->get('title');
+        if (strlen($url) && strpos($url, '://') == false) {
+            $this->set('title', 'http://' . $url);
+        }
+        parent::commit();
+    }
+
     public function render_self($options) {
-        if ($options['link'] == true) {
-            $html = make_link($this->title);
+        if (array_key_exists('link', $options) && $options['link'] == true) {
+            $html = '<a href="' . hsc($this->title) . '">' . hsc($this->title) . '</a>';
         }
         else {
             $html = $this->title;
@@ -475,7 +600,7 @@ class ArtefactTypeWebAddress extends ArtefactTypeProfileField {
 }
 class ArtefactTypeOfficialwebsite extends ArtefactTypeWebAddress {}
 class ArtefactTypePersonalwebsite extends ArtefactTypeWebAddress {}
-class ArtefactTypeBlogAddress extends ArtefactTypeProfileField {}
+class ArtefactTypeBlogAddress extends ArtefactTypeWebAddress {}
 class ArtefactTypeAddress extends ArtefactTypeProfileField {}
 class ArtefactTypeTown extends ArtefactTypeProfileField {}
 class ArtefactTypeCity extends ArtefactTypeProfileField {}
@@ -499,62 +624,6 @@ class ArtefactTypeJabberusername extends ArtefactTypeProfileField {}
 class ArtefactTypeOccupation extends ArtefactTypeProfileField {}
 class ArtefactTypeIndustry extends ArtefactTypeProfileField {}
 
-class ArtefactTypeProfileIcon extends ArtefactTypeProfileField {
-    public static function is_note_private() {
-        return true;
-    }
-
-    public function render_self($options) {
-        $options['id'] = $this->get('id');
-
-        $size = filesize(get_config('dataroot') . 'artefact/internal/profileicons/originals/'
-            . ($this->get('id') % 256) . '/' . $this->get('id'));
-
-        $downloadpath = get_config('wwwroot') . 'thumb.php?type=profileiconbyid&id=' . $this->id;
-        $smarty = smarty_core();
-        $smarty->assign('iconpath', $this->get_icon($options));
-        $smarty->assign('downloadpath', $downloadpath);
-        $smarty->assign('owner', display_name($this->get('owner')));
-        $smarty->assign('title', $this->get('note'));
-        $smarty->assign('description', $this->get('title'));
-        $smarty->assign('artefacttype', $this->get('artefacttype'));
-        $smarty->assign('created', strftime(get_string('strftimedaydatetime'), $this->get('ctime')));
-        $smarty->assign('modified', strftime(get_string('strftimedaydatetime'), $this->get('mtime')));
-        $smarty->assign('size', display_size($size) . ' (' . $size . ' ' . get_string('bytes') . ')');
-        $smarty->assign('previewpath', get_config('wwwroot') . 'thumb.php?type=profileiconbyid&id=' . $this->get('id') . '&maxwidth=400');
-        return array('html' => $smarty->fetch('artefact:internal:profileicon_render_self.tpl'), 'javascript' => '');
-    }
-
-    public static function get_links($id) {
-        $wwwroot = get_config('wwwroot');
-
-        return array(
-            '_default' => $wwwroot . 'artefact/internal/profileicons.php',
-        );
-    }
-
-    public static function get_icon($options=null) {
-        $url = get_config('wwwroot') . 'thumb.php?type=profileiconbyid&id=' . hsc($options['id']);
-
-        if (isset($options['size'])) {
-            $url .= '&size=' . $options['size'];
-        }
-        else {
-            $url .= '&size=20x20';
-        }
-
-        return $url;
-    }
-
-    public function in_view_list() {
-        return true;
-    }
-
-    public static function get_quota_usage($artefact) {
-        return filesize(get_config('dataroot') . 'artefact/internal/profileicons/originals/'
-            . ($artefact % 256) . '/' . $artefact);
-    }
-}
 
 
 ?>
