@@ -1,7 +1,8 @@
 <?php
 /**
  * Mahara: Electronic portfolio, weblog, resume builder and social networking
- * Copyright (C) 2006-2008 Catalyst IT Ltd (http://www.catalyst.net.nz)
+ * Copyright (C) 2006-2009 Catalyst IT Ltd and others; see:
+ *                         http://wiki.mahara.org/Contributors
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +21,7 @@
  * @subpackage artefact-resume
  * @author     Catalyst IT Ltd
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL
- * @copyright  (C) 2006-2008 Catalyst IT Ltd http://catalyst.net.nz
+ * @copyright  (C) 2006-2009 Catalyst IT Ltd http://catalyst.net.nz
  *
  */
 
@@ -79,6 +80,18 @@ class PluginArtefactResume extends Plugin {
         );
     }
 
+    public static function get_artefact_type_content_types() {
+        return array(
+            'coverletter'   => array('text'),
+            'interest'      => array('text'),
+            'personalgoal'  => array('text'),
+            'academicgoal'  => array('text'),
+            'careergoal'    => array('text'),
+            'personalskill' => array('text'),
+            'academicskill' => array('text'),
+            'workskill'     => array('text'),
+        );
+    }
 }
 
 class ArtefactTypeResume extends ArtefactType {
@@ -115,7 +128,7 @@ class ArtefactTypeResume extends ArtefactType {
      * Default render method for resume fields - show their description
      */
     public function render_self($options) {
-        return array('html' => $this->description);
+        return array('html' => clean_html($this->description));
     }
 
     /**
@@ -225,7 +238,7 @@ class ArtefactTypeContactinformation extends ArtefactTypeResume {
 
 class ArtefactTypePersonalinformation extends ArtefactTypeResume {
     
-    private $composites;
+    protected $composites;
 
     public function __construct($id=0, $data=null) {
         if (empty($id)) {
@@ -316,6 +329,14 @@ class ArtefactTypePersonalinformation extends ArtefactTypeResume {
         return array('html' => $smarty->fetch('artefact:resume:fragments/personalinformation.tpl'));
     }
 
+    public function delete() {
+        db_begin();
+
+        delete_records('artefact_resume_personal_information', 'artefact', $this->id);
+        parent::delete();
+
+        db_commit();
+    }
 }
 
 
@@ -347,6 +368,14 @@ abstract class ArtefactTypeResumeComposite extends ArtefactTypeResume {
 
     public static abstract function get_tablerenderer_body_js_string();
 
+    /**
+     * Can be overridden to format data retrieved from artefact tables for 
+     * display of the resume artefact by render_self
+     */
+    public static function format_render_self_data($data) {
+        return $data;
+    }
+
     /** 
     * This function should return an array suitable to 
     * put into the 'elements' part of a pieform array
@@ -359,17 +388,27 @@ abstract class ArtefactTypeResumeComposite extends ArtefactTypeResume {
     * @throws Exception
     */
     public static function process_compositeform(Pieform $form, $values) {
+        global $USER;
+        self::ensure_composite_value($values, $values['compositetype'], $USER->get('id'));
+    }
 
+    /**
+     * Ensures that the given value for the given composite is present
+     * TODO: expand on these docs.
+     */
+    public static function ensure_composite_value($values, $compositetype, $owner) {
+        if (!in_array($compositetype, self::get_composite_artefact_types())) {
+            throw new SystemException("ensure_composite_value called with invalid composite type");
+        }
         try {
-            $a = artefact_instance_from_type($values['compositetype']);
+            $a = artefact_instance_from_type($compositetype, $owner);
             $a->set('mtime', time());
         }
         catch (Exception $e) {
-            global $USER;
-            $classname = generate_artefact_class_name($values['compositetype']);
+            $classname = generate_artefact_class_name($compositetype);
             $a = new $classname(0, array(
-                'owner' => $USER->get('id'),
-                'title' => get_string($values['compositetype'], 'artefact.resume'),
+                'owner' => $owner,
+                'title' => get_string($compositetype, 'artefact.resume'),
                 )
             );
         }
@@ -378,13 +417,18 @@ abstract class ArtefactTypeResumeComposite extends ArtefactTypeResume {
 
         $values['artefact'] = $a->get('id');
 
-        $table = 'artefact_resume_' . $values['compositetype'];
+        $table = 'artefact_resume_' . $compositetype;
         if (!empty($values['id'])) {
             update_record($table, (object)$values, 'id');
         }
         else {
-            $max = get_field($table, 'MAX(displayorder)', 'artefact', $values['artefact']);
-            $values['displayorder'] = is_numeric($max) ? $max + 1 : 0;
+            if (isset($values['displayorder'])) {
+                $values['displayorder'] = intval($values['displayorder']);
+            }
+            else {
+                $max = get_field($table, 'MAX(displayorder)', 'artefact', $values['artefact']);
+                $values['displayorder'] = is_numeric($max) ? $max + 1 : 0;
+            }
             insert_record($table, (object)$values);
         }
     }
@@ -453,34 +497,42 @@ abstract class ArtefactTypeResumeComposite extends ArtefactTypeResume {
     }
 
     public function render_self($options) {
+        global $USER;
         $suffix = '_' . substr(md5(microtime()), 0, 4);
         $smarty = smarty_core();
         $smarty->assign('hidetitle', true);
         $smarty->assign('suffix', $suffix);
         $type = $this->get('artefacttype');
+        $othertable = 'artefact_resume_' . $type;
+        $owner = $USER->get('id');
+
+        $sql = 'SELECT ar.*, a.owner
+            FROM {artefact} a 
+            JOIN {' . $othertable . '} ar ON ar.artefact = a.id
+            WHERE a.owner = ? AND a.artefacttype = ?
+            ORDER BY ar.displayorder';
+
+        if (!empty($options['viewid'])) { 
+            if (!can_view_view($options['viewid'])) {
+                throw new AccessDeniedException();
+            }
+            require_once('view.php');
+            $v = new View($options['viewid']);
+            $owner = $v->get('owner');
+        }
+
+        if (!$data = get_records_sql_array($sql, array($owner, $type))) {
+            $data = array();
+        }
+
+        // Give the artefact type a chance to format the data how it sees fit
+        $data = call_static_method(generate_artefact_class_name($type), 'format_render_self_data', $data);
+        $smarty->assign('rows', $data);
+
         $content = array(
             'html'         => $smarty->fetch('artefact:resume:fragments/' . $type . '.tpl'),
-            'javascript'   =>
-                $this->get_showhide_composite_js()
-                ."
-                var {$type}list{$suffix} = new TableRenderer(
-                   '{$type}list{$suffix}',
-                   '" . get_config('wwwroot') . "artefact/resume/composite.json.php',
-                   [
-                   " . call_static_method(generate_artefact_class_name($type), 'get_tablerenderer_js') ."
-                   ]
-                );
-
-                {$type}list{$suffix}.type = '{$type}';
-                {$type}list{$suffix}.statevars.push('type');
-                " .
-                (( array_key_exists('viewid', $options))
-                    ? "{$type}list{$suffix}.view = " . $options['viewid'] . ";
-                       {$type}list{$suffix}.statevars.push('view');"
-                    : ""
-                ) . "
-                {$type}list{$suffix}.updateOnLoad();
-            ");
+            'javascript'   => $this->get_showhide_composite_js()
+        );
         return $content;
     }
 
@@ -544,14 +596,13 @@ class ArtefactTypeEmploymenthistory extends ArtefactTypeResumeComposite {
     }
 
     public static function get_tablerenderer_title_js_string() {
-        $at = get_string('at');
-        return " r.jobtitle + ' {$at} ' + r.employer";
+        return " r.jobtitle + ' : ' + r.employer";
     }
 
     public static function get_tablerenderer_body_js_string() {
         return " r.positiondescription";
     }
-    
+
     public static function get_addform_elements() {
         return array(
             'startdate' => array(
@@ -615,6 +666,14 @@ class ArtefactTypeEducationhistory extends ArtefactTypeResumeComposite {
         return " r.qualname + ' (' + r.qualtype + ') {$at} ' + r.institution";
     }
 
+    public static function format_render_self_data($data) {
+        $at = get_string('at');
+        foreach ($data as &$row) {
+            $row->qualification = $row->qualname . ' (' . $row->qualtype . ")  $at " . $row->institution;
+        }
+        return $data;
+    }
+
     public static function get_tablerenderer_body_js_string() {
         return " r.qualdescription"; 
     }
@@ -767,7 +826,7 @@ class ArtefactTypeBook extends ArtefactTypeResumeComposite {
                 'rows' => 10,
                 'cols' => 50,
                 'resizable' => false,
-                'title' => get_string('description', 'artefact.resume'),
+                'title' => get_string('detailsofyourcontribution', 'artefact.resume'),
             ),
         );
     }
