@@ -1,7 +1,8 @@
 <?php
 /**
  * Mahara: Electronic portfolio, weblog, resume builder and social networking
- * Copyright (C) 2006-2008 Catalyst IT Ltd (http://www.catalyst.net.nz)
+ * Copyright (C) 2006-2009 Catalyst IT Ltd and others; see:
+ *                         http://wiki.mahara.org/Contributors
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +21,7 @@
  * @subpackage auth
  * @author     Catalyst IT Ltd
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL
- * @copyright  (C) 2006-2008 Catalyst IT Ltd http://catalyst.net.nz
+ * @copyright  (C) 2006-2009 Catalyst IT Ltd http://catalyst.net.nz
  *
  */
 
@@ -34,6 +35,16 @@ require(get_config('docroot') . 'auth/user.php');
  */
 class AuthUnknownUserException extends UserException {}
 
+/**
+ * An instance of an auth plugin failed during execution 
+ * e.g. LDAP auth failed to connect to a directory
+ * Developers can use this to fail an individual auth
+ * instance, but not kill all from being tried.
+ * If appropriate - the 'message' of the exception will be used
+ * as the display message, so don't forget to language translate it
+ */
+class AuthInstanceException extends UserException {}
+
 /**
  * We tried to call a method on an auth plugin that hasn't been init'ed 
  * successfully
@@ -128,9 +139,8 @@ abstract class Auth {
         if (in_array($name, $approved_members)) {
             return $this->{$name};
         }
-
-        if (isset($this->config['name'])) {
-            return $this->config['name'];
+        if (isset($this->config[$name])) {
+            return $this->config[$name];
         }
         return null;
     }
@@ -243,36 +253,6 @@ abstract class Auth {
         return false;
     }
 
-    /**
-     * Given a username, return information about the user from the database.
-     * This object must_be_ready, which means it will have an authinstanceid. 
-     * This is used to disambiguate between users with the same username.
-     *
-     * The information retrieved must be all rows in the user table, with the
-     * timestamps formatted as unix timestamps. An example (taken from the
-     * internal authentication mechanism, which allows usernames to be case
-     * insensitive):
-     *
-     * <code>
-     * get_record('usr', 'LOWER(username)', strtolower($username), null, null, null, null,
-     *    '*, ' . db_format_tsfield('expiry') . ', ' . db_format_tsfield('lastlogin'));
-     * </code>
-     *
-     * @param string $username The username to get information for
-     * @return array           Data that can be used to populate the session
-     * @throws AuthUnknownUserException If the user is unknown to the
-     *                                  authentication method
-     */
-    public function get_user_info_cached($username) {
-        $this->must_be_ready();
-        if (!$result = get_record('usr', 'LOWER(username)', strtolower($username), null, null, null, null,
-                    '*, ' . db_format_tsfield('expiry') . ', ' . db_format_tsfield('lastlogin'))) {
-            throw new AuthUnknownUserException("\"$username\" is not known to AuthInternal");
-        }
-        $cache[$result->username] = $result;
-        return $result;
-    }
-
     /**
      * Given a password, returns whether it is in a valid format for this
      * authentication method.
@@ -296,6 +276,18 @@ abstract class Auth {
         return true;
     }
 
+    /**
+     * Called when a user is being logged in, after the main authentication routines.
+     *
+     * You can use $USER->login() to perform any additional tasks, for example
+     * to set a cookie that another application can read, or pull some data
+     * from somewhere.
+     *
+     * This method has no parameters and needs no return value
+     */
+    public function login() {
+    }
+
     /**
      * Called when a user is being logged out, either by clicking a logout 
      * link, their session timing out or some other method where their session 
@@ -355,6 +347,27 @@ function auth_setup () {
         return;
     }
 
+    // Lock the site until core upgrades are done
+    require(get_config('libroot') . 'version.php');
+    $siteclosed    = $config->version > get_config('version');
+    $disablelogin  = $config->disablelogin;
+
+    if (!$siteclosed && get_config('forcelocalupgrades')) {
+        require(get_config('docroot') . 'local/version.php');
+        $siteclosed = $config->version > get_config('localversion');
+    }
+
+    $cfgsiteclosed = get_config('siteclosed');
+    if ($siteclosed && !$cfgsiteclosed || !$siteclosed && $cfgsiteclosed) {
+        // If the admin closed the site manually, open it automatically
+        // when an upgrade is successful.
+        if ($cfgsiteclosed && get_config('siteclosedbyadmin')) {
+            set_config('siteclosedbyadmin', false);
+        }
+        set_config('siteclosed', $siteclosed);
+        set_config('disablelogin', $disablelogin);
+    }
+
     // Check the time that the session is set to log out. If the user does
     // not have a session, this time will be 0.
     $sessionlogouttime = $USER->get('logout_time');
@@ -392,20 +405,13 @@ function auth_setup () {
                 redirect();
             }
         } else if (defined('INSTITUTIONALADMIN') && !$USER->get('admin')) {
-            $userreallyadminfor = get_column('usr_institution', 'institution', 'usr', $USER->id, 'admin', 1);
-            if (!$USER->is_institutional_admin() && !empty($userreallyadminfor)) {
-                $USER->set_admin_institutions($userreallyadminfor);
-            }
-            else if ($USER->is_institutional_admin() && empty($userreallyadminfor)) {
-                $USER->set_admin_institutions(array());
-            }
+            $USER->reset_institutions();
             if (!$USER->is_institutional_admin()) {
                 $SESSION->add_error_msg(get_string('accessforbiddentoadminsection'));
                 redirect();
             }
         }
         $USER->renew();
-        auth_check_password_change();
         auth_check_required_fields();
     }
     else if ($sessionlogouttime > 0) {
@@ -539,7 +545,8 @@ function auth_get_auth_instances_for_institutions($institutions) {
             i.id,
             inst.name,
             inst.displayname,
-            i.instancename
+            i.instancename,
+            i.authname
         FROM 
             {institution} inst,
             {auth_instance} i
@@ -629,53 +636,6 @@ function auth_get_auth_instances_for_wwwroot($wwwroot) {
     return get_records_sql_array($query, array('value' => $wwwroot));
 }
 
-/**
- * Given an institution, returns the authentication methods used by it, sorted 
- * by priority.
- *
- * @param  string   $institution     Name of the institution
- * @return array                     Array of auth instance records
- */
-function auth_get_auth_instances_for_username($institution, $username) {
-    static $cache = array();
-
-    if (!isset($cache[$institution][$username])) {
-        // Get auth instances in order of priority
-        // DO NOT CHANGE THE SORT ORDER OF THIS RESULT SET
-        // YEAH EINSTEIN - THAT MEANS YOU!!!
-
-        // TODO: work out why this won't accept a placeholder - had to use db_quote
-        $sql ='
-            SELECT DISTINCT
-                i.id,
-                i.instancename,
-                i.priority,
-                i.authname,
-                a.requires_config,
-                a.requires_parent
-            FROM 
-                {auth_instance} i,
-                {auth_installed} a,
-                {usr} u
-            WHERE 
-                a.name = i.authname AND
-                i.institution = ? AND
-                u.username = ? AND
-                u.institution = i.institution
-            ORDER BY
-                i.priority,
-                i.instancename';
-
-        $cache[$institution][$username] = get_records_sql_array($sql, array(array('institution' => $institution), array('username' => $username)));
-
-        if (empty($cache[$institution])) {
-            return false;
-        }
-    }
-
-    return $cache[$institution];
-}
-
 /**
  * Given an institution, get all the auth types EXCEPT those that are already 
  * enabled AND do not require configuration.
@@ -734,20 +694,104 @@ function auth_get_available_auth_types($institution=null) {
 }
 /**
  * Checks that all the required fields are set, and handles setting them if required.
+ *
+ * Checks whether the current user needs to change their password, and handles
+ * the password changing if it's required.
  */
 function auth_check_required_fields() {
-    global $USER;
-    safe_require('artefact', 'internal');
-    require_once('pieforms/pieform.php');
+    global $USER, $SESSION;
+
+    $changepassword = true;
     $elements = array();
 
+    if (
+        !$USER->get('passwordchange')                                // User doesn't need to change their password
+        || ($USER->get('parentuser') && $USER->get('loginanyway'))   // User is masquerading and wants to log in anyway
+        || defined('NOCHECKPASSWORDCHANGE')                          // The page wants to skip this hassle
+        ) {
+        $changepassword = false;
+    }
+
+    // Check if the user wants to log in anyway
+    if ($USER->get('passwordchange') && $USER->get('parentuser') && isset($_GET['loginanyway'])) {
+        $USER->loginanyway = true;
+        $changepassword = false;
+    }
+
+    if ($changepassword) {
+        $authobj = AuthFactory::create($USER->authinstance);
+
+        if ($authobj->changepasswordurl) {
+            redirect($authobj->changepasswordurl);
+            exit;
+        }
+
+        if (method_exists($authobj, 'change_password')) {
+
+            if ($SESSION->get('resetusername')) {
+                $elements['username'] = array(
+                    'type' => 'text',
+                    'defaultvalue' => $USER->get('username'),
+                    'title' => get_string('changeusername', 'account'),
+                    'description' => get_string('changeusernamedesc', 'account', hsc(get_config('sitename'))),
+                );
+            }
+
+            $elements['password1'] = array(
+                'type'        => 'password',
+                'title'       => get_string('newpassword') . ':',
+                'description' => get_string('yournewpassword'),
+                'rules'       => array(
+                    'required' => true
+                )
+            );
+
+            $elements['password2'] = array(
+                'type'        => 'password',
+                'title'       => get_string('confirmpassword') . ':',
+                'description' => get_string('yournewpasswordagain'),
+                'rules'       => array(
+                    'required' => true,
+                ),
+            );
+
+            $elements['email'] = array(
+                'type'   => 'text',
+                'title'  => get_string('principalemailaddress', 'artefact.internal'),
+                'ignore' => (trim($USER->get('email')) != '' && !preg_match('/@example\.org$/', $USER->get('email'))),
+                'rules'  => array(
+                    'required' => true,
+                    'email'    => true,
+                ),
+            );
+        }
+    }
+    else if (defined('JSON')) {
+        // Don't need to check this for json requests
+        return;
+    }
+
+    safe_require('artefact', 'internal');
+    require_once('pieforms/pieform.php');
+
+    $alwaysmandatoryfields = array_keys(ArtefactTypeProfile::get_always_mandatory_fields());
     foreach(ArtefactTypeProfile::get_mandatory_fields() as $field => $type) {
+        // Always mandatory fields are stored in the usr table, so are part of 
+        // the user session object. We can save a query by grabbing them from 
+        // the session.
+        if (in_array($field, $alwaysmandatoryfields) && $USER->get($field) != null) {
+            continue;
+        }
+        // Not cached? Get value the standard way.
         if (get_profile_field($USER->get('id'), $field) != null) {
             continue;
         }
 
         if ($field == 'email') {
-            // Use a text field for their first e-mail address, not the 
+            if (isset($elements['email'])) {
+                continue;
+            }
+            // Use a text field for their first e-mail address, not the
             // emaillist element
             $type = 'text';
         }
@@ -794,119 +838,24 @@ function auth_check_required_fields() {
     ));
 
     $smarty = smarty();
+    if ($USER->get('parentuser')) {
+        $smarty->assign('loginasoverridepasswordchange',
+            get_string('loginasoverridepasswordchange', 'admin',
+                       '<a href="' . get_config('wwwroot') . '?loginanyway">', '</a>'));
+    }
+    $smarty->assign('changepassword', $changepassword);
+    $smarty->assign('changeusername', $SESSION->get('resetusername'));
     $smarty->assign('form', $form);
     $smarty->display('requiredfields.tpl');
     exit;
 }
 
-function requiredfields_submit(Pieform $form, $values) {
-    global $USER, $SESSION;
-    foreach ($values as $field => $value) {
-        if (in_array($field, array('submit', 'sesskey'))) {
-            continue;
-        }
-        set_profile_field($USER->get('id'), $field, $value);
-    }
-    $SESSION->add_ok_msg(get_string('requiredfieldsset', 'auth'));
-    redirect();
-}
-
-/**
- * Checks whether the current user needs to change their password, and handles
- * the password changing if it's required.
- *
- * This only applies for the internal authentication plugin. Other plugins
- * will, in theory, have different data stores, making changing the password
- * via the internal form difficult.
- */
-function auth_check_password_change() {
+function requiredfields_validate(Pieform $form, $values) {
     global $USER;
-    if (
-        !$USER->get('passwordchange')                                // User doesn't need to change their password
-        || ($USER->get('parentuser') && $USER->get('loginanyway'))   // User is masquerading and wants to log in anyway
-        || defined('NOCHECKPASSWORDCHANGE')                          // The page wants to skip this hassle
-        ) {
-        return;
+    if (!isset($values['password1'])) {
+        return true;
     }
 
-    // Check if the user wants to log in anyway
-    if ($USER->get('passwordchange') && $USER->get('parentuser') && isset($_GET['loginanyway'])) {
-        $USER->loginanyway = true;
-        return;
-    }
-
-    $authobj = AuthFactory::create($USER->authinstance);
-
-    if ($authobj->changepasswordurl) {
-        redirect($authobj->changepasswordurl);
-        exit;
-    }
-
-    // @todo auth preference for a password change screen for all auth methods other than internal
-    if (method_exists($authobj, 'change_password')) {
-
-        require_once('pieforms/pieform.php');
-        $form = array(
-            'name'       => 'change_password',
-            'method'     => 'post',
-            'plugintype' => 'auth',
-            'pluginname' => 'internal',
-            'elements'   => array(
-                'password1' => array(
-                    'type'        => 'password',
-                    'title'       => get_string('newpassword') . ':',
-                    'description' => get_string('yournewpassword'),
-                    'rules'       => array(
-                        'required' => true
-                    )
-                ),
-                'password2' => array(
-                    'type'        => 'password',
-                    'title'       => get_string('confirmpassword') . ':',
-                    'description' => get_string('yournewpasswordagain'),
-                    'rules'       => array(
-                        'required' => true,
-                    ),
-                ),
-                'email' => array(
-                    'type'   => 'text',
-                    'title'  => get_string('principalemailaddress', 'artefact.internal'),
-                    'ignore' => (trim($USER->get('email')) != '' && !preg_match('/@example\.org$/', $USER->get('email'))),
-                    'rules'  => array(
-                        'required' => true,
-                        'email'    => true,
-                    ),
-                ),
-                'submit' => array(
-                    'type'  => 'submit',
-                    'value' => get_string('changepassword'),
-                ),
-            )
-        );
-
-        $smarty = smarty();
-        $smarty->assign('change_password_form', pieform($form));
-        if ($USER->get('parentuser')) {
-            $smarty->assign('loginasoverridepasswordchange',
-                get_string('loginasoverridepasswordchange', 'admin',
-                    '<a href="' . get_config('wwwroot') . '?loginanyway">', '</a>'));
-        }
-        $smarty->display('change_password.tpl');
-        exit;
-    }
-}
-
-/**
- * Validates the form for changing the password for a user.
- *
- * Change password will only be if a URL for it exists, or a function exists.
- *
- * @param Pieform  $form   The form to check
- * @param array    $values The values to check
- */
-function change_password_validate(Pieform $form, $values) {
-    global $USER;
-
     // Get the authentication type for the user, and
     // use the information to validate the password
     $authobj = AuthFactory::create($USER->authinstance);
@@ -915,37 +864,72 @@ function change_password_validate(Pieform $form, $values) {
     password_validate($form, $values, $USER);
 
     // The password cannot be the same as the old one
-    if (!$form->get_error('password1')
-        && $authobj->authenticate_user_account($USER, $values['password1'])) {
-        $form->set_error('password1', get_string('passwordnotchanged'));
+    try {
+        if (!$form->get_error('password1')
+            && $authobj->authenticate_user_account($USER, $values['password1'])) {
+            $form->set_error('password1', get_string('passwordnotchanged'));
+        }
+    }
+    // propagate error up as the collective error AuthUnknownUserException
+     catch  (AuthInstanceException $e) {
+        $form->set_error('password1', $e->getMessage());
+    }
+
+
+    if ($authobj->authname == 'internal' && isset($values['username']) && $values['username'] != $USER->get('username')) {
+        if (!AuthInternal::is_username_valid($values['username'])) {
+            $form->set_error('username', get_string('usernameinvalidform', 'auth.internal'));
+        }
+        if (!$form->get_error('username') && record_exists_select('usr', 'LOWER(username) = ?', strtolower($values['username']))) {
+            $form->set_error('username', get_string('usernamealreadytaken', 'auth.internal'));
+        }
     }
 }
 
-/**
- * Changes the password for a user, given that it is valid.
- *
- * @param array $values The submitted form values
- */
-function change_password_submit(Pieform $form, $values) {
+function requiredfields_submit(Pieform $form, $values) {
     global $USER, $SESSION;
 
-    $authobj = AuthFactory::create($USER->authinstance);
+    if (isset($values['password1'])) {
+        $authobj = AuthFactory::create($USER->authinstance);
 
-    // This method should exist, because if it did not then the change
-    // password form would not have been shown.
-    if ($password = $authobj->change_password($USER, $values['password1'])) {
-        $SESSION->add_ok_msg(get_string('passwordsaved'));
-        if (!empty($values['email'])) {
-            $USER->email = $values['email'];
-            $USER->commit();
-            set_profile_field($USER->id, 'email', $values['email']);
+        // This method should exist, because if it did not then the change
+        // password form would not have been shown.
+        if ($password = $authobj->change_password($USER, $values['password1'])) {
+            $SESSION->add_ok_msg(get_string('passwordsaved'));
+        }
+        else {
+            // TODO: Exception is the wrong type here!
+            throw new Exception('Attempt by "' . $USER->get('username') . '@'
+                . $USER->get('institution') . 'to change their password failed');
         }
-        redirect();
     }
 
-    // TODO: Exception is the wrong type here!
-    throw new Exception('Attempt by "' . $USER->get('username') . '@'
-        . $USER->get('institution') . 'to change their password failed');
+    if (isset($values['username'])) {
+        $SESSION->set('resetusername', false);
+        if ($values['username'] != $USER->get('username')) {
+            $USER->username = $values['username'];
+            $USER->commit();
+            $otherfield = true;
+        }
+    }
+
+    foreach ($values as $field => $value) {
+        if (in_array($field, array('submit', 'sesskey', 'password1', 'password2', 'username'))) {
+            continue;
+        }
+        if ($field == 'email') {
+            $USER->email = $values['email'];
+            $USER->commit();
+        }
+        set_profile_field($USER->get('id'), $field, $value);
+        $otherfield = true;
+    }
+
+    if (isset($otherfield)) {
+        $SESSION->add_ok_msg(get_string('requiredfieldsset', 'auth'));
+    }
+
+    redirect();
 }
 
 /**
@@ -985,9 +969,9 @@ function auth_draw_login_page($message=null, Pieform $form=null) {
     if ($message) {
         $SESSION->add_info_msg($message);
     }
-    $smarty = smarty(array(), array(), array(), array('pagehelp' => false));
+    $smarty = smarty(array(), array(), array(), array('pagehelp' => false, 'sidebars' => false));
     $smarty->assign('login_form', $loginform);
-    $smarty->assign('loginmessage', get_string('loginto', 'mahara', get_config('sitename')));
+    $smarty->assign('PAGEHEADING', hsc(get_string('loginto', 'mahara', get_config('sitename'))));
     $smarty->display('login.tpl');
     exit;
 }
@@ -999,8 +983,6 @@ function auth_draw_login_page($message=null, Pieform $form=null) {
  * @access private
  */
 function auth_get_login_form() {
-    $institutions = get_records_menu('institution', '', '', 'name, displayname');
-
     $elements = array(
         'login_username' => array(
             'type'        => 'text',
@@ -1014,10 +996,7 @@ function auth_get_login_form() {
             'type'        => 'password',
             'title'       => get_string('password') . ':',
             'description' => get_string('passworddescription'),
-            'value'       => '',
-            'rules' => array(
-                'required'    => true
-            )
+            'defaultvalue'       => '',
         ),
         'submit' => array(
             'type'  => 'submit',
@@ -1033,15 +1012,20 @@ function auth_get_login_form() {
     // remembers the GET and POST data sent to it and resends that on
     // afterwards. 
     $action = '';
+    if (get_config('httpswwwroot')) {
+        $action = rtrim(get_config('httpswwwroot'), '/') . hsc(strip_querystring(get_relative_script_path()));
+    }
     if ($_GET) {
         if (isset($_GET['logout'])) {
             // You can log the user out on any particular page by appending
             // ?logout to the URL. In this case, we don't want the "action"
             // of the url to include that, or be blank, else the next time
             // the user logs in they will be logged out again.
-            $action = hsc(substr($_SERVER['REQUEST_URI'], 0, strpos($_SERVER['REQUEST_URI'], '?')));
+            if ($action == '') {
+                $action = hsc(substr($_SERVER['REQUEST_URI'], 0, strpos($_SERVER['REQUEST_URI'], '?')));
+            }
         } else {
-            $action = '?';
+            $action .= '?';
             foreach ($_GET as $key => $value) {
                 if ($key != 'logout' && $key != 'login') {
                     $action .= hsc($key) . '=' . hsc($value) . '&amp;';
@@ -1085,15 +1069,6 @@ function auth_get_login_form() {
  * @private
  */
 function get_login_form_js($form) {
-    // Add the autofocus class to the username input. This is because Mahara 
-    // doesn't always render the login form when it is built. Sometimes, the 
-    // form is built to process a login attempt, and then built again when that 
-    // attempt fails. This makes sure that every time the login form is 
-    // actually displayed it has the autofocus class.
-    $form = str_replace(
-        'class="required text" id="login_login_username"',
-        'class="required text autofocus" id="login_login_username"',
-        $form);
     $form = str_replace('/', '\/', str_replace("'", "\'", (str_replace(array("\n", "\t"), '', $form))));
     $strcookiesnotenabled    = json_encode(get_string('cookiesnotenabled'));
     $cookiename = get_config('cookieprefix') . 'ctest';
@@ -1128,6 +1103,9 @@ class AuthFactory {
      *                          instance doesn't exist (Should never happen)
      */
     public static function create($id) {
+        if (is_object($id) && isset($id->id)) {
+            $id = $id->id;
+        }
 
         if (isset(self::$authcache[$id]) && is_object(self::$authcache[$id])) {
             return self::$authcache[$id];
@@ -1177,10 +1155,11 @@ function login_submit(Pieform $form, $values) {
         try {
             // Reset the LiveUser object, since we are attempting to create a 
             // new user
+            $SESSION->destroy_session();
             $USER = new LiveUser();
 
             $authinstances = get_records_sql_array('
-                SELECT a.id, a.instancename, a.priority, a.authname, a.institution
+                SELECT a.id, a.instancename, a.priority, a.authname, a.institution, i.suspended, i.displayname
                 FROM {institution} i JOIN {auth_instance} a ON a.institution = i.name
                 ORDER BY a.institution, a.priority, a.instancename', null);
 
@@ -1196,10 +1175,16 @@ function login_submit(Pieform $form, $values) {
                 if (!$auth->can_auto_create_users()) {
                     continue;
                 }
-                if ($auth->authenticate_user_account($USER, $password)) {
-                    $authenticated = true;
-                }
-                else {
+                // catch semi-fatal auth errors, but allow next auth instance to be
+                // tried
+                try {
+                    if ($auth->authenticate_user_account($USER, $password)) {
+                        $authenticated = true;
+                    }
+                    else {
+                        continue;
+                    }
+                } catch  (AuthInstanceException $e) {
                     continue;
                 }
 
@@ -1216,6 +1201,13 @@ function login_submit(Pieform $form, $values) {
                 if (empty($userdata)) {
                     throw new AuthUnknownUserException("\"$username\" is not known");
                 }
+
+                // Check for a suspended institution
+                if ($authinstance->suspended) {
+                    $sitename = get_config('sitename');
+                    throw new AccessTotallyDeniedException(get_string('accesstotallydenied_institutionsuspended', 'mahara', $authinstance->displayname, $sitename));
+                }
+
                 // We have the data - create the user
                 $USER->lastlogin = db_format_timestamp(time());
                 if (isset($userdata->firstname)) {
@@ -1232,23 +1224,10 @@ function login_submit(Pieform $form, $values) {
                     $USER->email = null;
                 }
                 try {
-                    db_begin();
-                    $USER->commit();
-                    if (isset($userdata->firstname)) {
-                        set_profile_field($USER->id, 'firstname', $USER->firstname);
-                    }
-                    if (isset($userdata->lastname)) {
-                        set_profile_field($USER->id, 'lastname', $USER->lastname);
-                    }
-                    if (isset($userdata->email)) {
-                        set_profile_field($USER->id, 'email', $USER->email);
-                    }
-                    if ($authinstance->institution !== 'mahara') {
-                        $USER->join_institution($authinstance->institution);
-                    }
-
-                    handle_event('createuser', $USER->to_stdclass());
-                    db_commit();
+                    // If this authinstance is a parent auth for some xmlrpc authinstance, pass it along to create_user
+                    // so that this username also gets recorded as the username for sso from the remote sites.
+                    $remoteauth = count_records('auth_instance_config', 'field', 'parent', 'value', $authinstance->id) ? $authinstance : null;
+                    create_user($USER, array(), $institution, $remoteauth);
                     $USER->reanimate($USER->id, $authinstance->id);
                 }
                 catch (Exception $e) {
@@ -1310,8 +1289,20 @@ function login_submit(Pieform $form, $values) {
 
     // User is allowed to log in
     //$USER->login($userdata);
-    auth_check_password_change();
     auth_check_required_fields();
+
+    if (get_config('httpswwwroot') && !defined('JSON')) {
+        // If we are using HTTPS for logins we need to go back to
+        // non-HTTPS URLs. Otherwise, Javascript (and possibly CSS)
+        // breaks. Don't use get_full_script_path(), as it doesn't
+        // work if someone sets httpswwwroot to something like
+        // 'https://x.y.z.w:443/...'  (unlikely, but
+        // possible). get_full_script_path() doesn't gives us the
+        // ':443' part and things break horribly.
+        $parts = parse_url(get_config('httpswwwroot'));
+        $httpsrequest = rtrim($parts['path'], '/');
+        redirect(hsc(substr(get_script_path(), strlen($httpsrequest))));
+    }
 }
 
 /**
@@ -1416,10 +1407,10 @@ function auth_handle_account_expiries() {
         foreach ($users as $user) {
             $displayname  = display_name($user);
             email_user($user, null,
-                get_string('institutionexpirywarning'),
-                get_string('institutionexpirywarningtext', 'mahara', $displayname, $user->institutionname,
+                get_string('institutionmembershipexpirywarning'),
+                get_string('institutionmembershipexpirywarningtext', 'mahara', $displayname, $user->institutionname,
                            $sitename, $daystoexpire, $wwwroot . 'contact.php', $sitename),
-                get_string('institutionexpirywarninghtml', 'mahara', $displayname, $user->institutionname,
+                get_string('institutionmembershipexpirywarninghtml', 'mahara', $displayname, $user->institutionname,
                            $sitename, $daystoexpire, $wwwroot . 'contact.php', $sitename)
             );
             set_field('usr_institution', 'expirymailsent', 1, 'usr', $user->id,
@@ -1429,6 +1420,78 @@ function auth_handle_account_expiries() {
 
 }
 
+/**
+ * Sends notification e-mails to site and institutional admins when:
+ *
+ *  - An institution is expiring within the institution expiry warning
+ *    period, set in site options.
+ *
+ * The actual prevention of users logging in is handled by the authentication
+ * code. This cron job sends e-mails to notify users that these events will
+ * happen soon.
+ */
+function auth_handle_institution_expiries() {
+    // The 'expiry' flag on the usr table
+    $sitename = get_config('sitename');
+    $wwwroot  = get_config('wwwroot');
+    $expire   = get_config('institutionautosuspend');
+    $warn     = get_config('institutionexpirynotification');
+
+    $daystoexpire = ceil($warn / 86400) . ' ';
+    $daystoexpire .= ($daystoexpire == 1) ? get_string('day') : get_string('days');
+
+    // Get site administrators
+    $siteadmins = get_records_sql_array('SELECT u.id, u.username, u.firstname, u.lastname, u.preferredname, u.email, u.admin, u.staff FROM {usr} u WHERE u.admin = 1', array());
+
+    // Expiry warning messages
+    if ($institutions = get_records_sql_array(
+      'SELECT i.name, i.displayname FROM {institution} i ' .
+      'WHERE ' . db_format_tsfield('i.expiry', false) . ' < ? AND suspended != 1 AND expirymailsent != 1',
+      array(time() + $warn))) {
+        foreach ($institutions as $institution) {
+            $institution_displayname = $institution->displayname;
+            // Email site administrators
+            foreach ($siteadmins as $user) {
+                $user_displayname  = display_name($user);
+                email_user($user, null,
+                    get_string('institutionexpirywarning'),
+                    get_string('institutionexpirywarningtext_site', 'mahara', $user_displayname, $institution_displayname, $daystoexpire, $sitename, $sitename),
+                    get_string('institutionexpirywarninghtml_site', 'mahara', $user_displayname, $institution_displayname, $daystoexpire, $sitename, $sitename)
+                );
+            }
+
+            // Email institutional administrators
+            $institutionaladmins = get_records_sql_array(
+              'SELECT u.id, u.username, u.expiry, u.staff, u.admin AS siteadmin, ui.admin AS institutionadmin, u.firstname, u.lastname, u.email ' .
+              'FROM {usr_institution} ui JOIN {usr} u ON (ui.usr = u.id) WHERE ui.admin = 1', array()
+            );
+            foreach ($institutionaladmins as $user) {
+                $user_displayname  = display_name($user);
+                email_user($user, null,
+                    get_string('institutionexpirywarning'),
+                    get_string('institutionexpirywarningtext_institution', 'mahara', $user_displayname, $institution_displayname, $sitename, $daystoexpire, $wwwroot . 'contact.php', $sitename),
+                    get_string('institutionexpirywarninghtml_institution', 'mahara', $user_displayname, $institution_displayname, $sitename, $daystoexpire, $wwwroot . 'contact.php', $sitename)
+                );
+            }
+            set_field('institution', 'expirymailsent', 1, 'name', $institution->name);
+        }
+    }
+
+    // If we can automatically suspend expired institutions
+    $autosuspend = get_config('institutionautosuspend');
+    if ($autosuspend) {
+        // Actual expired institutions
+        if ($institutions = get_records_sql_array(
+          'SELECT name FROM {institution} ' .
+          'WHERE ' . db_format_tsfield('expiry', false) . ' < ?', array(time()))) {
+            // Institutions have expired!
+            foreach ($institutions as $institution) {
+                set_field('institution', 'suspended', 1, 'name', $institution->name);
+            }
+        }
+    }
+}
+
 /**
  * Clears out old session files
  *
@@ -1442,22 +1505,41 @@ function auth_remove_old_session_files() {
     exec('find ' . escapeshellarg(get_config('dataroot') . 'sessions') . ' -type f -mtime +1 | xargs -n 1000 -r rm');
 }
 
+/**
+ * Generates the login form for the sideblock
+ *
+ * {@internal{Not sure why this form definition doesn't use 
+ * auth_get_login_form, but keep that in mind when making changes.}}
+ */
 function auth_generate_login_form() {
     if (!get_config('installed')) {
         return;
     }
+    $action='';
+    if (get_config('httpswwwroot')) {
+        $action = rtrim(get_config('httpswwwroot'), '/') . hsc(strip_querystring(get_relative_script_path()));
+    }
     require_once('pieforms/pieform.php');
+    if (count_records('institution', 'registerallowed', 1, 'suspended', 0)) {
+        $registerlink = '<a href="' . get_config('wwwroot') . 'register.php" tabindex="2">' . get_string('register') . '</a><br>';
+    }
+    else {
+        $registerlink = '';
+    }
     $loginform = get_login_form_js(pieform(array(
         'name'       => 'login',
         'renderer'   => 'div',
         'submit'     => false,
+        'action'     => $action,
         'plugintype' => 'auth',
         'pluginname' => 'internal',
+        'autofocus'  => false,
         'elements'   => array(
             'login_username' => array(
                 'type'        => 'text',
                 'title'       => get_string('username') . ':',
                 'description' => get_string('usernamedescription'),
+                'defaultvalue' => (isset($_POST['login_username'])) ? $_POST['login_username'] : '',
                 'rules' => array(
                     'required'    => true
                 )
@@ -1466,18 +1548,15 @@ function auth_generate_login_form() {
                 'type'        => 'password',
                 'title'       => get_string('password') . ':',
                 'description' => get_string('passworddescription'),
-                'value'       => '',
-                'rules' => array(
-                    'required'    => true
-                )
+                'defaultvalue'       => '',
             ),
             'submit' => array(
                 'type'  => 'submit',
                 'value' => get_string('login')
             ),
             'register' => array(
-                'value' => '<div><a href="' . get_config('wwwroot') . 'register.php" tabindex="2">' . get_string('register') . '</a>'
-                    . '<br><a href="' . get_config('wwwroot') . 'forgotpass.php" tabindex="2">' . get_string('lostusernamepassword') . '</a></div>'
+                'value' => '<div id="login-helplinks">' . $registerlink
+                    . '<a href="' . get_config('wwwroot') . 'forgotpass.php" tabindex="2">' . get_string('lostusernamepassword') . '</a></div>'
             )
         )
     )));
@@ -1592,6 +1671,9 @@ class PluginAuth extends Plugin {
         }
     }
 
+    public static function can_be_disabled() {
+        return false;
+    }
 }
 
 ?>
