@@ -1,7 +1,8 @@
 <?php
 /**
  * Mahara: Electronic portfolio, weblog, resume builder and social networking
- * Copyright (C) 2006-2008 Catalyst IT Ltd (http://www.catalyst.net.nz)
+ * Copyright (C) 2006-2009 Catalyst IT Ltd and others; see:
+ *                         http://wiki.mahara.org/Contributors
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +21,7 @@
  * @subpackage core
  * @author     Catalyst IT Ltd
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL
- * @copyright  (C) 2006-2008 Catalyst IT Ltd http://catalyst.net.nz
+ * @copyright  (C) 2006-2009 Catalyst IT Ltd http://catalyst.net.nz
  *
  */
 
@@ -62,7 +63,8 @@ class User {
             'expiry'           => null,
             'expirymailsent'   => 0,
             'lastlogin'        => null,
-            'lastauthinstance' => null,
+            'lastlastlogin'    => null,
+            'lastaccess'       => null, /* Is not necessarily updated every request, see accesstimeupdatefrequency config variable */
             'inactivemailsent' => 0,
             'staff'            => 0,
             'admin'            => 0,
@@ -82,12 +84,13 @@ class User {
             'accountprefs'     => array(),
             'activityprefs'    => array(),
             'institutions'     => array(),
+            'grouproles'       => array(),
             'theme'            => null,
             'admininstitutions' => array(),
             'staffinstitutions' => array(),
             'parentuser'       => null,
             'loginanyway'       => false,
-            'sesskey'          => ''
+            'sesskey'          => '',
         );
         $this->attributes = array();
 
@@ -106,13 +109,15 @@ class User {
                     *, 
                     ' . db_format_tsfield('expiry') . ', 
                     ' . db_format_tsfield('lastlogin') . ', 
+                    ' . db_format_tsfield('lastlastlogin') . ',
+                    ' . db_format_tsfield('lastaccess') . ',
                     ' . db_format_tsfield('suspendedctime') . '
                 FROM
                     {usr}
                 WHERE
                     id = ?';
 
-        $user = get_record_sql($sql, $id);
+        $user = get_record_sql($sql, array($id));
 
         if (false == $user) {
             throw new AuthUnknownUserException("User with id \"$id\" is not known");
@@ -120,6 +125,7 @@ class User {
 
         $this->populate($user);
         $this->reset_institutions();
+        $this->reset_grouproles();
         return $this;
     }
 
@@ -140,6 +146,8 @@ class User {
                     *,
                     ' . db_format_tsfield('expiry') . ',
                     ' . db_format_tsfield('lastlogin') . ',
+                    ' . db_format_tsfield('lastlastlogin') . ',
+                    ' . db_format_tsfield('lastaccess') . ',
                     ' . db_format_tsfield('suspendedctime') . '
                 FROM
                     {usr}
@@ -201,15 +209,17 @@ class User {
 
             $sql = 'SELECT
                         u.*, 
-                        ' . db_format_tsfield('u.expiry', 'expiry') . ', 
-                        ' . db_format_tsfield('u.lastlogin', 'lastlogin') . ', 
+                        ' . db_format_tsfield('u.expiry', 'expiry') . ',
+                        ' . db_format_tsfield('u.lastlogin', 'lastlogin') . ',
+                        ' . db_format_tsfield('u.lastlastlogin', 'lastlastlogin') . ',
+                        ' . db_format_tsfield('u.lastaccess', 'lastaccess') . ',
                         ' . db_format_tsfield('u.suspendedctime', 'suspendedctime') . '
                     FROM {usr} u
                     LEFT JOIN {auth_remote_user} r ON u.id = r.localusr
                     WHERE
                         (
                             (
-                                LOWER(r.remoteusername) = ?
+                                r.remoteusername = ?
                                 AND r.authinstance = ?
                             )'
                             . $parentwhere
@@ -220,8 +230,10 @@ class User {
         else {
             $sql = 'SELECT
                         *, 
-                        ' . db_format_tsfield('expiry') . ', 
-                        ' . db_format_tsfield('lastlogin') . ', 
+                        ' . db_format_tsfield('expiry') . ',
+                        ' . db_format_tsfield('lastlogin') . ',
+                        ' . db_format_tsfield('lastlastlogin') . ',
+                        ' . db_format_tsfield('lastaccess') . ',
                         ' . db_format_tsfield('suspendedctime') . '
                     FROM
                         {usr}
@@ -308,10 +320,6 @@ class User {
             throw new InvalidArgumentException('quotaused should be set via the quota_* methods');
         }
 
-        if ($key == 'username' && $this->id != 0) {
-            throw new InvalidArgumentException('We cannot change the username of an existing user');
-        }
-
         $this->set($key, $value);
     }
 
@@ -378,6 +386,79 @@ class User {
         $this->set('accountprefs', $accountprefs);
     }
 
+    /**
+     * Return the profile view object for this user.
+     *
+     * If the user does not yet have a profile view, one is created for them.
+     *
+     * @return View
+     */
+    public function get_profile_view() {
+        $viewid = get_field('view', 'id', 'type', 'profile', 'owner', $this->get('id'));
+        if (!$viewid) {
+            global $USER;
+            if (!$USER->get('id')) {
+                return null;
+            }
+            return $this->install_profile_view();
+        }
+        return new View($viewid);
+    }
+
+    /**
+     * Installs a user's profile view.
+     *
+     * @return View
+     */
+    private function install_profile_view() {
+        static $systemprofileviewid = null;
+
+        db_begin();
+        if (is_null($systemprofileviewid)) {
+            $systemprofileviewid = get_field('view', 'id', 'owner', 0, 'type', 'profile');
+        }
+
+        require_once(get_config('libroot') . 'view.php');
+        list($view) = View::create_from_template(array(
+            'owner' => $this->get('id'),
+            'title' => get_field('view', 'title', 'id', $systemprofileviewid),
+            'type'  => 'profile',
+        ), $systemprofileviewid, $this->get('id'));
+
+        // Add about me block
+        $aboutme = new BlockInstance(0, array(
+            'blocktype'  => 'profileinfo',
+            'title'      => get_string('aboutme', 'blocktype.internal/profileinfo'),
+            'view'       => $view->get('id'),
+            'column'     => 1,
+            'order'      => 1,
+        ));
+        $configdata = array('artefactids' => array());
+        if ($intro = get_field('artefact', 'id', 'owner', $this->get('id'), 'artefacttype', 'introduction')) {
+            $configdata['artefactids'][] = $intro;
+        }
+        else {
+            $configdata['introtext'] = get_string('thisistheprofilepagefor', 'mahara', display_name($this, null, true));
+        }
+        if ($this->get('profileicon')) {
+            $configdata['profileicon'] = $this->get('profileicon');
+        }
+        $aboutme->set('configdata', $configdata);
+        $view->addblockinstance($aboutme);
+
+        $view->set_access(array(
+            array(
+                'type'      => 'loggedin',
+                'startdate' => null,
+                'stopdate'  => null,
+            ),
+        ));
+        db_commit();
+
+        return $view;
+    }
+
+
 
     /**
      * Determines if the user is currently logged in
@@ -392,7 +473,7 @@ class User {
         $this->stdclass = new StdClass;
         reset($this->defaults);
         foreach (array_keys($this->defaults) as $k) {
-            if ($k == 'expiry' || $k == 'lastlogin' || $k == 'suspendedctime') {
+            if ($k == 'expiry' || $k == 'lastlogin' || $k == 'lastlastlogin' || $k == 'lastaccess' || $k == 'suspendedctime') {
                 $this->stdclass->{$k} = db_format_timestamp($this->get($k));
             } else {
                 $this->stdclass->{$k} = $this->get($k);//(is_null($this->get($k))? 'NULL' : $this->get($k));
@@ -433,6 +514,14 @@ class User {
         return true;
     }
 
+    public function quota_init() {
+        if (!$this->get('quota')) {
+            if ($defaultquota = get_config_plugin('artefact', 'file', 'defaultquota')) {
+                $this->set('quota', $defaultquota);
+            }
+        }
+    }
+
     public function join_institution($institution) {
         if ($institution != 'mahara' && !$this->in_institution($institution)) {
             require_once('institution.php');
@@ -472,19 +561,18 @@ class User {
         return isset($a[$institution]);
     }
 
-    /**
-     * There is currently no difference in privileges of site staff
-     * and institutional staff
-     */
-    public function can_create_controlled_groups() {
-        return $this->get('admin') || $this->get('staff') || $this->is_institutional_admin()
-            || $this->is_institutional_staff();
-    }
-
     public function can_edit_institution($institution = null) {
         return $this->get('admin') || $this->is_institutional_admin($institution);
     }
 
+    /**
+     * Returns whether this user is allowed to perform administration type
+     * actions on another user.
+     *
+     * @param mixed $user The user to check we can perform actions on. Can
+     *                    either be a User object, a row from the usr table or
+     *                    an ID
+     */
     public function is_admin_for_user($user) {
         if ($this->get('admin')) {
             return true;
@@ -492,12 +580,29 @@ class User {
         if (!$this->is_institutional_admin()) {
             return false;
         }
+
+        // Check privileges for institutional admins now
         if ($user instanceof User) {
-            $userinstitutions = $user->get('institutions');
-        } else {
-            $userinstitutions = load_user_institutions($user->id);
+            $userobj = $user;
         }
-        foreach ($userinstitutions as $i) {
+        else if (is_numeric($user)) {
+            $userobj = new User;
+            $userobj->find_by_id($user);
+        }
+        else if (is_object($user)) {
+            // Should be a row from the usr table
+            $userobj = new User;
+            $userobj->find_by_id($user->id);
+        }
+        else {
+            throw new SystemException("Invalid argument pass to is_admin_for_user method");
+        }
+
+        if ($userobj->get('admin')) {
+            return false;
+        }
+
+        foreach ($userobj->get('institutions') as $i) {
             if ($this->is_institutional_admin($i->institution)) {
                 return true;
             }
@@ -526,14 +631,6 @@ class User {
         return false;
     }
 
-    public function set_admin_institutions($institutions) {
-        if (empty($institutions)) {
-            $this->set('admininstitutions', array());
-        } else {
-            $this->set('admininstitutions', array_combine($institutions, $institutions));
-        }
-    }
-
     public function add_institution_request($institution, $studentid = null) {
         if (empty($institution) || $institution == 'mahara') {
             return;
@@ -543,7 +640,7 @@ class User {
         $institution->addRequestFromUser($this, $studentid);
     }
 
-    protected function reset_institutions() {
+    public function reset_institutions() {
         $institutions             = load_user_institutions($this->id);
         $admininstitutions = array();
         $staffinstitutions = array();
@@ -564,10 +661,121 @@ class User {
         $this->staffinstitutions  = $staffinstitutions;
     }
 
-    public function can_view_artefact($a) {
-        return $this->get('admin') || $this->get('id') == $a->get('owner');
+    public function reset_grouproles() {
+        $memberships = get_records_array('group_member', 'member', $this->get('id'));
+        $roles = array();
+        if ($memberships) {
+            foreach ($memberships as $m) {
+                $roles[$m->group] = $m->role;
+            }
+        }
+        $this->set('grouproles', $roles);
     }
 
+    public function can_view_artefact($a) {
+        if ($this->get('admin')
+            || ($this->get('id') and $this->get('id') == $a->get('owner'))
+            || $this->is_institutional_admin($a->get('institution'))) {
+            return true;
+        }
+        if ($a->get('group')) {
+            // Only group artefacts can have artefact_access_role & artefact_access_usr records
+            return (bool) count_records_sql("SELECT COUNT(*) FROM {artefact_access_role} ar
+                INNER JOIN {group_member} g ON ar.role = g.role
+                WHERE ar.artefact = ? AND g.member = ? AND ar.can_view = 1 AND g.group = ?", array($a->get('id'), $this->get('id'), $a->get('group')))
+                || record_exists('artefact_access_usr', 'usr', $this->get('id'), 'artefact', $a->get('id'));
+        }
+        return false;
+    }
+
+    public function can_edit_artefact($a) {
+        if ($this->get('admin')
+            || ($this->get('id') and $this->get('id') == $a->get('owner'))
+            || $this->is_institutional_admin($a->get('institution'))) {
+            return true;
+        }
+        $group = $a->get('group');
+        if ($group) {
+            return count_records_sql("SELECT COUNT(*) FROM {artefact_access_role} ar
+                INNER JOIN {group_member} g ON ar.role = g.role
+                WHERE ar.artefact = ? AND g.member = ? AND ar.can_edit = 1 AND g.group = ?", array($a->get('id'), $this->get('id'), $group));
+            /*
+            require_once(get_config('docroot') . 'lib/group.php');
+            $role = group_user_access($group, $this->get('id'));
+            if ($role) {
+                $aperms = $a->get('rolepermissions');
+                return $aperms->{$role}->edit;
+            } */
+        }
+        return false;
+    }
+
+    public function can_edit_view($v) {
+        $owner = $v->get('owner');
+        if ($owner == $this->get('id')) {
+            return true;
+        }
+        $institution = $v->get('institution');
+        if ($institution && $this->can_edit_institution($institution)) {
+            return true;
+        }
+        $group = $v->get('group');
+        if ($group) {
+            $editroles = $v->get('editingroles');
+            $this->reset_grouproles();
+            return isset($this->grouproles[$group]) && in_array($this->grouproles[$group], $editroles);
+        }
+        return false;
+    }
+
+    public function can_delete_self() {
+        if (!$this->get('admin')) {
+            // Users who belong to an institution that doesn't allow
+            // registration cannot delete themselves.
+            foreach ($this->get('institutions') as $i) {
+                if (!$i->registerallowed) {
+                    return false;
+                }
+            }
+            return true;
+        }
+        // The last admin user should not be deleted.
+        return count_records('usr', 'admin', 1, 'deleted', 0) > 1;
+    }
+
+    /**
+     * Makes a literal copy of a list of views for this user.
+     *
+     * @param array $templateids A list of viewids to copy.
+     */
+    public function copy_views($templateids, $checkviewaccess=true) {
+        if (!$templateids) {
+            // Nothing to do
+            return;
+        }
+        if (!is_array($templateids)) {
+            throw new SystemException('User->copy_views: templateids must be a list of templates to copy for the user');
+        }
+        require_once(get_config('libroot') . 'view.php');
+
+        $views = array();
+        foreach (get_records_select_array('view', 'id IN (' . implode(', ', db_array_to_ph($templateids)) . ')', $templateids, '', 'id, title, description, type') as $result) {
+            $views[$result->id] = $result;
+        }
+
+        db_begin();
+        foreach ($templateids as $tid) {
+            View::create_from_template(array(
+                'owner' => $this->get('id'),
+                'title' => $views[$tid]->title,
+                'description' => $views[$tid]->description,
+                'type' => $views[$tid]->type == 'profile' && $checkviewaccess ? 'portfolio' : $views[$tid]->type,
+            ), $tid, $this->get('id'), $checkviewaccess);
+        }
+        db_commit();
+    }
+
+
 }
 
 
@@ -599,8 +807,10 @@ class LiveUser extends User {
     public function login($username, $password) {
         $sql = 'SELECT
                     *, 
-                    ' . db_format_tsfield('expiry') . ', 
-                    ' . db_format_tsfield('lastlogin') . ', 
+                    ' . db_format_tsfield('expiry') . ',
+                    ' . db_format_tsfield('lastlogin') . ',
+                    ' . db_format_tsfield('lastlastlogin') . ',
+                    ' . db_format_tsfield('lastaccess') . ',
                     ' . db_format_tsfield('suspendedctime') . '
                 FROM
                     {usr}
@@ -612,6 +822,18 @@ class LiveUser extends User {
             throw new AuthUnknownUserException("\"$username\" is not known");
         }
 
+        $siteclosedforupgrade = get_config('siteclosed');
+        if ($siteclosedforupgrade && get_config('disablelogin')) {
+            global $SESSION;
+            $SESSION->add_error_msg(get_string('siteclosedlogindisabled'));
+            return false;
+        }
+        if (!$user->admin && ($siteclosedforupgrade || get_config('siteclosedbyadmin'))) {
+            global $SESSION;
+            $SESSION->add_error_msg(get_string('siteclosed'));
+            return false;
+        }
+
         // Authentication instances that have parents do so because they cannot 
         // use Mahara's normal login mechanism - for example, XMLRPC. If the 
         // user is using one of these authentication instances, we look and try 
@@ -628,10 +850,33 @@ class LiveUser extends User {
             $instanceid = $parentid;
         }
         $auth = AuthFactory::create($instanceid);
-        if ($auth->authenticate_user_account($user, $password)) {
-            $user->lastauthinstance = $auth->instanceid;
-            $this->authenticate($user, $auth->instanceid);
-            return true;
+        
+        // catch the AuthInstanceException that allows authentication plugins to
+        // fail but pass onto the next possible plugin
+        try {
+            if ($auth->authenticate_user_account($user, $password)) {
+                $this->authenticate($user, $auth->instanceid);
+                // Check for a suspended institution
+                $authinstance = get_record_sql('
+                    SELECT i.suspended, i.displayname
+                    FROM {institution} i JOIN {auth_instance} a ON a.institution = i.name
+                    WHERE a.id = ?', array($instanceid));
+                if ($authinstance->suspended) {
+                    $sitename = get_config('sitename');
+                    throw new AccessTotallyDeniedException(get_string('accesstotallydenied_institutionsuspended', 'mahara', $authinstance->displayname, $sitename));
+                    return false;
+                }
+                return true;
+            }
+        } catch  (AuthInstanceException $e) {
+            return false;
+        }
+        
+        // Display a message to users who are only allowed to login via their
+        // external application.
+        if ($auth->authloginmsg != '') {
+            global $SESSION;
+            $SESSION->add_info_msg(clean_html($auth->authloginmsg), false);
         }
 
         return false;
@@ -661,14 +906,25 @@ class LiveUser extends User {
     }
 
     /**
-     * Assuming that a session is already active for a user, this method
-     * retrieves the information from the session and creates a user object
-     * that the script can use
-     *
-     * @return object
+     * Updates information in a users' session once we know their session is 
+     * continuing
      */
     public function renew() {
-        $this->set('logout_time', time() + get_config('session_timeout'));
+        $time = time();
+        $this->set('logout_time', $time + get_config('session_timeout'));
+        $oldlastaccess = $this->get('lastaccess');
+        // If there is an access time update frequency, we use a cookie to 
+        // prevent updating before this time has expired.
+        // If it is set to zero, we always update the accesstime.
+        $accesstimeupdatefrequency = get_config('accesstimeupdatefrequency');
+        if ($accesstimeupdatefrequency == 0) {
+            $this->set('lastaccess', $time);
+            $this->commit();
+        }
+        else if ($oldlastaccess + $accesstimeupdatefrequency < $time) {
+            $this->set('lastaccess', $time);
+            $this->commit();
+        }
     }
 
     /**
@@ -694,7 +950,9 @@ class LiveUser extends User {
 
         $this->populate($user);
         session_regenerate_id(true);
+        $this->lastlastlogin      = $this->lastlogin;
         $this->lastlogin          = time();
+        $this->lastaccess         = time();
         $this->sessionid          = session_id();
         $this->logout_time        = time() + get_config('session_timeout');
         $this->sesskey            = get_random_key();
@@ -704,7 +962,12 @@ class LiveUser extends User {
         $this->activityprefs      = load_activity_preferences($user->id);
         $this->accountprefs       = load_account_preferences($user->id);
         $this->reset_institutions();
+        $this->reset_grouproles();
         $this->commit();
+
+        // finally, after all is done, call the (maybe non existant) hook on their auth plugin
+        $authobj = AuthFactory::create($authinstance);
+        $authobj->login();
     }
 
     /**
@@ -717,7 +980,6 @@ class LiveUser extends User {
      */
     public function reanimate($id, $instanceid) {
         if ($user = get_record('usr','id',$id)) {
-            $user->lastauthinstance = $instanceid;
             $this->authenticate($user, $instanceid);
             return true;
         }
@@ -807,6 +1069,9 @@ class LiveUser extends User {
         $this->reset_institutions();
     }
 
-
+    public function update_theme() {
+        $this->reset_institutions();
+        $this->commit();
+    }
 }
 ?>
