@@ -1,7 +1,8 @@
 <?php
 /**
  * Mahara: Electronic portfolio, weblog, resume builder and social networking
- * Copyright (C) 2006-2008 Catalyst IT Ltd (http://www.catalyst.net.nz)
+ * Copyright (C) 2006-2009 Catalyst IT Ltd and others; see:
+ *                         http://wiki.mahara.org/Contributors
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +21,7 @@
  * @subpackage auth-internal
  * @author     Catalyst IT Ltd
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL
- * @copyright  (C) 2006-2008 Catalyst IT Ltd http://catalyst.net.nz
+ * @copyright  (C) 2006-2009 Catalyst IT Ltd http://catalyst.net.nz
  *
  */
 
@@ -38,9 +39,7 @@ require_once(get_config('docroot') . 'api/xmlrpc/lib.php');
  */
 class AuthXmlrpc extends Auth {
 
-    public $file = null;
-
-    /**
+    /**findByWwwroot
      * Get the party started with an optional id
      * TODO: appraise
      * @param int $id   The auth instance id
@@ -62,8 +61,9 @@ class AuthXmlrpc extends Auth {
         $this->config['theyautocreateusers']   = 0;
         $this->config['wessoout']              = 0;
         $this->config['theyssoin']             = 0;
+        $this->config['weimportcontent']       = 0;
         $this->config['parent']                = null;
-        $this->file = fopen('/tmp/out.txt', 'w');
+        $this->config['authloginmsg']          = '';
         if (!empty($id)) {
             return $this->init($id);
         }
@@ -82,6 +82,9 @@ class AuthXmlrpc extends Auth {
         if (array_key_exists($name, $this->config)) {
             return $this->config[$name];
         }
+        if (isset($this->$name)) {
+            return $this->$name;
+        }
     }
 
     /**
@@ -213,26 +216,8 @@ class AuthXmlrpc extends Auth {
 
             db_begin();
             $user->username           = get_new_username($remoteuser->username);
-            $user->commit();
 
-            // Make sure that there's no remote user record for this user. This 
-            // can happen when a user has SSOed in, and then been deleted from 
-            // Mahara.
-            //
-            // This makes undeleting the old user record "interesting", because 
-            // now we don't have the remoteuser record for them...
-            delete_records('auth_remote_user', 'authinstance', $user->authinstance, 'remoteusername', $remoteuser->username);
-            insert_record('auth_remote_user', (object) array(
-                'authinstance'   => $user->authinstance,
-                'remoteusername' => $remoteuser->username,
-                'localusr'       => $user->id,
-            ));
-
-            $user->join_institution($peer->institution);
-
-            set_profile_field($user->id, 'firstname', $user->firstname);
-            set_profile_field($user->id, 'lastname', $user->lastname);
-            set_profile_field($user->id, 'email', $user->email);
+            $user->id = create_user($user, array(), $this->institution, $this, $remoteuser->username);
 
             $this->import_user_settings($user, $remoteuser);
 
@@ -244,7 +229,6 @@ class AuthXmlrpc extends Auth {
              */
             $userobj = $user->to_stdclass();
             $userarray = (array)$userobj;
-            handle_event('createuser', $userarray);
             db_commit();
 
             // Now we have fired the create event, we need to re-get the data 
@@ -263,6 +247,7 @@ class AuthXmlrpc extends Auth {
 
             $this->import_user_settings($user, $remoteuser);
 
+            $user->lastlastlogin      = $user->lastlogin;
             $user->lastlogin          = time();
 
             //TODO: import institution's per-user-quota?:
@@ -286,78 +271,88 @@ class AuthXmlrpc extends Auth {
             $imageobject = (object)$client->response;
 
             $u = preg_replace('/[^A-Za-z0-9 ]/', '', $user->username);
-            $filename = '/tmp/'.intval($this->instanceid).'_'.$u;
+            $filename = get_config('dataroot') . 'temp/mpi_' . intval($this->instanceid) . '_' . $u;
 
             if (array_key_exists('f1', $client->response)) {
                 $imagecontents = base64_decode($client->response['f1']);
-                file_put_contents($filename, $imagecontents);
-                $imageexists = false;
-                $icons       = false;
+                if (file_put_contents($filename, $imagecontents)) {
+                    $imageexists = false;
+                    $icons       = false;
 
-                if ($update) {
-                    $newchecksum = sha1_file($filename);
-                    $icons = get_records_select_array('artefact', 'artefacttype = \'profileicon\' AND owner = ? ', array($user->id), '', 'id');
-                    if (false != $icons) {
-                        foreach ($icons as $icon) {
-                            $iconfile = get_config('dataroot') . 'artefact/internal/profileicons/originals/' . ($icon->id % 256) . '/'.$icon->id;
-                            $checksum = sha1_file($iconfile);
-                            if ($newchecksum == $checksum) {
-                                $imageexists = true;
-                                unlink($filename);
-                                break;
+                    if ($update) {
+                        $newchecksum = sha1_file($filename);
+                        $icons = get_records_select_array('artefact', 'artefacttype = \'profileicon\' AND owner = ? ', array($user->id), '', 'id');
+                        if (false != $icons) {
+                            foreach ($icons as $icon) {
+                                $iconfile = get_config('dataroot') . 'artefact/file/profileicons/originals/' . ($icon->id % 256) . '/'.$icon->id;
+                                $checksum = sha1_file($iconfile);
+                                if ($newchecksum == $checksum) {
+                                    $imageexists = true;
+                                    unlink($filename);
+                                    break;
+                                }
                             }
                         }
                     }
+
+                    if (false == $imageexists) {
+                        $filesize = filesize($filename);
+                        if (!$user->quota_allowed($filesize)) {
+                            $error = get_string('profileiconuploadexceedsquota', 'artefact.file', get_config('wwwroot'));
+                        }
+
+                        require_once('file.php');
+                        $imagesize = getimagesize($filename);
+                        if (!$imagesize || !is_image_type($imagesize[2])) {
+                            $error = get_string('filenotimage');
+                        }
+
+                        $mime   = $imagesize['mime'];
+                        $width  = $imagesize[0];
+                        $height = $imagesize[1];
+                        $imagemaxwidth  = get_config('imagemaxwidth');
+                        $imagemaxheight = get_config('imagemaxheight');
+                        if ($width > $imagemaxwidth || $height > $imagemaxheight) {
+                            $error = get_string('profileiconimagetoobig', 'artefact.file', $width, $height, $imagemaxwidth, $imagemaxheight);
+                        }
+
+                        try {
+                            $user->quota_add($filesize);
+                        }
+                        catch (QuotaException $qe) {
+                            $error =  get_string('profileiconuploadexceedsquota', 'artefact.file', get_config('wwwroot'));
+                        }
+
+                        require_once(get_config('docroot') .'/artefact/lib.php');
+                        require_once(get_config('docroot') .'/artefact/file/lib.php');
+
+                        // Entry in artefact table
+                        $artefact = new ArtefactTypeProfileIcon();
+                        $artefact->set('owner', $user->id);
+                        $artefact->set('title', 'Profile Icon');
+                        $artefact->set('note', 'Profile Icon');
+                        $artefact->set('size', $filesize);
+                        $artefact->set('filetype', $mime);
+                        $artefact->set('width', $width);
+                        $artefact->set('height', $height);
+                        $artefact->commit();
+
+                        $id = $artefact->get('id');
+
+                        // Move the file into the correct place.
+                        $directory = get_config('dataroot') . 'artefact/file/profileicons/originals/' . ($id % 256) . '/';
+                        check_dir_exists($directory);
+                        rename($filename, $directory . $id);
+                        if ($create || empty($icons)) {
+                            $user->profileicon = $id;
+                        }
+                    }
+
+                    $user->commit();
                 }
-
-                if (false == $imageexists) {
-                    $filesize = filesize($filename);
-                    if (!$user->quota_allowed($filesize)) {
-                        $error = get_string('profileiconuploadexceedsquota', 'artefact.internal', get_config('wwwroot'));
-                    }
-
-                    require_once('file.php');
-                    $mime = get_mime_type($filename);
-                    if (!is_image_mime_type($mime)) {
-                        $error = get_string('filenotimage');
-                    }
-
-                    list($width, $height) = getimagesize($filename);
-                    $imagemaxwidth  = get_config('imagemaxwidth');
-                    $imagemaxheight = get_config('imagemaxheight');
-                    if ($width > $imagemaxwidth || $height > $imagemaxheight) {
-                        $error = get_string('profileiconimagetoobig', 'artefact.internal', $width, $height, $imagemaxwidth, $imagemaxheight);
-                    }
-
-                    try {
-                        $user->quota_add($filesize);
-                    }
-                    catch (QuotaException $qe) {
-                        $error =  get_string('profileiconuploadexceedsquota', 'artefact.internal', get_config('wwwroot'));
-                    }
-
-                    require_once(get_config('docroot') .'/artefact/lib.php');
-                    require_once(get_config('docroot') .'/artefact/internal/lib.php');
-
-                    // Entry in artefact table
-                    $artefact = new ArtefactTypeProfileIcon();
-                    $artefact->set('owner', $user->id);
-                    $artefact->set('title', 'Profile Icon');
-                    $artefact->set('note', 'Profile Icon');
-                    $artefact->commit();
-
-                    $id = $artefact->get('id');
-
-                    // Move the file into the correct place.
-                    $directory = get_config('dataroot') . 'artefact/internal/profileicons/originals/' . ($id % 256) . '/';
-                    check_dir_exists($directory);
-                    rename($filename, $directory . $id);
-                    if ($create || empty($icons)) {
-                        $user->profileicon = $id;
-                    }
+                else {
+                    log_warn(get_string('cantcreatetempprofileiconfile', 'artefact.file', $filename));
                 }
-
-                $user->commit();
             }
         }
 
@@ -372,6 +367,7 @@ class AuthXmlrpc extends Auth {
         // the session object.
         $SESSION->set('mnetuser', $user->id);
         $SESSION->set('authinstance', $this->instanceid);
+        $SESSION->set('mnetuserfrom', $_SERVER['HTTP_REFERER']);
 
         return true;
     }
@@ -435,8 +431,8 @@ class AuthXmlrpc extends Auth {
     private function import_user_settings($user, $remoteuser) {
         // City
         if (!empty($remoteuser->city)) {
-            if (get_profile_field($user->id, 'city') != $remoteuser->city) {
-                set_profile_field($user->id, 'city', $remoteuser->city);
+            if (get_profile_field($user->id, 'town') != $remoteuser->city) {
+                set_profile_field($user->id, 'town', $remoteuser->city);
             }
         }
 
@@ -493,8 +489,8 @@ class AuthXmlrpc extends Auth {
             log_debug("XMLRPC error occured while calling MNET method kill_children on $this->wwwroot");
             log_debug("This means that single-signout probably didn't work properly, but the problem "
                 . "is at the remote application");
-            log_debug("If the remote application is Moodle, and you're happy with a Mahara developer "
-                . "getting access to your system so they can try and debug the problem, get in touch with dev@mahara.org");
+            log_debug("If the remote application is Moodle, you are likely a victim of "
+                . "http://tracker.moodle.org/browse/MDL-16872 - try applying the attached patch to fix the issue");
             log_debug("Exception message follows:");
             log_debug($e->getMessage());
         }
@@ -560,12 +556,14 @@ class PluginAuthXmlrpc extends PluginAuth {
         'name'                  => '',
         'appname'               => '',
         'portno'                => 80,
-        'updateuserinfoonlogin' => 0, 
+        'updateuserinfoonlogin' => 0,
         'weautocreateusers'     => 0,
         'theyautocreateusers'   => 0,
         'wessoout'              => 0,
         'theyssoin'             => 0,
-        'parent'                => null
+        'weimportcontent'       => 0,
+        'parent'                => null,
+        'authloginmsg'          => ''
     );
 
     public static function has_config() {
@@ -690,15 +688,6 @@ class PluginAuthXmlrpc extends PluginAuth {
             'value' => 'xmlrpc'
         );
 
-        $elements['parent'] = array(
-            'type'                => 'select',
-            'title'               => get_string('parent','auth'),
-            'collapseifoneoption' => false,
-            'options'             => $options,
-            'defaultvalue'        => self::$default_config['parent'],
-            'help'   => true
-        );
-
         $elements['wwwroot'] = array(
             'type' => 'text',
             'title' => get_string('wwwroot', 'auth'),
@@ -779,18 +768,31 @@ class PluginAuthXmlrpc extends PluginAuth {
             'help'   => true
         );
 
-        $elements['wessoout'] = array(
-            'type'         => 'checkbox',
-            'title'        => get_string('wessoout', 'auth'),
-            'defaultvalue' => self::$default_config['wessoout'],
+        $elements['parent'] = array(
+            'type'                => 'select',
+            'title'               => get_string('parent','auth'),
+            'collapseifoneoption' => false,
+            'options'             => $options,
+            'defaultvalue'        => self::$default_config['parent'],
             'help'   => true
         );
 
-        $elements['theyssoin'] = array(
-            'type'         => 'checkbox',
-            'title'        => get_string('theyssoin', 'auth'),
-            'defaultvalue' => self::$default_config['theyssoin'],
-            'help'   => true
+        $elements['authloginmsg'] = array(
+            'type'         => 'wysiwyg',
+            'rows'         => 10,
+            'cols'         => 70,
+            'title'        => '',
+            'description'  => get_string('authloginmsg', 'auth'),
+            'defaultvalue' => self::$default_config['authloginmsg'],
+            'help'         => true,
+        );
+
+        $elements['ssodirection'] = array(
+            'type'         => 'select',
+            'title'        => get_string('ssodirection', 'auth'),
+            'options'      => array(0 => '--', 'theyssoin' => get_string('theyssoin', 'auth'), 'wessoout' => get_string('wessoout', 'auth')),
+            'defaultvalue' => self::$default_config['wessoout'] ? 'wessoout' : 'theyssoin',
+            'help'   => true,
         );
 
         $elements['updateuserinfoonlogin'] = array(
@@ -813,7 +815,14 @@ class PluginAuthXmlrpc extends PluginAuth {
             'defaultvalue' => self::$default_config['theyautocreateusers'],
             'help'   => true
         );
-        
+
+        $elements['weimportcontent'] = array(
+            'type'         => 'checkbox',
+            'title'        => get_string('weimportcontent', 'auth'),
+            'defaultvalue' => self::$default_config['weimportcontent'],
+            'help'         => true,
+        );
+
         return array(
             'elements' => $elements,
             'renderer' => 'table'
@@ -832,9 +841,18 @@ class PluginAuthXmlrpc extends PluginAuth {
             try {
                 $peer->bootstrap($values['wwwroot'], null, $values['appname'], $values['institution']);
             } catch (RemoteServerException $e) {
+                log_debug($e->getMessage());
                 $form->set_error('wwwroot',get_string('cantretrievekey', 'auth'));
             }
         }
+        else if ($values['institution'] != $peer->institution) {
+            if (get_records_sql_array("
+                SELECT ai.*, aic.*
+                FROM {auth_instance} ai JOIN {auth_instance_config} aic ON ai.id = aic.instance
+                WHERE aic.field = 'wwwroot' AND aic.value = ? AND ai.institution = ?", array($values['wwwroot'], $peer->institution))) {
+                $form->set_error('wwwroot',get_string('hostwwwrootinuse', 'auth', hsc(get_field('institution', 'displayname', 'name', $peer->institution))));
+            }
+        }
 
         if (isset($values['publickey'])) {
             try {
@@ -919,14 +937,27 @@ class PluginAuthXmlrpc extends PluginAuth {
         }
 
         self::$default_config = array(  'wwwroot'               => $values['wwwroot'],
-                                        'updateuserinfoonlogin' => $values['updateuserinfoonlogin'],
-                                        'weautocreateusers'     => $values['weautocreateusers'],
-                                        'theyautocreateusers'   => $values['theyautocreateusers'],
                                         'parent'                => $values['parent'],
-                                        'wessoout'              => $values['wessoout'],
-                                        'theyssoin'             => $values['theyssoin']
+                                        'authloginmsg'          => $values['authloginmsg'],
+                                        'wessoout'              => 0,
+                                        'theyssoin'             => 0,
+                                        'theyautocreateusers'   => 0,
+                                        'weautocreateusers'     => 0,
+                                        'updateuserinfoonlogin' => 0,
+                                        'weimportcontent'       => 0,
                                         );
 
+        if ($values['ssodirection'] == 'wessoout') {
+            self::$default_config['wessoout']              = 1;
+            self::$default_config['theyautocreateusers']   = $values['theyautocreateusers'];
+        }
+        else if ($values['ssodirection'] == 'theyssoin') {
+            self::$default_config['theyssoin']             = 1;
+            self::$default_config['updateuserinfoonlogin'] = $values['updateuserinfoonlogin'];
+            self::$default_config['weautocreateusers']     = $values['weautocreateusers'];
+            self::$default_config['weimportcontent']       = $values['weimportcontent'];
+        }
+
         foreach(self::$default_config as $field => $value) {
             $record = new stdClass();
             $record->instance = $values['instance'];
@@ -950,6 +981,48 @@ class PluginAuthXmlrpc extends PluginAuth {
         return $values;
     }
 
+    public static function get_jump_url_prefix($hostwwwroot, $hostapp) {
+        return $hostwwwroot . '/' . ($hostapp == 'moodle' ? 'auth/mnet/jump.php' : 'auth/xmlrpc/jump.php')
+            . '?hostwwwroot=' . substr(get_config('wwwroot'), 0, -1) . '&wantsurl=';
+    }
+
+}
+
+/**
+ * Lifted from Moodle.
+ *
+ * Inline function to modify a url string so that mnet users are requested to
+ * log in at their mnet identity provider (if they are not already logged in)
+ * before ultimately being directed to the original url.
+ *
+ * uses global IDPJUMPURL - the url which user should initially be directed to
+ * @param array $url array with 3 elements
+ *     0 - context the url was taken from, possibly just the url, possibly href="url"
+ *     1 - the destination url
+ *     2 - the destination url, without the wwwroot part
+ * @return string the url the remote user should be supplied with.
+ */
+function localurl_to_jumpurl($url) {
+    global $IDPJUMPURL;
+    $localpart='';
+    $urlparts = parse_url($url[2]);
+    if ($urlparts) {
+        if (isset($urlparts['path'])) {
+            $localpart .= $urlparts['path'];
+        }
+        if (isset($urlparts['query'])) {
+            $localpart .= '?'.$urlparts['query'];
+        }
+        if (isset($urlparts['fragment'])) {
+            $localpart .= '#'.$urlparts['fragment'];
+        }
+    }
+    $indirecturl = $IDPJUMPURL . urlencode($localpart);
+    //If we matched on more than just a url (ie an html link), return the url to an href format
+    if ($url[0] != $url[1]) {
+        $indirecturl = 'href="'.$indirecturl.'"';
+    }
+    return $indirecturl;
 }
 
 ?>
