@@ -1,7 +1,8 @@
 <?php
 /**
  * Mahara: Electronic portfolio, weblog, resume builder and social networking
- * Copyright (C) 2006-2008 Catalyst IT Ltd (http://www.catalyst.net.nz)
+ * Copyright (C) 2006-2009 Catalyst IT Ltd and others; see:
+ *                         http://wiki.mahara.org/Contributors
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +21,7 @@
  * @subpackage core
  * @author     Catalyst IT Ltd
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL
- * @copyright  (C) 2006-2008 Catalyst IT Ltd http://catalyst.net.nz
+ * @copyright  (C) 2006-2009 Catalyst IT Ltd http://catalyst.net.nz
  *
  */
 
@@ -28,10 +29,15 @@ define('INTERNAL', 1);
 define('MENUITEM', 'groups');
 require(dirname(dirname(__FILE__)) . '/init.php');
 require_once('pieforms/pieform.php');
-require('group.php');
+require_once('group.php');
 $groupid = param_integer('id');
+define('GROUP', $groupid);
 
-$group = get_record('group', 'id', $groupid, 'owner', $USER->get('id'), 'deleted', 0);
+$group = get_record_sql("SELECT g.name
+    FROM {group} g
+    INNER JOIN {group_member} gm ON (gm.group = g.id AND gm.member = ? AND gm.role = 'admin')
+    WHERE g.id = ?
+    AND g.deleted = 0", array($USER->get('id'), $groupid));
 
 if (!$group) {
     throw new AccessDeniedException(get_string('cantdeletegroup', 'group'));
@@ -61,14 +67,14 @@ $form = pieform(array(
 ));
 
 $smarty = smarty();
-$smarty->assign('heading', TITLE);
+$smarty->assign('subheading', hsc(TITLE));
 $smarty->assign('message', $views ? get_string('groupconfirmdeletehasviews', 'group') : get_string('groupconfirmdelete', 'group'));
 $smarty->assign('form', $form);
 $smarty->display('group/delete.tpl');
 
 function deletegroup_submit(Pieform $form, $values) {
     global $SESSION, $USER, $groupid;
-    delete_group($groupid);
+    group_delete($groupid);
     $SESSION->add_ok_msg(get_string('deletegroup', 'group'));
     redirect('/group/mygroups.php');
 }
