@@ -1,7 +1,8 @@
 <?php
 /**
  * Mahara: Electronic portfolio, weblog, resume builder and social networking
- * Copyright (C) 2006-2008 Catalyst IT Ltd (http://www.catalyst.net.nz)
+ * Copyright (C) 2006-2009 Catalyst IT Ltd and others; see:
+ *                         http://wiki.mahara.org/Contributors
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +21,7 @@
  * @subpackage core
  * @author     Catalyst IT Ltd
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL
- * @copyright  (C) 2006-2008 Catalyst IT Ltd http://catalyst.net.nz
+ * @copyright  (C) 2006-2009 Catalyst IT Ltd http://catalyst.net.nz
  *
  */
 
@@ -28,20 +29,16 @@ define('INTERNAL', 1);
 define('MENUITEM', 'groups');
 require(dirname(dirname(__FILE__)) . '/init.php');
 require_once('pieforms/pieform.php');
-require('group.php');
+require_once('group.php');
 $groupid = param_integer('id');
 $returnto = param_alpha('returnto', 'mygroups');
 
-$group = get_record('group', 'id', $groupid, 'deleted', 0);
-if (!$group) {
-    throw new GroupNotFoundException(get_string('groupnotfound', 'group', $groupid));
-}
+define('GROUP', $groupid);
+$group = group_current_group();
 
-define('TITLE', get_string('leavespecifiedgroup', 'group', $group->name));
+define('TITLE', $group->name);
 
-$membership = user_can_access_group($group);
-
-if (!($membership & GROUP_MEMBERSHIP_MEMBER)) {
+if (!group_user_access($group->id)) {
     throw new AccessDeniedException(get_string('notamember', 'group'));
 }
 
@@ -49,6 +46,8 @@ if (!group_user_can_leave($group)) {
     throw new AccessDeniedException(get_string('cantleavegroup', 'group'));
 }
 
+$goto = get_config('wwwroot') . 'group/' . $returnto . '.php' . ($returnto == 'view' ? ('?id=' . $groupid) : '');
+
 $views = count_records_sql(
     'SELECT COUNT(*)
     FROM {view} v
@@ -68,7 +67,7 @@ $form = pieform(array(
         'submit' => array(
             'type' => 'submitcancel',
             'value' => array(get_string('yes'), get_string('no')),
-            'goto' => get_config('wwwroot') . ($returnto == 'find' ? 'group/find.php' : 'group/mygroups.php')
+            'goto' => $goto
         ),
         'returnto' => array(
             'type' => 'hidden',
@@ -78,16 +77,16 @@ $form = pieform(array(
 ));
 
 $smarty = smarty();
-$smarty->assign('heading', TITLE);
+$smarty->assign('subheading', get_string('leavespecifiedgroup', 'group', $group->name));
 $smarty->assign('form', $form);
 $smarty->assign('message', $views ? get_string('groupconfirmleavehasviews', 'group') : get_string('groupconfirmleave', 'group'));
 $smarty->assign('group', $group);
 $smarty->display('group/leave.tpl');
 
 function leavegroup_submit(Pieform $form, $values) {
-    global $USER, $SESSION, $groupid;
+    global $USER, $SESSION, $groupid, $goto;
     group_remove_user($groupid, $USER->get('id'));
     $SESSION->add_ok_msg(get_string('leftgroup', 'group'));
-    redirect($values['returnto'] == 'find' ? '/group/find.php' : '/group/mygroups.php');
+    redirect($goto);
 }
 ?>
