@@ -1,7 +1,8 @@
 <?php
 /**
  * Mahara: Electronic portfolio, weblog, resume builder and social networking
- * Copyright (C) 2006-2008 Catalyst IT Ltd (http://www.catalyst.net.nz)
+ * Copyright (C) 2006-2009 Catalyst IT Ltd and others; see:
+ *                         http://wiki.mahara.org/Contributors
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +21,7 @@
  * @subpackage core
  * @author     Catalyst IT Ltd
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL
- * @copyright  (C) 2006-2008 Catalyst IT Ltd http://catalyst.net.nz
+ * @copyright  (C) 2006-2009 Catalyst IT Ltd http://catalyst.net.nz
  *
  */
 
@@ -55,7 +56,9 @@ if (!is_readable($CFG->docroot . 'config.php')) {
 
 init_performance_info();
 
-require('config.php');
+require($CFG->docroot . 'config.php');
+$CFG = (object)array_merge((array)$cfg, (array)$CFG);
+require_once('config-defaults.php');
 $CFG = (object)array_merge((array)$cfg, (array)$CFG);
 
 // Fix up paths in $CFG
@@ -76,11 +79,9 @@ if (empty($CFG->directorypermissions)) {
 require('mahara.php');
 ensure_sanity();
 require('dml.php');
-require('ddl.php');
-require('constants.php');
 require('web.php');
-require('activity.php');
 require('user.php');
+require(get_config('docroot') . 'local/lib.php');
 
 // Database access functions
 require('adodb/adodb-exceptions.inc.php');
@@ -91,9 +92,15 @@ try {
     // fails for some reason, so we use output buffering to catch whatever
     // the error is instead.
     ob_start();
+
+    if (is_postgres()) {
+        $CFG->dbtype = 'postgres7';
+    }
+    else if (is_mysql()) {
+        $CFG->dbtype = 'mysql';
+    }
     
     $db = &ADONewConnection($CFG->dbtype);
-    $dbgenerator = null;
     if (empty($CFG->dbhost)) {
         $CFG->dbhost = '';
     }
@@ -107,6 +114,20 @@ try {
         $dbconnected = $db->Connect($CFG->dbhost,$CFG->dbuser,$CFG->dbpass,$CFG->dbname);
     }
 
+    // Now we have a connection, verify the server is a new enough version
+    $dbversion = $db->ServerInfo();
+    if (is_postgres()) {
+        $okversion = '8.1';
+        $dbfriendlyname = 'PostgreSQL';
+    }
+    else if (is_mysql()) {
+        $okversion = '5.0.25';
+        $dbfriendlyname = 'MySQL';
+    }
+    if ($dbversion['version'] < $okversion) {
+        throw new ConfigSanityException(get_string('dbversioncheckfailed', 'error', $dbfriendlyname, $dbversion['version'], $okversion));
+    }
+
     $db->SetFetchMode(ADODB_FETCH_ASSOC);
     configure_dbconnection();
     ensure_internal_plugins_exist();
@@ -114,6 +135,9 @@ try {
     ob_end_clean();
 }
 catch (Exception $e) {
+    if ($e instanceof ConfigSanityException) {
+        throw $e;
+    }
     $errormessage = ob_get_contents();
     if (!$errormessage) {
         $errormessage = $e->getMessage();
@@ -131,7 +155,7 @@ catch (SQLException $e) {
     db_ignore_sql_exceptions(false);
 }
 
-// Make sure wwwroot is set and available, either in the database or int the
+// Make sure wwwroot is set and available, either in the database or in the
 // config file. Cron requires it for some purposes.
 if (!isset($CFG->wwwroot) && isset($_SERVER['HTTP_HOST'])) {
     $proto = (isset($_SERVER['HTTPS'])) ? 'https://' : 'http://';
@@ -142,6 +166,7 @@ if (!isset($CFG->wwwroot) && isset($_SERVER['HTTP_HOST'])) {
     }
     $path  = substr(dirname(__FILE__), strlen($_SERVER['DOCUMENT_ROOT']));
     if ($path) {
+        $path = str_replace('\\', '/', $path);  // windows
         if (substr($path, 0, 1) != '/') {
             $path = '/' . $path;
         }
@@ -158,6 +183,11 @@ if (!isset($CFG->wwwroot) && isset($_SERVER['HTTP_HOST'])) {
         $CFG->wwwroot = $wwwroot;
     }
 }
+if (isset($CFG->httpswwwroot)) {
+    if (substr($CFG->httpswwwroot, -1, 1) != '/') {
+        $CFG->httpswwwroot .= '/';
+    }
+}
 if (!isset($CFG->noreplyaddress) && isset($_SERVER['HTTP_HOST'])) {
     $noreplyaddress = 'noreply@';
     $host  =  (isset($_SERVER['HTTP_X_FORWARDED_HOST'])) ? $_SERVER['HTTP_X_FORWARDED_HOST'] : $_SERVER['HTTP_HOST'];
@@ -181,7 +211,10 @@ if (!get_config('theme')) {
     $CFG->theme = 'default'; 
 }
 
-$CFG->themeurl = get_config('wwwroot') . 'theme/' . get_config('theme') . '/static/';
+if (defined('INSTALLER')) { 
+    // Custom themes sometimes cause upgrades to fail.
+    $CFG->theme = 'default';
+}
 
 // Make sure the search plugin is configured
 if (!get_config('searchplugin')) {
@@ -192,7 +225,6 @@ if (!get_config('searchplugin')) {
         $CFG->searchplugin = 'internal';
     }
 }
-
 header('Content-type: text/html; charset=UTF-8');
 
 // Only do authentication once we know the page theme, so that the login form
@@ -200,19 +232,43 @@ header('Content-type: text/html; charset=UTF-8');
 require_once('auth/lib.php');
 $SESSION = Session::singleton();
 $USER    = new LiveUser();
+$THEME   = new Theme($USER);
 // The installer does its own auth_setup checking, because some upgrades may
 // break logging in and so need to allow no logins.
 if (!defined('INSTALLER')) {
     auth_setup();
 }
 
+$siteclosedforupgrade = get_config('siteclosed');
+if ($siteclosedforupgrade && $USER->admin) {
+    if (get_config('disablelogin')) {
+        $USER->logout();
+    }
+    else if (!defined('INSTALLER')) {
+        redirect('/admin/upgrade.php');
+    }
+}
+
+$siteclosed = $siteclosedforupgrade || get_config('siteclosedbyadmin');
+if ($siteclosed && !$USER->admin) {
+    if ($USER->is_logged_in()) {
+        $USER->logout();
+    }
+    if (!defined('HOME') && !defined('INSTALLER')) {
+        redirect();
+    }
+}
+
 // check to see if we're installed...
-$scriptfilename = str_replace('\\', '/', $_SERVER['SCRIPT_FILENAME']);
-if (!get_config('installed')
-    && false === strpos($scriptfilename, 'admin/index.php')
+if (!get_config('installed')) {
+    ensure_install_sanity();
+
+    $scriptfilename = str_replace('\\', '/', $_SERVER['SCRIPT_FILENAME']);
+    if (false === strpos($scriptfilename, 'admin/index.php')
     && false === strpos($scriptfilename, 'admin/upgrade.php')
     && false === strpos($scriptfilename, 'admin/upgrade.json.php')) {
-    redirect('/admin/');
+        redirect('/admin/');
+    }
 }
 
 if (defined('JSON') && !defined('NOSESSKEY')) {
@@ -238,7 +294,7 @@ function init_performance_info() {
     global $PERF;
   
     $PERF = new StdClass;
-    $PERF->dbqueries = 0;   
+    $PERF->dbreads = $PERF->dbwrites = 0;
     $PERF->logwrites = 0;
     if (function_exists('microtime')) {
         $PERF->starttime = microtime();
