@@ -1,7 +1,8 @@
 <?php
 /**
  * Mahara: Electronic portfolio, weblog, resume builder and social networking
- * Copyright (C) 2006-2008 Catalyst IT Ltd (http://www.catalyst.net.nz)
+ * Copyright (C) 2006-2009 Catalyst IT Ltd and others; see:
+ *                         http://wiki.mahara.org/Contributors
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +21,7 @@
  * @subpackage interaction-forum
  * @author     Catalyst IT Ltd
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL
- * @copyright  (C) 2006-2008 Catalyst IT Ltd http://catalyst.net.nz
+ * @copyright  (C) 2006-2009 Catalyst IT Ltd http://catalyst.net.nz
  *
  */
 
@@ -28,6 +29,8 @@ $string['addpostsuccess'] = 'Added post successfully';
 $string['addtitle'] = 'Add forum';
 $string['addtopic'] = 'Add topic';
 $string['addtopicsuccess'] = 'Added topic successfully';
+$string['autosubscribeusers'] = 'Automatically subscribe users?';
+$string['autosubscribeusersdescription'] = 'Choose whether group users will automatically be subscribed to this forum';
 $string['Body'] = 'Body';
 $string['cantaddposttoforum'] = 'You are not allowed to post in this forum';
 $string['cantaddposttotopic'] = 'You are not allowed to post in this topic';
@@ -46,9 +49,12 @@ $string['chooseanaction'] = 'Choose an action';
 $string['clicksetsubject'] = 'Click to set a subject';
 $string['Closed'] = 'Closed';
 $string['Close'] = 'Close';
-$string['closeddescription'] = 'Closed topics can only be replied to by moderators and the group owner';
+$string['closeddescription'] = 'Closed topics can only be replied to by moderators and the group administrators';
 $string['Count'] = 'Count';
+$string['createtopicusersdescription'] = 'If set to "All group members", anyone can create new topics and reply to existing topics.  If set to "Moderators and group admins", only moderators and group administrators can start new topics, but once topics exist, all users can post replies to them.';
 $string['currentmoderators'] = 'Current Moderators';
+$string['defaultforumtitle'] = 'General Discussion';
+$string['defaultforumdescription'] = '%s general discussion forum';
 $string['deleteforum'] = 'Delete forum';
 $string['deletepost'] = 'Delete post';
 $string['deletepostsuccess'] = 'Post deleted successfully';
@@ -63,21 +69,46 @@ $string['editstothispost'] = 'Edits to this post:';
 $string['edittitle'] = 'Edit forum';
 $string['edittopic'] = 'Edit topic';
 $string['edittopicsuccess'] = 'Topic edited successfully';
+$string['forumname'] = 'Forum Name';
+$string['forumposthtmltemplate'] = "<div style=\"padding: 0.5em 0; border-bottom: 1px solid #999;\"><strong>%s by %s</strong><br>%s</div>
+
+<div style=\"margin: 1em 0;\">%s</div>
+
+<div style=\"font-size: smaller; border-top: 1px solid #999;\">
+<p><a href=\"%s\">Reply to this post online</a></p>
+<p><a href=\"%s\">Unsubscribe from this %s</a></p>
+</div>";
+$string['forumposttemplate'] = "%s by %s
+%s
+------------------------------------------------------------------------
+
+%s
+
+------------------------------------------------------------------------
+To see and reply to the post online, follow this link:
+%s
+
+To unsubscribe from this %s, visit:
+%s";
 $string['forumsuccessfulsubscribe'] = 'Forum subscribed successfully';
 $string['forumsuccessfulunsubscribe'] = 'Forum unsubscribed successfully';
-$string['groupowner'] = 'Group owner';
-$string['groupownerlist'] = 'Group owner:';
+$string['gotoforums'] = 'Go to forums';
+$string['groupadmins'] = 'Group administrators';
+$string['groupadminlist'] = 'Group admins:';
 $string['Key'] = 'Key';
 $string['lastpost'] = 'Last post';
+$string['latestforumposts'] = 'Latest Forum Posts';
 $string['Moderators'] = 'Moderators';
+$string['moderatorsandgroupadminsonly'] = 'Moderators and group admins only';
 $string['moderatorslist'] = 'Moderators:';
 $string['moderatorsdescription'] = 'Moderators can edit and delete topics and posts. They can also open, close, set and unset topics as sticky';
 $string['name'] = 'Forum';
 $string['nameplural'] = 'Forums';
 $string['newforum'] = 'New forum';
-$string['newforumpostin'] = 'New forum post in %s';
+$string['newforumpostnotificationsubject'] = '%s: %s: %s';
 $string['newpost'] = 'New post: ';
 $string['newtopic'] = 'New topic';
+$string['noforumpostsyet'] = 'There are no posts in this group yet';
 $string['noforums'] = 'There are no forums in this group';
 $string['notopics'] = 'There are no topics in this forum';
 $string['Open'] = 'Open';
@@ -85,6 +116,8 @@ $string['Order'] = 'Order';
 $string['orderdescription'] = 'Choose where you want the forum to be ordered compared to the other forums';
 $string['Post'] = 'Post';
 $string['postbyuserwasdeleted'] = 'A post by %s was deleted';
+$string['postdelay'] = 'Post Delay';
+$string['postdelaydescription'] = 'The minimum time (in minutes) that must pass before a new post can be mailed out to forum subscribers.  The author of a post may make edits during this time.';
 $string['postedin'] = '%s posted in %s';
 $string['Poster'] = 'Poster';
 $string['postreply'] = 'Post reply';
@@ -94,6 +127,7 @@ $string['potentialmoderators'] = 'Potential Moderators';
 $string['re'] ='Re: %s';
 $string['regulartopics'] = 'Regular topics';
 $string['Reply'] = 'Reply';
+$string['replyforumpostnotificationsubject'] = 'Re: %s: %s: %s';
 $string['replyto'] = 'Reply to: ';
 $string['Sticky'] = 'Sticky';
 $string['stickydescription'] = 'Sticky topics are at the top of every page';
@@ -104,13 +138,15 @@ $string['subscribetoforum'] = 'Subscribe to forum';
 $string['subscribetotopic'] = 'Subscribe to topic';
 $string['Subject'] = 'Subject';
 $string['Topic'] = 'Topic';
+$string['Topics'] = 'Topics';
 $string['topiclower'] = 'topic';
 $string['topicslower'] = 'topics';
 $string['topicclosedsuccess'] = 'Topics closed successfully';
-$string['topicisclosed'] = 'This topic is closed. Only moderators and the group owner can post new replies';
+$string['topicisclosed'] = 'This topic is closed. Only moderators and the group admins can post new replies';
 $string['topicopenedsuccess'] = 'Topics opened successfully';
 $string['topicstickysuccess'] = 'Topics set as sticky successfully';
 $string['topicsubscribesuccess'] = 'Topics subscribed successfully';
+$string['topicsuccessfulunsubscribe'] = 'Topic unsubscribed successfully';
 $string['topicunstickysuccess'] = 'Topic unset as sticky successfully';
 $string['topicunsubscribesuccess'] = 'Topics unsubscribed successfully';
 $string['topicupdatefailed'] = 'Topics update failed';
@@ -120,6 +156,10 @@ $string['Unsubscribe'] = 'Unsubscribe';
 $string['unsubscribefromforum'] = 'Unsubscribe from forum';
 $string['unsubscribefromtopic'] = 'Unsubscribe from topic';
 $string['updateselectedtopics'] = 'Update selected topics';
+$string['whocancreatetopics'] = 'Who can create topics';
+$string['youcannotunsubscribeotherusers'] = 'You cannot unsubscribe other users';
+$string['youarenotsubscribedtothisforum'] = 'You are not subscribed to this forum';
+$string['youarenotsubscribedtothistopic'] = 'You are not subscribed to this topic';
 
 $string['today'] = 'Today';
 $string['yesterday'] = 'Yesterday';
