@@ -80,6 +80,7 @@ function ViewManager() {
         // Now we're done, remove the loading message and display the page
         removeElement('views-loading');
         showElement(self.bottomPane);
+
     }
 
     /**
@@ -256,53 +257,153 @@ function ViewManager() {
     this.rewriteConfigureButton = function(button) {
         connect(button, 'onclick', function(e) {
             e.stop();
-            setNodeAttribute(button, 'disabled', 'disabled');
+            self.getConfigureForm(getFirstParentByTagAndClassName(button, 'div', 'blockinstance'));
+        });
+    }
 
-            // If there is a configuration form open, close it. This is because
-            // each one shares the same form tag - the one for the entire form.
-            // We might be able to support having more than one form open at
-            // any one time, as long as we can detect precisely which form was
-            // submitted
-            if (self.currentConfigureData) {
-                self.currentConfigureData['contentdiv'].innerHTML = self.currentConfigureData['oldcontent'];
-                removeNodeAttribute(self.currentConfigureData['button'], 'disabled');
-                self.currentConfigureData = null;
-            }
 
-            var pd = {'id': $('viewid').value, 'change': 1};
-            pd[getNodeAttribute(e.src(), 'name')] = 1;
+    this.getConfigureForm = function(blockinstance) {
+        var button = getFirstElementByTagAndClassName('input', 'configurebutton', blockinstance);
 
-            var blockinstance = getFirstParentByTagAndClassName(button, 'div', 'blockinstance');
-            var blockinstanceId = blockinstance.id.substr(blockinstance.id.lastIndexOf('_') + 1);
-            var contentDiv = getFirstElementByTagAndClassName('div', 'blockinstance-content', blockinstance);
-            var oldContent = contentDiv.innerHTML;
+        var blockinstanceId = blockinstance.id.substr(blockinstance.id.lastIndexOf('_') + 1);
+        var contentDiv = getFirstElementByTagAndClassName('div', 'blockinstance-content', blockinstance);
 
-            // Put a loading message in place while the form downloads
-            replaceChildNodes(contentDiv, IMG({'src': config.theme['images/loading.gif']}), ' ', get_string('loading'));
+        var pd = {'id': $('viewid').value, 'change': 1};
+        pd[getNodeAttribute(button, 'name')] = 1;
 
-            sendjsonrequest('blocks.json.php', pd, 'POST', function(data) {
-                self.currentConfigureData = {
-                    'contentdiv': contentDiv,
-                    'oldcontent': oldContent,
-                    'button'    : button
-                };
-                contentDiv.innerHTML = data.data['html'];
-                eval(data.data.javascript);
-                $('action-dummy').name = getNodeAttribute(e.src(), 'name');
+        var oldContent = contentDiv.innerHTML;
 
-                // Make the cancel button be supersmart
-                var cancelButton = $('cancel_cb_' + blockinstanceId + '_action_configureblockinstance_id_' + blockinstanceId);
-                connect(cancelButton, 'onclick', function(e) {
-                    e.stop();
-                    contentDiv.innerHTML = oldContent;
-                    self.currentConfigureData = null;
-                    removeNodeAttribute(button, 'disabled');
-                });
+        // Put a loading message in place while the form downloads
+        replaceChildNodes(contentDiv, IMG({'src': config.theme['images/loading.gif']}), ' ', get_string('loading'));
 
-            }, function() {
-                removeNodeAttribute(button, 'disabled');
+        sendjsonrequest('blocks.json.php', pd, 'POST', function(data) {
+            contentDiv.innerHTML = oldContent;
+            self.addConfigureBlock(blockinstance, data.data);
+            $('action-dummy').name = getNodeAttribute(button, 'name');
+
+            var cancelButton = $('cancel_instconf_action_configureblockinstance_id_' + blockinstanceId);
+            connect(cancelButton, 'onclick', function(e) {
+                e.stop();
+                self.removeConfigureBlocks();
+                self.showMediaPlayers();
+            });
+
+        });
+    }
+
+
+    this.hideMediaPlayers = function () {
+        var cols = $('column-container');
+        forEach(getElementsByTagAndClassName(null, 'mediaplayer-container', cols), function (e) {
+            var d = getElementDimensions(e);
+            setStyle(e, {'height': d.h+'px'});
+            hideElement(getFirstElementByTagAndClassName(null, 'mediaplayer', e));
+            forEach(getElementsByTagAndClassName('object', null, e), function(o) {
+                addElementClass(o, 'in-mediaplayer');
             });
         });
+
+        // Try to find and hide players floating around in text blocks, etc. by looking for object elements
+        forEach(getElementsByTagAndClassName('object', null, cols), function (e) {
+            if (!hasElementClass(e, 'in-mediaplayer')) {
+                var d = getElementDimensions(e);
+                var temp = DIV({'class': 'hidden mediaplayer-placeholder'});
+                setStyle(temp, {'height': d.h+'px'});
+                insertSiblingNodesAfter(e, temp);
+                addElementClass(e, 'hidden');
+                removeElementClass(temp, 'hidden');
+            }
+        });
+
+        insertSiblingNodesBefore(document.body.firstChild, DIV({'id': 'overlay'}));
+    }
+
+
+    this.showMediaPlayers = function () {
+        if (tinyMCE && tinyMCE.activeEditor && tinyMCE.activeEditor.editorId) {
+            tinyMCE.execCommand('mceRemoveControl', false, tinyMCE.activeEditor.editorId);
+        }
+        var cols = $('column-container');
+        forEach(getElementsByTagAndClassName(null, 'mediaplayer-container', cols), function (e) {
+            showElement(getFirstElementByTagAndClassName(null, 'mediaplayer', e));
+            setStyle(e, {'height': 'auto'});
+        });
+        forEach(getElementsByTagAndClassName(null, 'mediaplayer-placeholder', cols), function (e) {
+            addElementClass(e, 'hidden');
+            removeElementClass(e.previousSibling, 'hidden');
+            removeElement(e);
+        });
+        removeElement('overlay');
+    }
+
+
+    this.addConfigureBlock = function(oldblock, configblock, removeoncancel) {
+        self.hideMediaPlayers();
+
+        var temp = DIV();
+        temp.innerHTML = configblock.html;
+        var newblock = getFirstElementByTagAndClassName('div', 'blockinstance', temp);
+        hideElement(newblock);
+        appendChildNodes(getFirstElementByTagAndClassName('body'), newblock);
+
+        var d = getElementDimensions(newblock);
+        var vpdim = getViewportDimensions();
+        var newtop = getViewportPosition().y + Math.max((vpdim.h - d.h) / 2, 5);
+        setStyle(newblock, {
+            'width': d.w + 'px',
+            'left': (vpdim.w - d.w) / 2 + 'px',
+            'top': newtop + 'px',
+            'position': 'absolute',
+            'z-index': 1
+        });
+
+        var deletebutton = getFirstElementByTagAndClassName('input', 'deletebutton', newblock);
+
+        if (removeoncancel) {
+            self.rewriteDeleteButton(deletebutton);
+
+            var oldblockid = newblock.id.substr(0, newblock.id.length - '_configure'.length);
+            var blockinstanceId = oldblockid.substr(oldblockid.lastIndexOf('_') + 1);
+            var cancelbutton = $('cancel_instconf_action_configureblockinstance_id_' + blockinstanceId);
+            if (cancelbutton) {
+                setNodeAttribute(cancelbutton, 'name', getNodeAttribute(deletebutton, 'name'));
+                disconnectAll(cancelbutton);
+                self.rewriteCancelButton(cancelbutton, blockinstanceId);
+            }
+        }
+        else {
+            disconnectAll(deletebutton);
+            connect(deletebutton, 'onclick', function(e) {
+                e.stop();
+                self.removeConfigureBlocks();
+                self.showMediaPlayers();
+            });
+        }
+
+        showElement(newblock);
+        eval(configblock.javascript);
+    }
+
+
+    this.replaceConfigureBlock = function(data) {
+        var oldblock = $('blockinstance_' + data.blockid);
+        if (oldblock) {
+            var temp = DIV();
+            temp.innerHTML = data.data.html;
+            var newblock = getFirstElementByTagAndClassName('div', 'blockinstance', temp)
+            swapDOM(oldblock, newblock);
+            eval(data.data.javascript);
+            self.makeBlockinstanceDraggable(newblock);
+            self.rewriteConfigureButton(getFirstElementByTagAndClassName('input', 'configurebutton', newblock));
+            self.rewriteDeleteButton(getFirstElementByTagAndClassName('input', 'deletebutton', newblock));
+        }
+        self.removeConfigureBlocks();
+        self.showMediaPlayers();
+    }
+
+    this.removeConfigureBlocks = function() {
+        // FF3 hangs unless you delay removal of the iframe inside the old configure block
+        callLater(0.0001, function () { forEach(getElementsByTagAndClassName('div', 'configure'), removeElement); });
     }
 
     /**
@@ -324,8 +425,12 @@ function ViewManager() {
                 var pd = {'id': $('viewid').value, 'change': 1};
                 pd[getNodeAttribute(e.src(), 'name')] = 1;
                 sendjsonrequest(config['wwwroot'] + 'view/blocks.json.php', pd, 'POST', function(data) {
-                    removeElement(getFirstParentByTagAndClassName(button, 'div', 'blockinstance'));
-                    removeNodeAttribute(button, 'disabled');
+                    var blockinstanceId = button.name.substr(button.name.lastIndexOf('_') + 1);
+                    removeElement('blockinstance_' + blockinstanceId);
+                    if ($('blockinstance_' + blockinstanceId + '_configure')) {
+                        self.removeConfigureBlocks();
+                        self.showMediaPlayers();
+                    }
                 }, function() {
                     removeNodeAttribute(button, 'disabled');
                 });
@@ -337,6 +442,25 @@ function ViewManager() {
         });
     }
 
+
+    /**
+     * Rewrites cancel button to remove a block
+     */
+    this.rewriteCancelButton = function(button, blockinstanceId) {
+        connect(button, 'onclick', function(e) {
+            var pd = {'id': $('viewid').value, 'change': 1};
+            pd[getNodeAttribute(e.src(), 'name')] = 1;
+            sendjsonrequest(config['wwwroot'] + 'view/blocks.json.php', pd, 'POST', function(data) {
+                removeElement('blockinstance_' + blockinstanceId);
+                if ($('blockinstance_' + blockinstanceId + '_configure')) {
+                    self.removeConfigureBlocks();
+                    self.showMediaPlayers();
+                }
+            });
+            e.stop();
+        });
+    }
+
     /**
      * Rewrites the add column buttons to be AJAX
      *
@@ -764,8 +888,8 @@ function ViewManager() {
                 var columnContainerPosition   = elementPosition(self.columnContainer);
                 var columnContainerDimensions = elementDimensions(self.columnContainer);
 
-                var footerDimensions = getElementDimensions('footer', self.bottomPane);
-                var footerPosition   = getElementPosition('footer', self.bottomPane);
+                var footerDimensions = getElementDimensions('footer-wrap', self.bottomPane);
+                var footerPosition   = getElementPosition('footer-wrap', self.bottomPane);
 
                 setElementPosition(hotzone, {x: blockinstancePosition.x, y: previousHotzonePosition.y + previousHotzoneDimensions.h});
                 setElementDimensions(hotzone, {
@@ -783,8 +907,8 @@ function ViewManager() {
                 var columnContent = getFirstElementByTagAndClassName('div', 'column-content', i);
                 var hotzone = self.createHotzone(columnContent, appendChildNodes, true);
 
-                var footerDimensions = getElementDimensions('footer', self.bottomPane);
-                var footerPosition   = getElementPosition('footer', self.bottomPane);
+                var footerDimensions = getElementDimensions('footer-wrap', self.bottomPane);
+                var footerPosition   = getElementPosition('footer-wrap', self.bottomPane);
 
                 setElementPosition(hotzone, {x: getElementPosition(columnContent, self.columnContainer).x, y: 0});
                 setElementDimensions(hotzone, {
@@ -842,33 +966,23 @@ function ViewManager() {
                 pd['action_addblocktype_column_' + whereTo['column'] + '_order_' + whereTo['order']] = true;
                 sendjsonrequest(config['wwwroot'] + 'view/blocks.json.php', pd, 'POST', function(data) {
                     var div = DIV();
-                    div.innerHTML = data.data.html;
+                    div.innerHTML = data.data.display.html;
                     var blockinstance = getFirstElementByTagAndClassName('div', 'blockinstance', div);
-
                     // Make configure button clickable, but disabled as blocks are rendered in configure mode by default
                     var configureButton = getFirstElementByTagAndClassName('input', 'configurebutton', blockinstance);
                     if (configureButton) {
                         self.rewriteConfigureButton(configureButton);
-                        if (self.currentConfigureData) {
-                            self.currentConfigureData['contentdiv'].innerHTML = self.currentConfigureData['oldcontent'];
-                            removeNodeAttribute(self.currentConfigureData['button'], 'disabled');
-                            self.currentConfigureData = null;
-                        }
-                        self.currentConfigureData = {
-                            'contentdiv': getFirstElementByTagAndClassName('div', 'blockinstance-content', blockinstance),
-                            'oldcontent': '',
-                            'button'    : configureButton
-                        };
                         $('action-dummy').name = 'action_addblocktype_column_' + whereTo['column'] + '_order_' + whereTo['order'];
-                        setNodeAttribute(configureButton, 'disabled', 'disabled');
                     }
 
                     self.rewriteDeleteButton(getFirstElementByTagAndClassName('input', 'deletebutton', blockinstance));
                     self.makeBlockinstanceDraggable(blockinstance);
                     insertSiblingNodesAfter(self.blockPlaceholder, blockinstance);
                     removeElement(self.blockPlaceholder);
-                    eval(data.data.javascript);
 
+                    if (data.data.configure) {
+                        self.addConfigureBlock(blockinstance, data.data.configure, true);
+                    }
                 });
 
             };
@@ -989,10 +1103,6 @@ function ViewManager() {
     // it is dropped. Needs a margin the same as the blockinstances
     this.blockPlaceholder = DIV({'id': 'block-placeholder'});
 
-    // A container for information about the blockinstance current being
-    // configured
-    this.currentConfigureData = null;
-
     // The column container - set in self.init
     this.columnContainer = null;
 
@@ -1010,3 +1120,23 @@ function ViewManager() {
 
 viewManager = new ViewManager();
 viewManager.addCSSRules();
+
+function blockConfigSuccess(form, data) {
+    if (data.formelementsuccess) {
+        eval(data.formelementsuccess + '(form, data)');
+    }
+    if (data.blockid) {
+        if (viewManager.isIE6 && data.viewid) {
+            document.location.href = config['wwwroot'] + 'view/blocks.php?id=' + data.viewid;
+        }
+        viewManager.replaceConfigureBlock(data);
+    }
+}
+
+function blockConfigError(form, data) {
+    if (data.formelementerror) {
+        eval(data.formelementerror + '(form, data)');
+        return;
+    }
+    formError(form, data);
+}
