@@ -1,7 +1,8 @@
 <?php
 /**
  * Mahara: Electronic portfolio, weblog, resume builder and social networking
- * Copyright (C) 2006-2008 Catalyst IT Ltd (http://www.catalyst.net.nz)
+ * Copyright (C) 2006-2009 Catalyst IT Ltd and others; see:
+ *                         http://wiki.mahara.org/Contributors
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +21,7 @@
  * @subpackage core
  * @author     Catalyst IT Ltd
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL
- * @copyright  (C) 2006-2008 Catalyst IT Ltd http://catalyst.net.nz
+ * @copyright  (C) 2006-2009 Catalyst IT Ltd http://catalyst.net.nz
  *
  */
 
@@ -33,9 +34,9 @@ defined('INTERNAL') || die();
  * @param string $activitytype type of activity
  * @param mixed $data data 
  */
-function activity_occurred($activitytype, $data, $plugintype=null, $pluginname=null) {
+function activity_occurred($activitytype, $data, $plugintype=null, $pluginname=null, $overridedelay=false) {
     $at = activity_locate_typerecord($activitytype, $plugintype, $pluginname);
-    if (!empty($at->delay)) {
+    if (!empty($at->delay) && !$overridedelay) {
         $delayed = new StdClass;
         $delayed->type = $at->id;
         $delayed->data = serialize($data);
@@ -48,23 +49,13 @@ function activity_occurred($activitytype, $data, $plugintype=null, $pluginname=n
 }
 
 /** 
- * This function dispatches all the activity stuff
- * to whatever notification plugin it needs to
- * and figures out all the implications of 
- * activity and who needs to know about it.
+ * This function dispatches all the activity stuff to whatever notification 
+ * plugin it needs to, and figures out all the implications of activity and who 
+ * needs to know about it.
  * 
- * @param object $activitytype record from activity_type
+ * @param object $activitytype record from database table activity_type
  * @param mixed $data must contain message to save.
- * it can also contain url.
  * each activity type has different requirements of $data - 
- *  - <b>contactus</b> must contain $message, $subject (optional), $fromname, $fromaddress, $userfrom (if a logged in user)
- *  - <b>objectionable</b> must contain $message, $view and $artefact if applicable
- *  - <b>maharamessage</b> must contain $users, an array of userids. $subject and $message (contents of message)
- *  - <b>usermessage</b> must contain $userto, id of recipient user, $userfrom, id of user from 
-    -       and $subject and $message (contents of message)
- *  - <b>feedback (artefact)</b> must contain both $artefact (id) and $view (id) and $message 
- *  - <b>feedback (view)</b> must contain $view (id) and $message
- *  - <b>watchlist (view) </b> must contain $view (id of view) as $message
  *  - <b>viewaccess</b> must contain $owner userid of view owner AND $view (id of view) and $oldusers array of userids before access change was committed.
  */
 function handle_activity($activitytype, $data, $cron=false) {
@@ -105,16 +96,22 @@ function activity_get_users($activitytype, $userids=null, $userobjs=null, $admin
     $sql = '
         SELECT
             u.id, u.username, u.firstname, u.lastname, u.preferredname, u.email, u.admin, u.staff, 
-            p.method, ap.value AS lang
+            p.method, ap.value AS lang, aic.value AS mnethostwwwroot, h.appname AS mnethostapp
         FROM {usr} u
         LEFT JOIN {usr_activity_preference} p
-            ON p.usr = u.id ' . (empty($admininstitutions) ? '' : '
+            ON (p.usr = u.id AND p.activity = ?)' . (empty($admininstitutions) ? '' : '
         LEFT OUTER JOIN {usr_institution} ui
             ON (u.id = ui.usr
                 AND ui.institution IN ('.join(',',array_map('db_quote',$admininstitutions)).'))') . '
         LEFT OUTER JOIN {usr_account_preference} ap
             ON (ap.usr = u.id AND ap.field = \'lang\')
-        WHERE (p.activity = ? ' . ($adminonly ? '' : ' OR p.activity IS NULL') . ')';
+        LEFT OUTER JOIN {auth_instance} ai
+            ON (ai.id = u.authinstance AND ai.authname = \'xmlrpc\')
+        LEFT OUTER JOIN {auth_instance_config} aic
+            ON (aic.instance = ai.id AND aic.field = \'wwwroot\')
+        LEFT OUTER JOIN {host} h
+            ON aic.value = h.wwwroot
+        WHERE TRUE';
     if (!empty($userobjs) && is_array($userobjs)) {
         $sql .= ' AND u.id IN (' . implode(',',db_array_to_ph($userobjs)) . ')';
         $values = array_merge($values, array_to_fields($userobjs));
@@ -127,7 +124,7 @@ function activity_get_users($activitytype, $userids=null, $userobjs=null, $admin
         $sql .= '
         GROUP BY
             u.id, u.username, u.firstname, u.lastname, u.preferredname, u.email, u.admin, u.staff,
-            p.method, ap.value
+            p.method, ap.value, aic.value, h.appname
         HAVING (u.admin = 1 OR SUM(ui.admin) > 0)';
     } else if ($adminonly) {
         $sql .= ' AND u.admin = 1';
@@ -135,21 +132,30 @@ function activity_get_users($activitytype, $userids=null, $userobjs=null, $admin
     return get_records_sql_array($sql, $values);
 }
 
-/**
- * this function inserts a default set of activity preferences for a given user
- * id
- */
-function activity_set_defaults($user_id) {
-    $activitytypes = get_records_array('activity_type', 'admin', 0);
-    $haveemail = in_array('email', array_map(create_function('$a', 'return $a->name;'),
-                                             plugins_installed('notification')));
-    foreach ($activitytypes as $type) {
-        if ($haveemail) {
+
+function activity_default_notification_method() {
+    static $method = null;
+    if (is_null($method)) {
+        if (in_array('email', array_map(create_function('$a', 'return $a->name;'), plugins_installed('notification')))) {
             $method = 'email';
         }
         else {
             $method = 'internal';
         }
+    }
+    return $method;
+}
+
+/**
+ * this function inserts a default set of activity preferences for a given user
+ * id
+ */
+function activity_set_defaults($eventdata) {
+    $user_id = $eventdata['id'];
+    $activitytypes = get_records_array('activity_type', 'admin', 0);
+    $method = activity_default_notification_method();
+
+    foreach ($activitytypes as $type) {
         insert_record('usr_activity_preference', (object)array(
             'usr' => $user_id,
             'activity' => $type->id,
@@ -161,13 +167,15 @@ function activity_set_defaults($user_id) {
 
 function activity_add_admin_defaults($userids) {
     $activitytypes = get_records_array('activity_type', 'admin', 1);
+    $method = activity_default_notification_method();
+
     foreach ($activitytypes as $type) {
         foreach ($userids as $id) {
             if (!record_exists('usr_activity_preference', 'usr', $id, 'activity', $type->id)) {
                 insert_record('usr_activity_preference', (object)array(
                     'usr' => $id,
                     'activity' => $type->id,
-                    'method' => 'internal',
+                    'method' => $method,
                 ));
             }
         }
@@ -220,16 +228,12 @@ function activity_get_viewaccess_users($view, $owner, $type) {
                     FROM {group_member} m
                     JOIN {view_access_group} vg ON vg.group = m.group
                     JOIN {group} g ON (g.id = vg.group AND g.deleted = 0)
-                        WHERE vg.view = ? AND (vg.tutoronly = ? OR m.tutor = ?)
-                UNION SELECT g.owner
-                    FROM {group} g
-                    JOIN {view_access_group} ag ON ag.group = g.id
-                        WHERE ag.view = ? AND g.deleted = 0
+                        WHERE vg.view = ? AND (vg.role IS NULL OR vg.role = m.role)
                 ) AS userlist
                 JOIN {usr} u ON u.id = userlist.userid
                 LEFT JOIN {usr_activity_preference} p ON p.usr = u.id AND p.activity = ?
                 LEFT JOIN {usr_account_preference} ap ON ap.usr = u.id AND ap.field = 'lang'";
-    $values = array($owner, $owner, $owner, 'friends', $view, $view, $view, 0, 1, $view, $type->id);
+    $values = array($owner, $owner, $owner, 'friends', $view, $view, $view, $type->id);
     if (!$u = get_records_sql_assoc($sql, $values)) {
         $u = array();
     }
@@ -259,9 +263,32 @@ function activity_locate_typerecord($activitytype, $plugintype=null, $pluginname
     return $at;
 }
 
+function generate_activity_class_name($name, $plugintype, $pluginname) {
+    if (!empty($plugintype)) {
+        safe_require($plugintype, $pluginname);
+        return 'ActivityType' .
+            ucfirst($plugintype) .
+            ucfirst($pluginname) .
+            ucfirst($name);
+    }
+    return 'ActivityType' . $name;
+}
+
 /** activity type classes **/
 abstract class ActivityType {
     
+    /**
+     * Who any notifications about this activity should appear to come from
+     */
+    protected $fromuser;
+
+    /**
+     * When sending notifications, should the email of the person sending it be 
+     * hidden? (Almost always yes, will cause the email to appear to come from 
+     * the 'noreply' address)
+     */
+    protected $hideemail = true;
+
     protected $subject;
     protected $message;
     protected $users = array();
@@ -270,6 +297,7 @@ abstract class ActivityType {
     protected $type;
     protected $activityname;
     protected $cron;
+    protected $overridemessagecontents;
    
     public function get_id() {
         if (!isset($this->id)) {
@@ -327,9 +355,16 @@ abstract class ActivityType {
         return $this->subject;
     }
 
+    // rewrite the url with the internal notification id?
+    protected function update_url() {
+        return false;
+    }
+
     abstract function get_required_parameters();
 
     public function notify_user($user) {
+        $changes = new stdClass;
+
         $userdata = $this->to_stdclass();
         // some stuff gets overridden by user specific stuff
         if (!empty($user->url)) {
@@ -341,32 +376,49 @@ abstract class ActivityType {
         $userdata->message = $this->get_message($user);
         $userdata->subject = $this->get_subject($user);
         if (empty($user->method)) {
-            $user->method = 'internal';
+            $user->method = call_static_method(get_class($this), 'default_notification_method');
         }
+
+        // always do internal
+        $userdata->internalid = call_static_method('PluginNotificationInternal', 'notify_user', $user, $userdata);
+        if ($this->update_url($userdata->internalid)) {
+            $changes->url = $userdata->url = $this->url;
+        }
+
+        if ($user->method != 'internal' || isset($changes->url)) {
+            $changes->read = (int) ($user->method != 'internal');
+            $changes->id = $userdata->internalid;
+            update_record('notification_internal_activity', $changes);
+        }
+
         if ($user->method != 'internal') {
             $method = $user->method;
-            safe_require('notification', $method, 'lib.php', 'require_once');
+            safe_require('notification', $method);
             try {
                 call_static_method(generate_class_name('notification', $method), 'notify_user', $user, $userdata);
-                $user->markasread = true; // if we're doing something else, don't generate unread internal ones.
             }
-            catch (Exception $e) {
-                $user->markasread = false; // if we fail (eg email falls over), don't mark it as read...
-                // @todo Catalyst IT Ltd
+            catch (MaharaException $e) {
+                // We don't mind other notification methods failing, as it'll 
+                // go into the activity log as 'unread'
+                $changes->read = 0;
+                update_record('notification_internal_activity', $changes);
             }
         }
-        // always do internal
-        call_static_method('PluginNotificationInternal', 'notify_user', $user, $userdata);
+
     }
 
     public function notify_users() {
-        safe_require('notification', 'internal', 'lib.php', 'require_once');
+        safe_require('notification', 'internal');
         $this->type = $this->get_id();
 
         foreach ($this->get_users() as $user) {
             $this->notify_user($user);
         }
     }
+
+    public static function default_notification_method() {
+        return activity_default_notification_method();
+    }
 }
 
 
@@ -382,12 +434,25 @@ class ActivityTypeContactus extends ActivityTypeAdmin {
     
     protected $fromname;
     protected $fromemail;
-    protected $userfrom;
+    protected $hideemail = false;
 
+    /**
+     * @param array $data Parameters:
+     *                    - message (string)
+     *                    - subject (string) (optional)
+     *                    - fromname (string)
+     *                    - fromaddress (email address)
+     *                    - fromuser (int) (if a logged in user)
+     */
     function __construct($data, $cron=false) { 
         parent::__construct($data, $cron);
-        if (!empty($this->userfrom)) {
-            $this->url = get_config('wwwroot') . 'user/view.php?id=' . $this->userfrom;
+        if (!empty($this->fromuser)) {
+            $this->url = get_config('wwwroot') . 'user/view.php?id=' . $this->fromuser;
+        }
+        else {
+            $this->customheaders = array(
+                'Reply-to: ' . $this->fromname . ' <' . $this->fromemail . '>',
+            );
         }
     }
 
@@ -397,7 +462,7 @@ class ActivityTypeContactus extends ActivityTypeAdmin {
 
     function get_message($user) {
         return get_string_from_language($user->lang, 'newcontactusfrom', 'activity') . ' ' . $this->fromname 
-            . '<' . $this->fromemail .'>' . (isset($this->subject) ? ': ' . $this->subject : '')
+            . ' <' . $this->fromemail .'>' . (isset($this->subject) ? ': ' . $this->subject : '')
             . "\n\n" . $this->message;
     }
 
@@ -412,6 +477,13 @@ class ActivityTypeObjectionable extends ActivityTypeAdmin {
     protected $artefact;
     protected $reporter;
 
+    /**
+     * @param array $data Parameters:
+     *                    - message (string)
+     *                    - view (int)
+     *                    - artefact (int) (optional)
+     *                    - reporter (int)
+     */
     function __construct($data, $cron=false) { 
         parent::__construct($data, $cron);
         if (empty($this->artefact)) {
@@ -482,6 +554,12 @@ class ActivityTypeVirusRelease extends ActivityTypeAdmin {
 
 class ActivityTypeMaharamessage extends ActivityType {
 
+    /**
+     * @param array $data Parameters:
+     *                    - subject (string)
+     *                    - message (string)
+     *                    - users (list of user ids)
+     */
     public function __construct($data, $cron=false) { 
         parent::__construct($data, $cron);
         $this->users = activity_get_users($this->get_id(), $this->users);
@@ -540,12 +618,16 @@ class ActivityTypeUsermessage extends ActivityType {
     protected $userto;
     protected $userfrom;
 
+    /**
+     * @param array $data Parameters:
+     *                    - userto (int)
+     *                    - userfrom (int)
+     *                    - subject (string)
+     *                    - message (string)
+     */
     public function __construct($data, $cron=false) { 
         parent::__construct($data, $cron);
         $this->users = activity_get_users($this->get_id(), array($this->userto));
-        if (empty($this->url)) {
-            $this->url = get_config('wwwroot') . 'user/view.php?id=' . $this->userfrom;
-        }
     } 
 
     public function get_subject($user) {
@@ -556,6 +638,11 @@ class ActivityTypeUsermessage extends ActivityType {
         return $this->subject;
     }
 
+    protected function update_url($internalid) {
+        $this->url = get_config('wwwroot') . 'user/sendmessage.php?id=' . $this->userfrom . '&replyto=' . $internalid;
+        return true;
+    }
+
     public function get_required_parameters() {
         return array('message', 'userto', 'userfrom');
     }
@@ -570,6 +657,12 @@ class ActivityTypeFeedback extends ActivityType {
     private $viewrecord;
     private $artefactinstance;
 
+    /**
+     * @param array $data Parameters:
+     *                    - view (int)
+     *                    - artefact (int) (optional)
+     *                    - message (string)
+     */
     public function __construct($data, $cron=false) { 
         parent::__construct($data, $cron);
 
@@ -621,6 +714,10 @@ class ActivityTypeWatchlist extends ActivityType {
 
     private $viewinfo;
 
+    /**
+     * @param array $data Parameters:
+     *                    - view (int)
+     */
     public function __construct($data, $cron) { 
         parent::__construct($data, $cron); 
         //$oldsubject = $this->subject;
@@ -713,6 +810,12 @@ class ActivityTypeViewaccess extends ActivityType {
 
     private $viewinfo;
 
+    /**
+     * @param array $data Parameters:
+     *                    - owner (int)
+     *                    - view (int)
+     *                    - oldusers (array of user IDs)
+     */
     public function __construct($data, $cron=false) { 
         parent::__construct($data, $cron);
         if (!$this->viewinfo = get_record_sql('SELECT u.*, v.title FROM {usr} u
@@ -744,6 +847,57 @@ class ActivityTypeViewaccess extends ActivityType {
     }
 }
 
+class ActivityTypeGroupMessage extends ActivityType {
+
+    protected $group;
+    protected $roles;
+    protected $submittedview;
+    private $viewinfo;
+    private $groupinfo;
+
+    /**
+     * @param array $data Parameters:
+     *                    - subject (string)
+     *                    - message (string)
+     *                    - group (integer)
+     *                    - roles (list of roles)
+     */
+    public function __construct($data, $cron=false) {
+        require_once('group.php');
+
+        parent::__construct($data, $cron);
+
+        $this->groupinfo = get_record('group', 'id', $this->group);
+
+        $members = group_get_member_ids($this->group, isset($this->roles) ? $this->roles : null);
+        $this->users = activity_get_users($this->get_id(), $members);
+
+        if ($this->submittedview) {
+            $this->viewinfo = get_record('view', 'id', $this->submittedview);
+            $this->viewinfo->ownername = display_name($this->viewinfo->owner);
+            $this->url = get_config('wwwroot') . 'view/view.php?id=' . $this->submittedview;
+        }
+    }
+
+    public function get_subject($user) {
+        if ($this->submittedview) {
+            return get_string_from_language($user->lang, 'viewsubmittedsubject', 'activity', $this->groupinfo->name);
+        }
+        return $this->subject;
+    }
+
+    public function get_message($user) {
+        if ($this->submittedview) {
+            return get_string_from_language($user->lang, 'viewsubmittedmessage', 'activity', $this->viewinfo->ownername, $this->viewinfo->title, $this->groupinfo->name);
+        }
+        return $this->subject;
+    }
+
+    public function get_required_parameters() {
+        return array('message', 'subject', 'group');
+    }
+}
+
 abstract class ActivityTypePlugin extends ActivityType {
 
     abstract public function get_plugintype();
