@@ -1,7 +1,8 @@
 <?php
 /**
  * Mahara: Electronic portfolio, weblog, resume builder and social networking
- * Copyright (C) 2006-2008 Catalyst IT Ltd (http://www.catalyst.net.nz)
+ * Copyright (C) 2006-2009 Catalyst IT Ltd and others; see:
+ *                         http://wiki.mahara.org/Contributors
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +21,7 @@
  * @subpackage core
  * @author     Catalyst IT Ltd
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL
- * @copyright  (C) 2006-2008 Catalyst IT Ltd http://catalyst.net.nz
+ * @copyright  (C) 2006-2009 Catalyst IT Ltd http://catalyst.net.nz
  *
  */
 
@@ -34,6 +35,13 @@ defined('INTERNAL') || die();
 define('LOG_TARGET_SCREEN', 1);
 /** Write the errors to the error log, as specified in your php configuration */
 define('LOG_TARGET_ERRORLOG', 2);
+/** Write the error to stdout (using echo) */
+define('LOG_TARGET_STDOUT', 4);
+/** Display the errors on the screen in the admin area only (short term hack
+    until we create an admin notifications page) */
+define('LOG_TARGET_ADMIN', 8);
+/** Log to a specific file */
+define('LOG_TARGET_FILE', 16);
 
 // Logging levels
 /** Environment type errors, such as register_globals being on */
@@ -44,8 +52,21 @@ define('LOG_LEVEL_DBG', 2);
 define('LOG_LEVEL_INFO', 4);
 /** Warnings */
 define('LOG_LEVEL_WARN', 8);
+
+// developermodes,  also bitmaps
+/** inlude debug.js */
+define('DEVMODE_DEBUGJS', 1);
+/** include debug.css */
+define('DEVMODE_DEBUGCSS', 2);
+/** include firebug lite */
+define('DEVMODE_FIREBUGLITE', 4);
+/** include unpacked mochikit */
+define('DEVMODE_UNPACKEDJS', 8);
+// more here.. start at 16 :)
+
 /**#@-*/
 
+
 // Tell PHP about our error settings
 error_reporting(E_ALL);
 set_error_handler('error');
@@ -179,7 +200,7 @@ function log_message ($message, $loglevel, $escape, $backtrace, $file=null, $lin
     }
     $prefix = '[' . str_pad(substr(strtoupper($loglevelnames[$loglevel]), 0, 3), 3) . '] ' . $prefix;
 
-    if ($targets & LOG_TARGET_SCREEN) {
+    if ($targets & LOG_TARGET_SCREEN || (defined('ADMIN') && $targets & LOG_TARGET_ADMIN)) {
         // Work out which method to call for displaying the message
         if ($loglevel == LOG_LEVEL_DBG || $loglevel == LOG_LEVEL_INFO) {
             $method = 'add_info_msg';
@@ -194,7 +215,7 @@ function log_message ($message, $loglevel, $escape, $backtrace, $file=null, $lin
             $message = str_replace('  ', '&nbsp; ', $message);
         }
         $message = nl2br($message);
-        $message = '<div style="font-family: monospace;">' . $prefix . $message . "</div>\n";
+        $message = '<div class="backtrace">' . $prefix . $message . "</div>\n";
         if (is_a($SESSION, 'Session')) {
             $SESSION->$method($message, false);
         }
@@ -225,6 +246,49 @@ function log_message ($message, $loglevel, $escape, $backtrace, $file=null, $lin
             }
         }
     }
+
+    if ($targets & LOG_TARGET_STDOUT) {
+        foreach ($loglines as $line) {
+            echo($prefix . $line . "\n");
+        }
+        if ($backtrace && $textbacktrace) {
+            echo $textbacktrace;
+        }
+    }
+
+    if (function_exists('get_config')) {
+        $logfilename = get_config('log_file');
+        if (($targets & LOG_TARGET_FILE) && $logfilename) {
+            global $LOGFILE_FH;
+            static $logfile_open_attempted = null;
+            if (!$logfile_open_attempted) {
+                $logfile_open_attempted = true;
+                $LOGFILE_FH = fopen($logfilename, 'wb');
+                if ($LOGFILE_FH !== false) {
+                    function _close_logfile() {
+                        global $LOGFILE_FH;
+                        fclose($LOGFILE_FH);
+                    }
+                    register_shutdown_function('_close_logfile');
+                }
+                else {
+                    error_log("Could not open your custom log file ($logfilename)");
+                }
+            }
+
+            if (is_resource($LOGFILE_FH)) {
+                foreach ($loglines as $line) {
+                    fwrite($LOGFILE_FH, $prefix . $line . "\n");
+                }
+                if ($backtrace && $textbacktrace) {
+                    $lines = explode("\n", $textbacktrace);
+                    foreach ($lines as $line) {
+                        fwrite($LOGFILE_FH, $line . "\n");
+                    }
+                }
+            }
+        }
+    }
 }
 
 /**
@@ -297,7 +361,7 @@ function log_build_backtrace($backtrace) {
     }
 
     $textmessage = "Call stack (most recent first):\n";
-    $htmlmessage = "<pre><strong>Call stack (most recent first):</strong>\n<ul>";
+    $htmlmessage = "<div class=\"backtrace\"><strong>Call stack (most recent first):</strong>\n<ul>";
 
     foreach ($calls as $call) {
         $textmessage .= "  * {$call['class']}{$call['type']}{$call['func']}({$call['args']}) at {$call['file']}:{$call['line']}\n";
@@ -305,7 +369,7 @@ function log_build_backtrace($backtrace) {
             . htmlspecialchars($call['type'], ENT_COMPAT, 'UTF-8') . '</span><span style="color:#339;">' . htmlspecialchars($call['func'], ENT_COMPAT, 'UTF-8')
             . '</span><span style="color:#060;">(</span><span style="color:#f00;">' . htmlspecialchars($call['args'], ENT_COMPAT, 'UTF-8') . '</span><span style="color:#060;">)</span> at <strong>' . htmlspecialchars($call['file'], ENT_COMPAT, 'UTF-8') . ':' . $call['line'] . '</strong></li>';
     }
-    $htmlmessage .= "</pre>\n";
+    $htmlmessage .= "</div>\n";
 
     return array($textmessage, $htmlmessage);
 }
@@ -317,7 +381,7 @@ function log_build_backtrace($backtrace) {
  * @todo this function should go away
  */
 function die_info($message) {
-    $smarty = smarty();
+    $smarty = smarty(array(), array(), array(), array('sidebars' => false));
     $smarty->assign('message', $message);
     $smarty->assign('type', 'info');
     $smarty->display('message.tpl');
@@ -350,6 +414,10 @@ function error ($code, $message, $file, $line, $vars) {
         E_USER_ERROR => 'User Error'
     );
 
+    if (defined('E_RECOVERABLE_ERROR')) {
+        $error_lookup[E_RECOVERABLE_ERROR] = 'Warning';
+    }
+
     if (!error_reporting()) {
         return;
     }
@@ -360,11 +428,15 @@ function error ($code, $message, $file, $line, $vars) {
 
     // Ignore errors from smarty templates, which happen all too often
     if (function_exists('get_config')) {
-        $compiledir = realpath(get_config('dataroot') . 'smarty/compile');
+        $compiledir = realpath(get_config('dataroot') . 'dwoo/compile');
 
         if (E_NOTICE == $code && substr($file, 0, strlen($compiledir)) == $compiledir) {
             return;
         }
+
+        if (E_NOTICE == $code && preg_match('#^' . quotemeta(get_config('docroot') . 'theme/') . '[a-z0-9-]+/pieforms#', $file)) {
+            return;
+        }
     }
 
     // Fix up the message, which is in HTML form
@@ -419,6 +491,13 @@ class MaharaException extends Exception {
 
     protected $log = true;
 
+    public function __construct($message, $code=0) {
+        parent::__construct($message, $code);
+        if (!defined('MAHARA_CRASHING')) {
+            define('MAHARA_CRASHING', true);
+        }
+    }
+
     public function get_string() {
         $args = func_get_args();
         if (function_exists('get_string')) {
@@ -485,11 +564,16 @@ class MaharaException extends Exception {
             exit;
         }
 
+        if (headers_sent()) {
+            echo '<span style="color: red;">ERROR - something bad happened after headers have been sent. Check the error log for more information.</span>';
+            die();
+        }
+
         $outputtitle = $this->get_string('title');
         $outputmessage = trim($this->render_exception());
 
         if (function_exists('smarty') && !$this instanceof ConfigSanityException) {
-            $smarty = smarty();
+            $smarty = smarty(array(), array(), array(), array('sidebars' => false));
             $smarty->assign('title', $outputtitle);
             $smarty->assign('message', $outputmessage);
             $smarty->display('error.tpl');
@@ -580,15 +664,15 @@ class SystemException extends MaharaException implements MaharaThrowable {
 class ConfigException extends MaharaException  implements MaharaThrowable {
      
     public function render_exception () {
-        return $this->getMessage();
+        return $this->get_string('message') . "\n\n" . $this->getMessage();
     }
 
     public function strings() {
         return array_merge(parent::strings(), 
-                           array('message' => $this->get_sitename() 
-                           . ' is misconfigured and this is causing problems. '
-                           . 'You probably need to contact an administrator to get this fixed.  '
-                           . ' Details, if any, follow:'));
+                           array('message' => 'The environment where ' . $this->get_sitename() 
+                           . ' is running is misconfigured and this is causing problems. '
+                           . 'You probably need to contact a server administrator to get this fixed. '
+                           . 'Details, if any, follow:'));
     }
 }
 
@@ -726,6 +810,12 @@ class InteractionInstanceNotFoundException extends NotFoundException {}
  */
 class ViewNotFoundException extends NotFoundException {}
 
+/**
+ * Exception - view limit exceeded
+ * for example, multiple profile views
+ */
+class ViewLimitExceededException extends UserException {}
+
 /**
  * Exception - user not found
  */
@@ -752,6 +842,11 @@ class GroupNotFoundException extends NotFoundException {}
  */
 class QuotaExceededException extends UserException {}
 
+/**
+ * Exception - fired when uploading a file fails
+ */
+class UploadException extends UserException {}
+
 /**
  * Exception - Access denied. Throw this if a user is trying to view something they can't
  */
@@ -775,6 +870,34 @@ class AccessDeniedException extends UserException {
     }
 }
 
+/**
+ * Exception - Access denied to a group page because the user isn't a member
+ * Give the user a chance to join then continue
+ */
+class GroupAccessDeniedException extends AccessDeniedException {
+    public function render_exception() {
+        global $USER, $SESSION;
+        if (defined('GROUP') && $USER->is_logged_in()) {
+            $roles = $USER->get('grouproles');
+            if (!isset($roles[GROUP])) {
+                $group = group_current_group();
+                if ($group->jointype == 'open'
+                    || $group->jointype == 'invite' && get_record('group_member_invite', 'group', GROUP, 'member', $USER->get('id'))) {
+                    $SESSION->add_error_msg(get_string('notmembermayjoin', 'group', $group->name));
+                    $next = substr($_SERVER['REQUEST_URI'], strlen(get_mahara_install_subdirectory()) - 1);
+                    redirect(get_config('wwwroot') . 'group/view.php?id=' . GROUP . '&next=' . urlencode($next));
+                }
+                if ($group->jointype == 'request' && !get_record('group_member_request', 'group', GROUP, 'member', $USER->get('id'))) {
+                    $SESSION->add_error_msg(get_string('notamember', 'group'));
+                    redirect(get_config('wwwroot') . 'group/requestjoin.php?id=' . GROUP . '&returnto=view');
+                }
+            }
+        }
+        header("HTTP/1.0 403 Forbidden", true);
+        return parent::render_exception();
+    }
+}
+
 /**
  * Exception - Access totally denied, the user won't be able to access it even if they log in 
  * as the administrator
@@ -792,5 +915,33 @@ class AccessTotallyDeniedException extends UserException {
     }
 }
 
+/**
+* something has happened during import.
+* either: the user is there, in which case they get the bug screen,
+*         it's a spawned request during an xmlrpc server ping (content_ready) in which case XMLRPC will be defined
+*         or it's a queued fetch, in which case CRON will be defined.
+* @todo maybe refactor at the point that we have something other than importing over mnet (eg userland)
+*/
+class ImportException extends SystemException {
+
+    public function __construct($importer, $message=null, $code=0) {
+        parent::__construct($message, $code);
+        if ($importer instanceof PluginImport) {
+            $importer->cleanup();
+        }
+    }
+
+    public function render_exception() {
+        return $this->getMessage();
+    }
+}
+
+class ExportException extends SystemException {
+
+    public function __construct($exporter, $message=null, $code=0) {
+        parent::__construct($message, $code);
+    }
+}
+
 
 ?>
