@@ -43,14 +43,15 @@ define('BYTESERVING_BOUNDARY', 'm1i2k3e40516'); //unique string constant
  * @param string $path     The file to send. Must include the dataroot path.
  * @param string $filename The name of the file as the browser should use to
  *                         serve it.
+ * @param string $mimetype Mime type to be sent in header
  * @param array  $options  Any options to use when serving the file. Currently
  *                         lifetime = 0 for no cache
  *                         forcedownload - force application rather than inline
  *                         overridecontenttype - send this instead of the mimetype
  *                         there are none.
  */
-function serve_file($path, $filename, $options=array()) {
-    $dataroot = get_config('dataroot');
+function serve_file($path, $filename, $mimetype, $options=array()) {
+    $dataroot = realpath(get_config('dataroot'));
     $path = realpath($path);
     $options = array_merge(array(
         'lifetime' => 86400
@@ -66,7 +67,6 @@ function serve_file($path, $filename, $options=array()) {
 
     session_write_close(); // unlock session during fileserving
 
-    $mimetype     = get_mime_type($path);
     $lastmodified = filemtime($path);
     $filesize     = filesize($path);
 
@@ -79,7 +79,7 @@ function serve_file($path, $filename, $options=array()) {
         $mimetype = 'application/octet-stream';
     }
 
-    if (!$mimetype || (!is_image_mime_type($mimetype) && (isset($_SERVER['HTTP_USER_AGENT']) && false !== strpos($_SERVER['HTTP_USER_AGENT'], 'MSIE')))) {
+    if (!$mimetype) {
         $mimetype = 'application/forcedownload';
     }
 
@@ -192,9 +192,9 @@ function readfile_chunked($filename, $retbytes=true) {
         return false;
     }
 
-    @set_time_limit(60 * 60); //reset time limit to 60 min - should be enough for 1 MB chunk
 
     while (!feof($handle)) {
+        @set_time_limit(60 * 60); //reset time limit to 60 min - should be enough for 1 MB chunk
         $buffer = fread($handle, $chunksize);
         echo $buffer;
         flush();
@@ -225,10 +225,10 @@ function byteserving_send_file($filename, $mimetype, $ranges) {
         header('Content-Range: bytes ' . $ranges[0][1] . '-' . $ranges[0][2] . '/' . filesize($filename));
         header('Content-Type: ' . $mimetype);
         while (@ob_end_flush()); //flush the buffers - save memory and disable sid rewrite
-        @set_time_limit(60*60); //reset time limit to 60 min - should be enough for 1 MB chunk
         $buffer = '';
         fseek($handle, $ranges[0][1]);
         while (!feof($handle) && $length > 0) {
+            @set_time_limit(60*60); //reset time limit to 60 min - should be enough for 1 MB chunk
             $buffer = fread($handle, ($chunksize < $length ? $chunksize : $length));
             echo $buffer;
             flush();
@@ -253,8 +253,8 @@ function byteserving_send_file($filename, $mimetype, $ranges) {
             echo $range[0];
             $buffer = '';
             fseek($handle, $range[1]);
-            @set_time_limit(60 * 60); //reset time limit to 60 min - should be enough for 1 MB chunk
             while (!feof($handle) && $length > 0) {
+                @set_time_limit(60 * 60); //reset time limit to 60 min - should be enough for 1 MB chunk
                 $buffer = fread($handle, ($chunksize < $length ? $chunksize : $length));
                 echo $buffer;
                 flush();
@@ -278,6 +278,14 @@ function byteserving_send_file($filename, $mimetype, $ranges) {
  * Sometimes file will be unable to detect the mimetype, in which case
  * it will return the empty string.
  *
+ *
+ * This function should no longer be required.  Mime types are now
+ * stored along with files in the artefact tables, and passed directly
+ * to serve_file.  Left in place for the upgrade to initially populate
+ * the mime type of existing files.
+ * See htdocs/artefact/file/db/upgrade.php.
+ *
+ *
  * @param string $file The file to check
  * @return string      The mime type of the file, or false if file is not available.
  */
@@ -320,6 +328,54 @@ function is_image_mime_type($type) {
     return in_array($type, $supported);
 }
 
+
+/**
+ * Given an image type returned by getimagesize or exif_imagetype, returns whether
+ * Mahara thinks it is a valid image type.
+ *
+ * Not all image types are valid for Mahara. Mahara supports JPEG, PNG, GIF
+ * and BMP.
+ *
+ * @param string $type The type to check
+ * @return boolean     Whether the type is a valid image type for Mahara
+ */
+function is_image_type($type) {
+    $supported = array(IMAGETYPE_GIF, IMAGETYPE_JPEG, IMAGETYPE_PNG);
+    if (extension_loaded('imagick')) {
+        $supported[] = IMAGETYPE_BMP;
+    }
+    return $type && in_array($type, $supported);
+}
+
+
+/**
+ * Given path to a file, returns whether Mahara thinks it is a valid image file.
+ *
+ * Not all image types are valid for Mahara. Mahara supports JPEG, PNG, GIF
+ * and BMP.
+ *
+ * @param string $path The file to check
+ * @return boolean     Whether the file is a valid image file for Mahara
+ */
+function is_image_file($path) {
+    if (function_exists('exif_imagetype')) {
+        // exif_imagetype is faster
+        // surpressing errors because exif_imagetype spews "read error!" to the logs on small files
+        // http://nz.php.net/manual/en/function.exif-imagetype.php#79283
+        if (!$type = @exif_imagetype($path)) {
+            return false;
+        }
+    }
+    else {
+        // getimagesize returns the same answer
+        if (!list ($width, $height, $type) = getimagesize($path)) {
+            return false;
+        }
+    }
+    return is_image_type($type);
+}
+
+
 /**
  * Given a path under dataroot, an ID and a size, return the path to a file
  * matching all criteria.
@@ -328,7 +384,7 @@ function is_image_mime_type($type) {
  * will make a copy that is resized to the correct size.
  *
  * @param string $path The base path in dataroot where the image is stored. For 
- *                     example, 'artefact/internal/profileicons/' for profile 
+ *                     example, 'artefact/file/profileicons/' for profile 
  *                     icons
  * @param int $id      The ID of the image to return. Is typically the ID of an 
  *                     artefact
@@ -397,16 +453,20 @@ function get_dataroot_image_path($path, $id, $size=null) {
         // it, we can make one however.
         if (is_readable($originalimage)) {
 
-            $originalmimetype = get_mime_type($originalimage);
+            $imageinfo = getimagesize($originalimage);
+            $originalmimetype = $imageinfo['mime'];
+            $format = 'png';
             switch ($originalmimetype) {
                 case 'image/jpeg':
                 case 'image/jpg':
+                    $format = 'jpeg';
                     $oldih = imagecreatefromjpeg($originalimage);
                     break;
                 case 'image/png':
                     $oldih = imagecreatefrompng($originalimage);
                     break;
                 case 'image/gif':
+                    $format = 'gif';
                     $oldih = imagecreatefromgif($originalimage);
                     break;
                 case 'image/bmp':
@@ -464,7 +524,8 @@ function get_dataroot_image_path($path, $id, $size=null) {
                 //imagecopyresized($newih, $oldih, 0, 0, 0, 0, $newdimensions['w'], $newdimensions['h'], $oldx, $oldy);
             }
 
-            $result = imagepng($newih, $resizedimagefile);
+            $outputfunction = "image$format";
+            $result = $outputfunction($newih, $resizedimagefile);
             if ($result) {
                 return $resizedimagefile;
             }
@@ -514,8 +575,8 @@ function image_get_new_dimensions($oldx, $oldy, $size) {
     }
     else if (isset($size['maxw']) && isset($size['maxh'])) {
         $scale = min(min($size['maxw'], $oldx) / $oldx, min($size['maxh'], $oldy) / $oldy);
-        $newx = $oldx * $scale;
-        $newy = $oldy * $scale;
+        $newx = max(1, $oldx * $scale);
+        $newy = max(1, $oldy * $scale);
     }
     else if (isset($size['maxw'])) {
         // Else if just maximum width
@@ -546,11 +607,14 @@ function image_get_new_dimensions($oldx, $oldy, $size) {
 }
 
 /**
- * Deletes an image, including all resized versions of it, from dataroot.
+ * Deletes an image, excluding all resized versions of it, from dataroot.
  *
  * This function does not delete anything from anywhere else - it is your
  * responsibility to delete any database records.
  *
+ * This function also does not try to delete all resized versions of this
+ * image, as it would take a lot of effort to find them all.
+ *
  * @param string $path The path in dataroot of the base directory where the
  *                     image resides.
  * @param int $id      The id of the image to delete.
@@ -612,4 +676,48 @@ function rmdirr($dirname)
     return rmdir($dirname);
 }
 
+/**
+ * Copy a file, or recursively copy a folder and its contents
+ *
+ * @author      Aidan Lister <aidan@php.net>
+ * @version     1.0.1
+ * @link        http://aidanlister.com/repos/v/function.copyr.php
+ * @param       string   $source    Source path
+ * @param       string   $dest      Destination path
+ * @return      bool     Returns TRUE on success, FALSE on failure
+ */
+function copyr($source, $dest)
+{
+    // Check for symlinks
+    if (is_link($source)) {
+        return symlink(readlink($source), $dest);
+    }
+
+    // Simple copy for a file
+    if (is_file($source)) {
+        return copy($source, $dest);
+    }
+
+    // Make destination directory
+    if (!is_dir($dest)) {
+        mkdir($dest);
+    }
+
+    // Loop through the folder
+    $dir = dir($source);
+    while (false !== $entry = $dir->read()) {
+        // Skip pointers
+        if ($entry == '.' || $entry == '..') {
+            continue;
+        }
+
+        // Deep copy directories
+        copyr("$source/$entry", "$dest/$entry");
+    }
+
+    // Clean up
+    $dir->close();
+    return true;
+}
+
 ?>
