@@ -34,7 +34,7 @@
  * @return string            The error message, if the value is invalid.
  */
 function pieform_rule_maxvalue(Pieform $form, $value, $element, $maxvalue) {/*{{{*/
-    if ($value != '' && intval($value) > $maxvalue) {
+    if ($value != '' && doubleval($value) > $maxvalue) {
         return sprintf($form->i18n('rule', 'maxvalue', 'maxvalue', $element), $maxvalue);
     }
 }/*}}}*/
@@ -56,6 +56,15 @@ function pieform_rule_maxvalue_i18n() {/*{{{*/
         'es.utf8' => array(
             'maxvalue' => 'Este valor no puede ser superior a %d'
         ),
+        'sl.utf8' => array(
+            'maxvalue' => 'Ta vrednost ne sme biti večja od %d'
+        ),
+        'nl.utf8' => array(
+            'maxvalue' => 'Deze waarde kan niet groter zijn dan %d'
+        ),
+        'cs.utf8' => array(
+            'maxvalue' => 'Hodnota nemůže být větší než %d'
+        ),
 
     );
 }/*}}}*/
