@@ -64,6 +64,15 @@ function pieform_rule_validateoptions_i18n() {/*{{{*/
         'es.utf8' => array(
             'validateoptions' => 'La opción "%s" no es válida'
         ),
+        'sl.utf8' => array(
+            'validateoptions' => 'Možnost "%s" je neveljavna'
+        ),
+        'nl.utf8' => array(
+            'validateoptions' => 'De optie "%s" is niet geldig'
+        ),
+        'cs.utf8' => array(
+            'validateoptions' => 'Neplatný výběr "%s"'
+        ),
 
     );
 }/*}}}*/
