@@ -1,7 +1,8 @@
 <?php
 /**
  * Mahara: Electronic portfolio, weblog, resume builder and social networking
- * Copyright (C) 2006-2008 Catalyst IT Ltd (http://www.catalyst.net.nz)
+ * Copyright (C) 2006-2009 Catalyst IT Ltd and others; see:
+ *                         http://wiki.mahara.org/Contributors
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +21,7 @@
  * @subpackage core
  * @author     Catalyst IT Ltd
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL
- * @copyright  (C) 2006-2008 Catalyst IT Ltd http://catalyst.net.nz
+ * @copyright  (C) 2006-2009 Catalyst IT Ltd http://catalyst.net.nz
  *
  */
 
@@ -31,6 +32,7 @@ defined('INTERNAL') || die();
  */
 class InstallationException extends SystemException {}
 
+require_once('ddl.php');
 
 /**
  * This function checks core and plugins for which need to be upgraded/installed
@@ -55,21 +57,28 @@ function check_upgrades($name=null) {
     if (empty($name) || $name == 'core') {
         try {
             $coreversion = get_config('version');
-        } 
+        }
         catch (Exception $e) {
             $coreversion = 0;
         }
         if (empty($coreversion)) {
+            if (is_mysql()) { // Show a more informative error message if using mysql with skip-innodb
+                global $db;
+                $result = $db->Execute("SHOW VARIABLES LIKE 'have_innodb'");
+                if ($result->fields['Value'] != 'YES') {
+                    throw new ConfigSanityException("Mahara requires InnoDB tables.  Please ensure InnoDB tables are enabled in your MySQL server.");
+                }
+            }
             $core = new StdClass;
             $core->install = true;
             $core->to = $config->version;
             $core->torelease = $config->release;
             $toupgrade['core'] = $core;
             $installing = true;
-        } 
+        }
         else if ($config->version > $coreversion) {
             $corerelease = get_config('release');
-            if (isset($config->minupgradefrom) && isset($config->minupgraderelease) 
+            if (isset($config->minupgradefrom) && isset($config->minupgraderelease)
                 && $coreversion < $config->minupgradefrom) {
                 throw new ConfigSanityException("Must upgrade to $config->minupgradefrom "
                                           . "($config->minupgraderelease) first "
@@ -91,6 +100,33 @@ function check_upgrades($name=null) {
         return $toupgrade['core'];
     }
 
+    if (!$installing && (empty($name) || $name == 'local')) {
+        $localversion = get_config('localversion');
+        $localrelease = get_config('localrelease');
+        if (is_null($localversion)) {
+            $localversion = 0;
+            $localrelease = 0;
+        }
+
+        $config = new StdClass;
+        require(get_config('docroot') . 'local/version.php');
+
+        if ($config->version > $localversion) {
+            $toupgrade['local'] = (object) array(
+                'upgrade'     => true,
+                'from'        => $localversion,
+                'fromrelease' => $localrelease,
+                'to'          => $config->version,
+                'torelease'   => $config->release,
+            );
+        }
+
+        if ($name == 'local') {
+            $toupgrade['local']->disablelogin = $disablelogin;
+            return $toupgrade['local'];
+        }
+    }
+
     $plugins = array();
     if (!empty($name)) {
         try {
@@ -131,7 +167,7 @@ function check_upgrades($name=null) {
                     $btlocation = get_config('docroot') . $plugin . '/' . $dir . '/blocktype';
                     if (!is_dir($btlocation)) {
                         continue;
-                    }   
+                    }
                     $btdirhandle = opendir($btlocation);
                     while (false !== ($btdir = readdir($btdirhandle))) {
                         if (strpos($btdir, '.') === 0) {
@@ -254,9 +290,7 @@ function upgrade_core($upgrade) {
     db_begin();
 
     if (!empty($upgrade->install)) {
-        if (!install_from_xmldb_file($location . 'install.xml')) {
-            throw new SQLException("Failed to upgrade core (check logs for xmldb errors)");
-        }
+        install_from_xmldb_file($location . 'install.xml');
     }
     else {
         require_once($location . 'upgrade.php');
@@ -265,7 +299,7 @@ function upgrade_core($upgrade) {
 
     set_config('version', $upgrade->to);
     set_config('release', $upgrade->torelease);
-    
+
     if (!empty($upgrade->install)) {
         core_postinst();
     }
@@ -274,6 +308,26 @@ function upgrade_core($upgrade) {
     return true;
 }
 
+/**
+ * Upgrades local customisations.
+ *
+ * @param object $upgrade   The version to upgrade to
+ * @return bool             Whether the upgrade succeeded or not
+ * @throws SQLException     If the upgrade failed due to a database error
+ */
+function upgrade_local($upgrade) {
+    db_begin();
+
+    require_once(get_config('docroot') . 'local/upgrade.php');
+    xmldb_local_upgrade($upgrade->from);
+
+    set_config('localversion', $upgrade->to);
+    set_config('localrelease', $upgrade->torelease);
+
+    db_commit();
+    return true;
+}
+
 /**
  * Upgrades the plugin to a new version
  *
@@ -291,32 +345,24 @@ function upgrade_plugin($upgrade) {
 
     if ($plugintype == 'blocktype' && strpos($pluginname, '/') !== false) {
         list($artefactplugin, $blocktypename) = explode('/', $pluginname);
-    }   
+    }
 
     $location = get_config('docroot') . $plugintype . '/' . $pluginname . '/db/';
-    $db->StartTrans();
+    db_begin();
 
     if (!empty($upgrade->install)) {
         if (is_readable($location . 'install.xml')) {
-            $status = install_from_xmldb_file($location . 'install.xml');
-        }
-        else {
-            $status = true;
+            install_from_xmldb_file($location . 'install.xml');
         }
     }
     else {
         if (is_readable($location .  'upgrade.php')) {
             require_once($location . 'upgrade.php');
             $function = 'xmldb_' . $plugintype . '_' . $pluginname . '_upgrade';
-            $status = $function($upgrade->from);
+            if (!$function($upgrade->from)) {
+                throw new InstallationException("Failed to run " . $function . " (check logs for errors)");
+            }
         }
-        else {
-            $status = true;
-        }
-    }
-    if (!$status || $db->HasFailedTrans()) {
-        $db->CompleteTrans();
-        throw new SQLException("Failed to upgrade $upgrade->name (check logs for xmldb errors)");
     }
 
     $installed = new StdClass;
@@ -341,7 +387,7 @@ function upgrade_plugin($upgrade) {
 
     if (!empty($upgrade->install)) {
         insert_record($installtable,$installed);
-    } 
+    }
     else {
         update_record($installtable, $installed, 'name');
     }
@@ -354,11 +400,9 @@ function upgrade_plugin($upgrade) {
         foreach ($crons as $cron) {
             $cron = (object)$cron;
             if (empty($cron->callfunction)) {
-                $db->RollbackTrans();
                 throw new InstallationException("cron for $pcname didn't supply function name");
             }
             if (!is_callable(array($pcname, $cron->callfunction))) {
-                $db->RollbackTrans();
                 throw new InstallationException("cron $cron->callfunction for $pcname supplied but wasn't callable");
             }
             $new = false;
@@ -378,21 +422,18 @@ function upgrade_plugin($upgrade) {
             }
         }
     }
-    
+
     if ($events = call_static_method($pcname, 'get_event_subscriptions')) {
         foreach ($events as $event) {
             $event = (object)$event;
 
             if (!record_exists('event_type', 'name', $event->event)) {
-                $db->RollbackTrans();
                 throw new InstallationException("event $event->event for $pcname doesn't exist!");
             }
             if (empty($event->callfunction)) {
-                $db->RollbackTrans();
                 throw new InstallationException("event $event->event for $pcname didn't supply function name");
             }
             if (!is_callable(array($pcname, $event->callfunction))) {
-                $db->RollbackTrans();
                 throw new InstallationException("event $event->event with function $event->callfunction for $pcname supplied but wasn't callable");
             }
             $exists = false;
@@ -422,7 +463,7 @@ function upgrade_plugin($upgrade) {
             $where = $activity;
             unset($where->admin);
             unset($where->delay);
-            // Work around the fact that insert_record cached the columns that 
+            // Work around the fact that insert_record cached the columns that
             // _were_ in the activity_type table before it was upgraded
             global $INSERTRECORD_NOCACHE;
             $INSERTRECORD_NOCACHE = true;
@@ -453,36 +494,18 @@ function upgrade_plugin($upgrade) {
                                   array_merge(array($pluginname),$types));
         }
     }
-    
+
     // install blocktype categories.
     if ($plugintype == 'blocktype' && get_config('installed')) {
         install_blocktype_categories_for_plugin($pluginname);
+        install_blocktype_viewtypes_for_plugin($pluginname);
     }
 
     $prevversion = (empty($upgrade->install)) ? $upgrade->from : 0;
     call_static_method($pcname, 'postinst', $prevversion);
-    
-    if ($db->HasFailedTrans()) {
-        $status = false;
-    }
-    $db->CompleteTrans();
-
-    /*  they do themselves already
-    // we have to do this after committing the current transaction because we call ourselves recursively...
-    if ($plugintype == 'artefact' && get_config('installed')) {
-        // only install associated blocktype plugins if we're not in the process of installing 
-        if ($blocktypes = call_static_method($pcname, 'get_block_types')) {
-            foreach ($blocktypes as $bt) {
-                if ($upgrade = check_upgrades('blocktype.' . $pluginname . '/' . $bt)) {
-                    upgrade_plugin($upgrade); 
-                }
-            }
-        }
-    }
-    */
-    
-    return $status;
 
+    db_commit();
+    return true;
 }
 
 function core_postinst() {
@@ -536,19 +559,37 @@ function core_postinst() {
     set_config('searchplugin', 'internal');
 
     set_config('lang', 'en.utf8');
+    set_config('installation_key', get_random_key());
 
     // PostgreSQL supports indexes over functions of columns, MySQL does not. 
-    // So we can improve the index on the username field of the usr table for 
-    // Postgres
+    // We make use if this if we can
     if (is_postgres()) {
+        // Improve the username index
         execute_sql('DROP INDEX {usr_use_uix}');
         execute_sql('CREATE UNIQUE INDEX {usr_use_uix} ON {usr}(LOWER(username))');
+
+        // Only one profile view per user
+        execute_sql("CREATE UNIQUE INDEX {view_own_type_uix} ON {view}(owner) WHERE type = 'profile'");
     }
 
+    // Some more advanced constraints. XMLDB can't handle this in its xml file format
+    execute_sql('ALTER TABLE {artefact} ADD CHECK (
+        (owner IS NOT NULL AND "group" IS NULL     AND institution IS NULL) OR
+        (owner IS NULL     AND "group" IS NOT NULL AND institution IS NULL) OR
+        (owner IS NULL     AND "group" IS NULL     AND institution IS NOT NULL)
+    )');
+    execute_sql('ALTER TABLE {view} ADD CHECK (
+        (owner IS NOT NULL AND "group" IS NULL     AND institution IS NULL) OR
+        (owner IS NULL     AND "group" IS NOT NULL AND institution IS NULL) OR
+        (owner IS NULL     AND "group" IS NULL     AND institution IS NOT NULL)
+    )');
+
+    reload_html_filters();
     return $status;
 }
 
 function core_install_lastcoredata_defaults() {
+    global $USER;
     db_begin();
     $institution = new StdClass;
     $institution->name = 'mahara';
@@ -558,7 +599,7 @@ function core_install_lastcoredata_defaults() {
     insert_record('institution', $institution);
 
     $auth_instance = new StdClass;
-    $auth_instance->instancename  = 'internal';
+    $auth_instance->instancename  = 'Internal';
     $auth_instance->priority='1';
     $auth_instance->institution   = 'mahara';
     $auth_instance->authname      = 'internal';
@@ -585,6 +626,8 @@ function core_install_lastcoredata_defaults() {
         insert_record('usr', $user);
     }
 
+    install_system_profile_view();
+
     // Insert the admin user
     $user = new StdClass;
     $user->username = 'admin';
@@ -600,25 +643,33 @@ function core_install_lastcoredata_defaults() {
     set_profile_field($user->id, 'email', $user->email);
     set_profile_field($user->id, 'firstname', $user->firstname);
     set_profile_field($user->id, 'lastname', $user->lastname);
-    set_config('installed', true);
+    handle_event('createuser', $user);
+    activity_add_admin_defaults(array($user->id));
     db_commit();
 
     // if we're installing, set up the block categories here and then poll the plugins.
-    // if we're upgrading this happens somewhere else.  This is because of dependency issues around 
+    // if we're upgrading this happens somewhere else.  This is because of dependency issues around
     // the order of installation stuff.
 
-    install_blocktype_categories();
-
+    install_blocktype_extras();
 }
 
 function core_install_firstcoredata_defaults() {
     // Install the default institution
     db_begin();
-    
-    set_config('session_timeout', 1800);
+
+    set_config('session_timeout', 86400);
     set_config('sitename', 'Mahara');
-    set_config('pathtofile', '/usr/bin/file');
     set_config('defaultaccountinactivewarn', 604800);
+    set_config('creategroups', 'all');
+    set_config('createpublicgroups', 'all');
+    set_config('allowpublicviews', 1);
+    set_config('allowpublicprofiles', 1);
+    set_config('captchaoncontactform', 1);
+    set_config('captchaonregisterform', 1);
+    set_config('showselfsearchsideblock', 0);
+    set_config('showtagssideblock', 1);
+    set_config('tagssideblockmaxtags', 20);
 
     // install the applications
     $app = new StdClass;
@@ -646,11 +697,18 @@ function core_install_firstcoredata_defaults() {
         'unexpireuser',
         'deactivateuser',
         'activateuser',
+        'userjoinsgroup',
         'saveartefact',
         'deleteartefact',
+        'deleteartefacts',
         'saveview',
         'deleteview',
         'blockinstancecommit',
+        'addfriend',
+        'removefriend',
+        'addfriendrequest',
+        'removefriendrequest',
+        'creategroup',
     );
 
     foreach ($eventtypes as $et) {
@@ -659,6 +717,22 @@ function core_install_firstcoredata_defaults() {
         insert_record('event_type', $e);
     }
 
+    // install the core event subscriptions
+    $subs = array(
+        array(
+            'event'        => 'createuser',
+            'callfunction' => 'activity_set_defaults',
+        ),
+        array(
+            'event'        => 'createuser',
+            'callfunction' => 'add_user_to_autoadd_groups',
+        ),
+    );
+
+    foreach ($subs as $sub) {
+        insert_record('event_subscription', (object)$sub);
+    }
+
     // install the activity types
     $activitytypes = array(
         array('maharamessage', 0, 0),
@@ -671,6 +745,7 @@ function core_install_firstcoredata_defaults() {
         array('virusrepeat', 1, 1),
         array('virusrelease', 1, 1),
         array('institutionmessage', 0, 0),
+        array('groupmessage', 0, 0),
     );
 
     foreach ($activitytypes as $at) {
@@ -687,9 +762,14 @@ function core_install_firstcoredata_defaults() {
         'rebuild_artefact_parent_cache_complete' => array('0', '4', '*', '*', '*'),
         'auth_clean_partial_registrations'       => array('5', '0', '*', '*', '*'),
         'auth_handle_account_expiries'           => array('5', '10', '*', '*', '*'),
+        'auth_handle_institution_expiries'       => array('5', '9', '*', '*', '*'),
         'activity_process_queue'                 => array('*/5', '*', '*', '*', '*'),
         'auth_remove_old_session_files'          => array('30', '20', '*', '*', '*'),
         'recalculate_quota'                      => array('15', '2', '*', '*', '*'),
+        'import_process_queue'                   => array('*/5', '*', '*', '*', '*'),
+        'cron_send_registration_data'            => array(rand(0, 59), rand(0, 23), '*', '*', rand(0, 6)),
+        'export_cleanup_old_exports'             => array('0', '3,15', '*', '*', '*'),
+        'import_cleanup_old_imports'             => array('0', '4,16', '*', '*', '*'),
     );
     foreach ($cronjobs as $callfunction => $times) {
         $cron = new StdClass;
@@ -701,17 +781,23 @@ function core_install_firstcoredata_defaults() {
         $cron->dayofweek    = $times[4];
         insert_record('cron', $cron);
     }
-    
+
     // install the view column widths
     install_view_column_widths();
 
+    $viewtypes = array('portfolio', 'profile');
+    foreach ($viewtypes as $vt) {
+        insert_record('view_type', (object)array(
+            'type' => $vt,
+        ));
+    }
     db_commit();
 }
 
 
-/** 
-* xmldb will pass us the xml file and we can perform any substitution as necessary
-*/ 
+/**
+ * xmldb will pass us the xml file and we can perform any substitution as necessary
+ */
 function local_xmldb_contents_sub(&$contents) {
     // the main install.xml file needs to sub in plugintype tables.
     $searchstring = '<!-- PLUGIN_TYPE_SUBSTITUTION -->';
@@ -723,12 +809,12 @@ function local_xmldb_contents_sub(&$contents) {
     $plugintables = file_get_contents(get_config('docroot') . 'lib/db/plugintables.xml');
     $tosub = '';
     foreach (plugin_types() as $plugin) {
-        // any that want their own stuff can put it in docroot/plugintype/lib/db/plugintables.xml 
+        // any that want their own stuff can put it in docroot/plugintype/lib/db/plugintables.xml
         //- like auth is a bit special
         $specialcase = get_config('docroot') . $plugin . '/plugintables.xml';
         if (is_readable($specialcase)) {
             $tosub .= file_get_contents($specialcase) . "\n";
-        } 
+        }
         else {
             $replaced = '';
             // look for tables to put at the start...
@@ -798,6 +884,18 @@ function sort_upgrades($k1, $k2) {
     else if ($k2 == 'firstcoredata') {
         return 1;
     }
+    else if ($k1 == 'localpreinst') {
+        return -1;
+    }
+    else if ($k2 == 'localpreinst') {
+        return 1;
+    }
+    else if ($k1 == 'localpostinst') {
+        return 1;
+    }
+    else if ($k2 == 'localpostinst') {
+        return -1;
+    }
     else if ($k1 == 'lastcoredata') {
         return 1;
     }
@@ -832,7 +930,23 @@ function install_blocktype_categories_for_plugin($blocktype) {
     db_commit();
 }
 
-function install_blocktype_categories() {
+function install_blocktype_viewtypes_for_plugin($blocktype) {
+    safe_require('blocktype', $blocktype);
+    $blocktype = blocktype_namespaced_to_single($blocktype);
+    db_begin();
+    delete_records('blocktype_installed_viewtype', 'blocktype', $blocktype);
+    if ($viewtypes = call_static_method(generate_class_name('blocktype', $blocktype), 'get_viewtypes')) {
+        foreach($viewtypes as $vt) {
+            insert_record('blocktype_installed_viewtype', (object)array(
+                'blocktype' => $blocktype,
+                'viewtype'  => $vt,
+            ));
+        }
+    }
+    db_commit();
+}
+
+function install_blocktype_extras() {
     db_begin();
 
     $categories = get_blocktype_categories();
@@ -848,38 +962,24 @@ function install_blocktype_categories() {
 
 
     // poll all the installed blocktype plugins and ask them what categories they export
-    if ($blocktypes = plugins_installed('blocktype')) {
+    if ($blocktypes = plugins_installed('blocktype', true)) {
         foreach ($blocktypes as $bt) {
             install_blocktype_categories_for_plugin(blocktype_single_to_namespaced($bt->name, $bt->artefactplugin));
+            install_blocktype_viewtypes_for_plugin(blocktype_single_to_namespaced($bt->name, $bt->artefactplugin));
         }
     }
 }
 
 /**
- * Installs all the allowed column widths for views. Used when installing core 
+ * Installs all the allowed column widths for views. Used when installing core
  * defaults, and also when upgrading from 0.8 to 0.9
  */
 function install_view_column_widths() {
     db_begin();
-    $layouts = array(
-        2 => array(
-            '50,50',
-            '67,33',
-            '33,67',
-            ),
-        3 => array(
-            '33,33,33',
-            '25,50,25',
-            '15,70,15',
-            ),
-        4 => array(
-            '25,25,25,25',
-            '20,30,30,20',
-            ),
-    );
+    require_once('view.php');
 
     $layout = new StdClass;
-    foreach ($layouts as $column => $widths) {
+    foreach (View::$layouts as $column => $widths) {
         foreach ($widths as $width) {
             $layout->columns = $column;
             $layout->widths = $width;
@@ -889,4 +989,24 @@ function install_view_column_widths() {
     db_commit();
 }
 
+/**
+ * Reload htmlpurifier filters from the XML configuration file.
+ */
+function reload_html_filters() {
+    require_once('xmlize.php');
+    log_info('Reading HTML filters');
+
+    $newlist = xmlize(file_get_contents(get_config('libroot') . 'htmlpurifiercustom/filters.xml'));
+    $filters = $newlist['filters']['#']['filter'];
+    foreach ($filters as &$f) {
+        $f = (object) array(
+            'site' => $f['#']['site'][0]['#'],
+            'file' => $f['#']['filename'][0]['#']
+        );
+        log_info('- ' . $f->file);
+    }
+    $filters[] = (object) array('site' => 'http://www.youtube.com', 'file' => 'YouTube');
+    log_info('- YouTube');
+    set_config('filters', serialize($filters));
+}
 ?>
