@@ -1,7 +1,8 @@
 <?php
 /**
  * Mahara: Electronic portfolio, weblog, resume builder and social networking
- * Copyright (C) 2006-2008 Catalyst IT Ltd (http://www.catalyst.net.nz)
+ * Copyright (C) 2006-2009 Catalyst IT Ltd and others; see:
+ *                         http://wiki.mahara.org/Contributors
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +21,7 @@
  * @subpackage search-internal
  * @author     Catalyst IT Ltd
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL
- * @copyright  (C) 2006-2008 Catalyst IT Ltd http://catalyst.net.nz
+ * @copyright  (C) 2006-2009 Catalyst IT Ltd http://catalyst.net.nz
  *
  */
 
@@ -32,6 +33,10 @@ defined('INTERNAL') || die();
  */
 class PluginSearchInternal extends PluginSearch {
 
+    public static function can_be_disabled() {
+        return false;
+    }
+
     /**
      * Implement user searching with SQL
      *
@@ -79,190 +84,115 @@ class PluginSearchInternal extends PluginSearch {
         if (empty($publicfields)) {
             $publicfields = array('preferredname');
         }
-        if (is_postgres()) {
-            return self::search_user_pg($query_string, $limit, $offset, $data, $publicfields);
-        } 
-        else if (is_mysql()) {
-            return self::search_user_my($query_string, $limit, $offset, $data, $publicfields);
-        }
-        else {
-            throw new SQLException('search_user() is not implemented for your database engine (' . get_config('dbtype') . ')');
-        }
-    }
-
-    public static function search_user_pg($query_string, $limit, $offset, $data, $publicfields) {
-        $fieldlist = "('" . join("','", $publicfields) . "')";
-        $sql = 'SELECT
+        $fieldlist = '(' . join(',', array_map('db_quote', $publicfields)) . ')';
+        $ilike = db_ilike();
+        $data = self::prepare_search_user_options($data);
+        $sql = '
+            SELECT
                 COUNT(DISTINCT u.id)
-            FROM
-                {usr} u
-                LEFT JOIN {artefact} a ON u.id=a.owner
-                ';
+            FROM {artefact} a
+                INNER JOIN {usr} u ON u.id = a.owner';
         if (isset($data['group'])) {
-            $sql .= 'INNER JOIN {group_member} gm ON (gm.member = u.id AND gm.group = ' . (int)$data['group'] . ')
-                ';
-            if (isset($data['owner']) && !$data['owner']) {
-                $sql .= 'INNER JOIN {group} g ON (g.id = gm.group AND g.owner != u.id)
-                    ';
+            $groupadminsql = '';
+            if (isset($data['includeadmins']) and !$data['includeadmins']) {
+                $groupadminsql = " AND gm.role != 'admin'";
             }
+            $groupjoin = '
+                INNER JOIN {group_member} gm ON
+                    (gm.member = u.id AND gm.group = ' . (int)$data['group'] . $groupadminsql . ")\n";
+            $sql .= $groupjoin;
         }
         $querydata = split(' ', preg_replace('/\s\s+/', ' ', strtolower(trim($query_string))));
-        $namesql = '(
-                        u.preferredname ILIKE \'%\' || ? || \'%\'
-                    )
-                    OR (
-                        (u.preferredname IS NULL OR u.preferredname = \'\')
-                        AND (
-                            u.firstname ILIKE \'%\' || ? || \'%\'
-                            OR u.lastname ILIKE \'%\' || ? || \'%\'
-                        )
-                    )
-                    OR (
-                        a.artefacttype IN ' . $fieldlist . '
-                        AND ( a.title ILIKE \'%\' || ? || \'%\')
-                    )';
-        $namesql = join(' OR ', array_fill(0, count($querydata), $namesql));
+        $namesql = "(u.preferredname $ilike '%' || ? || '%')
+                    OR ((u.preferredname IS NULL OR u.preferredname = '')
+                        AND (u.firstname $ilike '%' || ? || '%' OR u.lastname $ilike '%' || ? || '%'))
+                    OR (a.artefacttype IN $fieldlist
+                        AND ( a.title $ilike '%' || ? || '%'))";
+        $namesql = join('
+                    OR ', array_fill(0, count($querydata), $namesql));
         $values = array();
         foreach ($querydata as $w) {
             $values = array_pad($values, count($values) + 4, $w);
         }
 
-        $sql .= 'WHERE
+        $where = '
+            WHERE
                 u.id <> 0 AND u.active = 1 AND u.deleted = 0
-                AND ( ' . $namesql . '
+                AND (
+                    ' . $namesql . '
                 )
                 ' . (isset($data['exclude']) ? 'AND u.id != ' . $data['exclude'] : '') . '
             ';
-        $count = get_field_sql($sql, $values);
 
-        if ($count > 0) {
-            $sql = 'SELECT DISTINCT ON (u.firstname, u.lastname, u.id)
-                    u.id, u.username, u.firstname, u.lastname, u.preferredname, u.email, u.staff
-                FROM {artefact} a
-                    INNER JOIN {usr} u ON u.id = a.owner
-                ';
-            if (isset($data['group'])) {
-                $sql .= 'INNER JOIN {group_member} gm ON (gm.member = u.id AND gm.group = ' . (int)$data['group'] . ')
-                    ';
-                if (isset($data['owner']) && !$data['owner']) {
-                    $sql .= 'INNER JOIN {group} g ON (g.id = gm.group AND g.owner != u.id)
-                        ';
-                    }
-            }
-            $sql .= 'WHERE
-                    u.id <> 0 AND u.active = 1 AND u.deleted = 0
-                    AND (' . $namesql . '
-                    )
-                    ' . (isset($data['exclude']) ? 'AND u.id != ' . $data['exclude'] : '') . '
-                ORDER BY u.firstname, u.lastname, u.id';
-            $data = get_records_sql_array($sql, $values, $offset, $limit);
+        $sql .= $where;
+        $count = count_records_sql($sql, $values);
 
-            if ($data) {
-                foreach ($data as &$item) {
-                    $item = (array)$item;
-                }
-            }
-        }
-        else {
-            $data = false;
-        }
-
-        return array(
+        $result = array(
             'count'   => $count,
             'limit'   => $limit,
             'offset'  => $offset,
-            'data'    => $data,
+            'data'    => false,
         );
-    }
 
-    public static function search_user_my($query_string, $limit, $offset, $data, $publicfields) {
-        $fieldlist = "('" . join("','", $publicfields) . "')";
-
-        $sql = 'SELECT
-                COUNT(DISTINCT u.id)
-            FROM
-                {usr} u
-                LEFT JOIN {artefact} a ON u.id=a.owner
-                ';
-        if (isset($data['group'])) {
-            $sql .= 'INNER JOIN {group_member} gm ON (gm.member = u.id AND gm.group = ' . (int)$data['group'] . ')
-                ';
-            if (isset($data['owner']) && !$data['owner']) {
-                $sql .= 'INNER JOIN {group} g ON (g.id = gm.group AND g.owner != u.id)
-                    ';
-            }
-        }
-        $querydata = split(' ', preg_replace('/\s\s+/', ' ', strtolower(trim($query_string))));
-        $namesql = '(
-                        u.preferredname LIKE \'%\' || ? || \'%\'
-                    )
-                    OR (
-                        (u.preferredname IS NULL OR u.preferredname = \'\')
-                        AND (
-                            u.firstname LIKE \'%\' || ? || \'%\'
-                            OR u.lastname LIKE \'%\' || ? || \'%\'
-                        )
-                    )
-                    OR (
-                        a.artefacttype IN ' . $fieldlist . '
-                        AND ( a.title LIKE \'%\' || ? || \'%\')
-                    )';
-        $namesql = join(' OR ', array_fill(0, count($querydata), $namesql));
-        $values = array();
-        foreach ($querydata as $w) {
-            $values = array_pad($values, count($values) + 4, $w);
+        if ($count < 1) {
+            return $result;
         }
 
-        $sql .= 'WHERE
-                u.id <> 0 AND u.active = 1 AND u.deleted = 0
-                AND (' . $namesql . '
-                )
-                ' . (isset($data['exclude']) ? 'AND u.id != ' . $data['exclude'] : '') . '
-            ';
-
-        $count = get_field_sql($sql, $values);
-        if ($count > 0) {
+        if (is_postgres()) {
+            $sql = '
+            SELECT DISTINCT ON (' . $data['orderby'] . ')
+                u.id, u.username, u.firstname, u.lastname, u.preferredname, u.email, u.staff
+            FROM {artefact} a
+                INNER JOIN {usr} u ON u.id = a.owner';
+        }
+        else if (is_mysql()) {
             // @todo This is quite possibly not correct. See the postgres 
             // query. It should be DISTINCT ON the fields as specified by the 
             // postgres query. There is a workaround by using SELECT * followed 
             // by GROUP BY 1, 2, 3, but it's not really valid SQL. The other 
             // way is to rewrite the query using a self join on the usr table.
-            $sql = 'SELECT DISTINCT
-                    u.id, u.username, u.firstname, u.lastname, u.preferredname, u.email, u.staff
-                FROM {artefact} a
-                    INNER JOIN {usr} u ON u.id = a.owner
-                    ';
-            if (isset($data['group'])) {
-                $sql .= 'INNER JOIN {group_member} gm ON (gm.member = u.id AND gm.group = ' . (int)$data['group'] . ')
-                    ';
-                if (isset($data['owner']) && !$data['owner']) {
-                    $sql .= 'INNER JOIN {group} g ON (g.id = gm.group AND g.owner != u.id)
-                        ';
-                    }
-            }
-            $sql .= 'WHERE
-                    u.id <> 0 AND u.active = 1 AND u.deleted = 0
-                    AND ( ' . $namesql . '
-                    )
-                    ' . (isset($data['exclude']) ? 'AND u.id != ' . $data['exclude'] : '') . '
-                ORDER BY u.firstname, u.lastname, u.id';
-            $data = get_records_sql_array($sql, $values, $offset, $limit);
-            if ($data) {
-                foreach ($data as &$item) {
-                    $item = (array)$item;
-                }
-            }
+            $sql = '
+            SELECT DISTINCT
+                u.id, u.username, u.firstname, u.lastname, u.preferredname, u.email, u.staff
+            FROM {artefact} a
+                INNER JOIN {usr} u ON u.id = a.owner';
         }
         else {
-            $data = false;
+            throw new SQLException('search_user() not implemented for your database engine (' . get_config('dbtype') . ')');
         }
 
-        return array(
-            'count'   => $count,
-            'limit'   => $limit,
-            'offset'  => $offset,
-            'data'    => $data,
-        );
+        if (isset($data['group'])) {
+            $sql .= $groupjoin;
+        }
+
+        $sql .= $where . '
+            ORDER BY ' . $data['orderby'];
+
+        $result['data'] = get_records_sql_array($sql, $values, $offset, $limit);
+
+        if ($result['data']) {
+            foreach ($result['data'] as &$item) {
+                $item = (array)$item;
+            }
+        }
+
+        return $result;
+    }
+
+    private static function prepare_search_user_options($options) {
+        if (isset($options['group'])) {
+            $options['group'] = intval($options['group']);
+        }
+        if (isset($options['includeadmins'])) {
+            $options['includeadmins'] = (bool)$options['includeadmins'];
+        }
+        if (isset($options['orderby']) && $options['orderby'] == 'lastname') {
+            $options['orderby'] = 'u.lastname, u.firstname, u.id';
+        }
+        else {
+            $options['orderby'] = 'u.firstname, u.lastname, u.id';
+        }
+        return $options;
     }
     
 
@@ -402,6 +332,69 @@ class PluginSearchInternal extends PluginSearch {
     }
 
 
+    public static function group_search_user($group, $queries, $constraints, $offset, $limit, $membershiptype) {
+        $where = 'WHERE gm.group = ?';
+        $values = array($group);
+
+        // Get the correct keyword for case insensitive LIKE
+        $ilike = db_ilike();
+
+        // Only handle OR/AND expressions at the top level.  Eventually we may need subexpressions.
+
+        if (!empty($queries)) {
+            $where .= ' AND ( ';
+            $str = array();
+            foreach ($queries as $f) {
+                $str[] = 'u.' . $f['field'] 
+                    . PluginSearchInternal::match_expression($f['type'], $f['string'], $values, $ilike);
+            }
+            $where .= join(' OR ', $str) . ') ';
+        }
+
+        $group_member = 'group_member';
+        if (!empty($membershiptype) && in_array($membershiptype, array('request', 'invite'))) {
+            $group_member .= '_' . $membershiptype;
+            $gm_role = '';
+            $gm_role_order = '';
+        }
+        else {
+            $gm_role = ', gm.role';
+            $gm_role_order = "gm.role = 'admin' DESC, ";
+        }
+
+        $count = get_field_sql('SELECT COUNT(*) FROM {usr} u INNER JOIN {' . $group_member . '} gm ON (gm.member = u.id) ' . $where, $values);
+
+        if ($count > 0) {
+            $data = get_records_sql_assoc('
+                SELECT
+                    u.id, u.firstname, u.lastname, u.username, u.email, u.staff, ' . db_format_tsfield('gm.ctime', 'jointime') . $gm_role . '
+                FROM
+                    {usr} u
+                INNER JOIN {' . $group_member . '} gm ON (gm.member = u.id) ' . $where . '
+                ORDER BY ' . $gm_role_order . 'gm.ctime, u.firstname, u.lastname, u.id',
+                $values,
+                $offset,
+                $limit);
+
+            if ($data) {
+                foreach ($data as &$item) {
+                    $item = (array)$item;
+                }
+                $data = array_values($data);
+            }
+        }
+        else {
+            $data = array();
+        }
+
+        return array(
+            'count'   => $count,
+            'limit'   => $limit,
+            'offset'  => $offset,
+            'data'    => $data,
+        );
+    }
+
 
     public static function institutional_admin_search_user($query, $institution, $limit) {
         $sql = '
@@ -535,7 +528,7 @@ class PluginSearchInternal extends PluginSearch {
         global $USER;
         $sql = "
             SELECT
-                id, name, description, jointype, owner, ctime, mtime
+                id, name, description, grouptype, jointype, ctime, mtime
             FROM
                 {group}
             WHERE (
@@ -545,21 +538,19 @@ class PluginSearchInternal extends PluginSearch {
         $values = array($query_string, $query_string, 0);
         if ($type == 'member') {
             $sql .=  'AND (
-                owner = ? OR id IN (
+                id IN (
                     SELECT "group" FROM {group_member} WHERE member = ?
                 )
             )';
             $values[] = $USER->get('id');
-            $values[] = $USER->get('id');
         }
         else if ($type == 'notmember') {
             $sql .=  'AND (
-                owner != ? AND id NOT IN (
+                id NOT IN (
                     SELECT "group" FROM {group_member} WHERE member = ?
                 )
             )';
             $values[] = $USER->get('id');
-            $values[] = $USER->get('id');
         }
         $sql .= 'ORDER BY name';
         $data = get_records_sql_array($sql, $values, $offset, $limit);
@@ -575,7 +566,7 @@ class PluginSearchInternal extends PluginSearch {
             ) AND deleted = ? ";
         if ($type == 'member') {
             $sql .= 'AND (
-                    owner = ? OR id IN (
+                    id IN (
                         SELECT "group" FROM {group_member} WHERE member = ?
                     )
                 )
@@ -583,7 +574,7 @@ class PluginSearchInternal extends PluginSearch {
         }
         else if ($type == 'notmember') {
             $sql .= 'AND (
-                    owner != ? AND id NOT IN (
+                    id NOT IN (
                         SELECT "group" FROM {group_member} WHERE member = ?
                     )
                 )
@@ -603,7 +594,7 @@ class PluginSearchInternal extends PluginSearch {
         global $USER;
         $sql = "
             SELECT
-                id, name, description, jointype, owner, ctime, mtime
+                id, name, description, grouptype, jointype, ctime, mtime
             FROM
                 {group}
             WHERE (
@@ -613,21 +604,19 @@ class PluginSearchInternal extends PluginSearch {
         $values = array($query_string, $query_string, 0);
         if ($type == 'member') {
             $sql .=  'AND (
-                owner = ? OR id IN (
+                id IN (
                     SELECT gm.group FROM {group_member} gm WHERE gm.member = ?
                 )
             )';
             $values[] = $USER->get('id');
-            $values[] = $USER->get('id');
         }
         else if ($type == 'notmember') {
             $sql .=  "AND (
-                owner != ? AND id NOT IN (
+                id NOT IN (
                     SELECT gm.group FROM {group_member} gm WHERE gm.member = ?
                 )
             )";
             $values[] = $USER->get('id');
-            $values[] = $USER->get('id');
         }
         $sql .= 'ORDER BY name';
         $data = get_records_sql_array($sql, $values, $offset, $limit);
@@ -643,7 +632,7 @@ class PluginSearchInternal extends PluginSearch {
             ) AND deleted = ? ";
         if ($type == 'member') {
             $sql .= 'AND (
-                    owner = ? OR id IN (
+                    id IN (
                         SELECT gm.group FROM {group_member} gm WHERE gm.member = ?
                     )
                 )
@@ -651,7 +640,7 @@ class PluginSearchInternal extends PluginSearch {
         }
         else if ($type == 'notmember') {
             $sql .= "AND (
-                    owner != ? AND id NOT IN (
+                    id NOT IN (
                         SELECT gm.group FROM {group_member} gm WHERE gm.member = ?
                     )
                 )
@@ -758,6 +747,91 @@ class PluginSearchInternal extends PluginSearch {
     }
 
 
+    /**
+     * Returns portfolio items (artefacts, views) owned by $owner and tagged
+     * with $tag.
+     *
+     * @param string   $tag Tag
+     * @param object   $owner: owner type (user,group,institution), and id
+     * @param integer  $limit
+     * @param integer  $offset
+     * @param string   $sort
+     * @param array    $types view/artefacttype filters
+     * @param boolean  $returntags Return all the tags that have been attached to each result
+     */
+    public static function portfolio_search_by_tag($tag, $owner, $limit, $offset, $sort, $types, $returntags) {
+        $viewfilter = is_null($types) || $types['view'] == true ? 'AND TRUE' : 'AND FALSE';
+
+        if (is_null($types)) {
+            $artefacttypefilter = '';
+        }
+        else if (!empty($types['artefact'])) {
+            $artefacttypefilter = ' AND a.artefacttype IN (' . join(',', array_map('db_quote', $types['artefact'])) . ')';
+        }
+        else {
+            $artefacttypefilter = ' AND FALSE';
+        }
+
+        if (!is_null($tag)) {
+            $artefacttypefilter .= ' AND at.tag = ?';
+            $viewfilter .= ' AND vt.tag = ?';
+            $values = array($owner->id, $tag, $owner->id, $tag);
+        }
+        else {
+            $values = array($owner->id, $owner->id);
+        }
+
+        $from = "FROM (
+           (SELECT a.id, a.title, a.description, 'artefact' AS type, a.artefacttype, " . db_format_tsfield('a.ctime', 'ctime') . "
+            FROM {artefact} a JOIN {artefact_tag} at ON (a.id = at.artefact)
+            WHERE a.owner = ?" . $artefacttypefilter . ")
+           UNION
+           (SELECT v.id, v.title, v.description, 'view' AS type, NULL AS artefacttype, " . db_format_tsfield('v.ctime', 'ctime') . "
+            FROM {view} v JOIN {view_tag} vt ON (v.id = vt.view)
+            WHERE v.owner = ? " . $viewfilter . ")
+        ) p";
+
+        $result = (object) array(
+            'tag'    => $tag,
+            'owner'  => $owner,
+            'offset' => $offset,
+            'limit'  => $limit,
+            'sort'   => $sort,
+            'count'  => 0,
+            'data'   => array(),
+        );
+
+        if ($count = count_records_sql('SELECT COUNT(*) ' . $from, $values, $offset, $limit)) {
+            $result->count = $count;
+            $sort = $sort == 'date' ? 'ctime DESC' : 'title ASC';
+            if ($data = get_records_sql_assoc("SELECT type || ':' || id AS tid, p.* " . $from . ' ORDER BY ' . $sort, $values, $offset, $limit)) {
+                if ($returntags) {
+                    $ids = array('view' => array(), 'artefact' => array());
+                    foreach ($data as &$d) {
+                        $ids[$d->type][$d->id] = 1;
+                    }
+                    if (!empty($ids['view'])) {
+                        if ($viewtags = get_records_select_array('view_tag', 'view IN (' . join(',', array_keys($ids['view'])) . ')')) {
+                            foreach ($viewtags as &$vt) {
+                                $data['view:' . $vt->view]->tags[] = $vt->tag;
+                            }
+                        }
+                    }
+                    if (!empty($ids['artefact'])) {
+                        if ($artefacttags = get_records_select_array('artefact_tag', 'artefact IN (' . join(',', array_keys($ids['artefact'])) . ')')) {
+                            foreach ($artefacttags as &$at) {
+                                $data['artefact:' . $at->artefact]->tags[] = $at->tag;
+                            }
+                        }
+                    }
+                }
+                $result->data = $data;
+            }
+        }
+        return $result;
+    }
+
+
     /**
      * Parses a query string into SQL fragments for searching. Supports 
      * phrases, AND/OR etc.
