@@ -1,7 +1,8 @@
 <?php
 /**
  * Mahara: Electronic portfolio, weblog, resume builder and social networking
- * Copyright (C) 2006-2008 Catalyst IT Ltd (http://www.catalyst.net.nz)
+ * Copyright (C) 2006-2009 Catalyst IT Ltd and others; see:
+ *                         http://wiki.mahara.org/Contributors
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,12 +21,13 @@
  * @subpackage core
  * @author     Catalyst IT Ltd
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL
- * @copyright  (C) 2006-2008 Catalyst IT Ltd http://catalyst.net.nz
+ * @copyright  (C) 2006-2009 Catalyst IT Ltd http://catalyst.net.nz
  *
  */
 
 define('INTERNAL', 1);
 define('PUBLIC', 1);
+define('NOCHECKPASSWORDCHANGE', 1);
 require('init.php');
 require_once('file.php');
 
@@ -34,17 +36,31 @@ $type = param_alpha('type');
 switch ($type) {
     case 'profileiconbyid':
     case 'profileicon':
-        $id = param_integer('id');
+        $id = param_integer('id', 0);
         $size = get_imagesize_parameters();
 
-        if ($type == 'profileicon') {
-            // Convert ID of user to the ID of a profileicon
-            $id = get_field('usr', 'profileicon', 'id', $id);
+        if ($id) {
+            if ($type == 'profileicon') {
+                // Convert ID of user to the ID of a profileicon
+                $data = get_record_sql('
+                    SELECT u.profileicon, f.filetype
+                    FROM {usr} u INNER JOIN {artefact_file_files} f ON u.profileicon = f.artefact
+                    WHERE u.id = ?', array($id));
+                if ($data) {
+                    $id = $data->profileicon;
+                    $mimetype = $data->filetype;
+                }
+                else {
+                    $id = null;
+                }
+            }
+            else {
+                $mimetype = get_field('artefact_file_files', 'filetype', 'artefact', $id);
+            }
         }
 
         if ($id) {
-            if ($path = get_dataroot_image_path('artefact/internal/profileicons', $id, $size)) {
-                $mimetype = get_mime_type($path);
+            if ($path = get_dataroot_image_path('artefact/file/profileicons', get_field('artefact_file_files', 'fileid', 'artefact', $id), $size)) {
                 if ($mimetype) {
                     header('Content-type: ' . $mimetype);
 
@@ -81,7 +97,7 @@ switch ($type) {
         // down. This means that $USER->get('theme') should be used here 
         // instead, when that branch is merged. And don't forget to change it 
         // below at the other get_config('theme') call!
-        if ($path = get_dataroot_image_path('artefact/internal/profileicons/no_userphoto/' . get_config('theme'), 0, $size)) {
+        if ($path = get_dataroot_image_path('artefact/file/profileicons/no_userphoto/' . get_config('theme'), 0, $size)) {
             header('Content-type: ' . 'image/png');
             readfile($path);
             exit;
@@ -89,14 +105,14 @@ switch ($type) {
 
         // If we couldn't find the no user photo picture, we put it into 
         // dataroot if we can
-        $nouserphotopic = theme_get_path('images/no_userphoto.png');
+        $nouserphotopic = $THEME->get_path('images/no_userphoto.png');
         if ($nouserphotopic) {
             // Move the file into the correct place.
-            $directory = get_config('dataroot') . 'artefact/internal/profileicons/no_userphoto/' . get_config('theme') . '/originals/0/';
+            $directory = get_config('dataroot') . 'artefact/file/profileicons/no_userphoto/' . get_config('theme') . '/originals/0/';
             check_dir_exists($directory);
             copy($nouserphotopic, $directory . '0');
             // Now we can try and get the image in the correct size
-            if ($path = get_dataroot_image_path('artefact/internal/profileicons/no_userphoto/' . get_config('theme'), 0, $size)) {
+            if ($path = get_dataroot_image_path('artefact/file/profileicons/no_userphoto/' . get_config('theme'), 0, $size)) {
                 header('Content-type: ' . 'image/png');
                 readfile($path);
                 exit;
@@ -106,7 +122,7 @@ switch ($type) {
 
         // Emergency fallback
         header('Content-type: ' . 'image/png');
-        readfile(theme_get_path('images/no_userphoto.png'));
+        readfile($THEME->get_path('images/no_userphoto.png'));
         exit;
         break;
 
@@ -128,18 +144,18 @@ switch ($type) {
             readfile($path);
             exit;
         }
-        readfile(theme_get_path('images/no_thumbnail.png'));
+        readfile($THEME->get_path('images/no_thumbnail.png'));
         break;
     case 'viewlayout':
         header('Content-type: image/png');
         $vl = param_integer('vl');
         if ($widths = get_field('view_layout', 'widths', 'id', $vl)) {
-            if ($path = theme_get_path('images/vl-' . str_replace(',', '-', $widths) . '.png')) {
+            if ($path = $THEME->get_path('images/vl-' . str_replace(',', '-', $widths) . '.png')) {
                 readfile($path);
                 exit;
             }
         }
-        readfile(theme_get_path('images/no_thumbnail.png'));
+        readfile($THEME->get_path('images/no_thumbnail.png'));
         break;
 }
 
